/*****************************************************************************

Copyright 2008 - 2013 by Teradata Corporation.
All Rights Reserved.

Module:      error.cpp

Author:
             Teradata Corporation
             Teradata Warehouse Engineering
             ODBC

Created:

Description: This source file is responsible for retrieving ODBC Error information.
             It calls the SQLError() API which is a 2x API for Error retrieval.

Functions:

Revision History:

   Revision     Date       DR/RFC  Userid   Comments
   -----------  --------   ------- -------- -----------------------------------
   13.00.00.00  01/23/08           jk121225 Updates with copyright and header
   13.00.00.03  11/19/09   137636  sm185109 Use <iostream> instead of <iostream.h> on LINUX
   13.10.00.01  01/29/10   138569  kk250025 changes to display_size() to avoid errors for INTERVAL/PERIOD datatypes.
   14.10.00.00  10/22/12           jb518661 Adding display of more types for convenience.
   14.10.00.xx  06/11/13 ODBC-11303 st255032 Fix compiling issues for Mac OS X and warnings.
*****************************************************************************/

#if defined(INTERSOLV)
    #if defined(NCR_MPRAS) || defined(HP_UX) || defined(AIXVER42) || defined(LINUX) || defined(SOLARIS)
      #include "sqlunx.h"
    #else
      #include "sqlunix.h"
    #endif
#endif

#ifdef WIN32
    #include <windows.h>
#endif

#include <sql.h>
#include <sqlext.h>
#include <tdsql.h>
#include <stdio.h>

#if defined(LINUX) || defined(MACOSX)
    #include <iostream>
    using namespace std;
#else
    #include <iostream.h>
#endif

#include <string.h>
#include <stdlib.h>

#if !defined(WIN32) && !defined(SOLARIS) && !defined(LINUX) && !defined(MACOSX)    /*DR100699 */
    #include <stream.h>
#endif

#define MSG_LNG    256            /* Maximum message length          */
#ifndef WIN32
#define max(a,b) (a>b?a:b)
#endif

int print_err(HENV henv, HDBC hdbc, HSTMT hstmt)
{
RETCODE		rc;			/* general return code for API		*/
UCHAR		szSqlState[MSG_LNG];	/* SQL state string			*/
SQLINTEGER	pfNativeError;		/* Native error code			*/
UCHAR		szErrorMsg[MSG_LNG];	/* Error msg text buffer pointer	*/
SWORD		pcbErrorMsg;		/* Error msg text Available bytes	*/

rc = SQLError(henv, hdbc, hstmt, szSqlState, &pfNativeError, szErrorMsg,
		MSG_LNG, &pcbErrorMsg);
if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO)
	{
	switch (rc)
		{
		case SQL_NO_DATA_FOUND:
			printf("SQLERROR() couldn't find text, RC=%d\n", rc);
			break;
		case SQL_ERROR:
			printf("SQLERROR() couldn't access text, RC=%d\n", rc);
			break;
		case SQL_INVALID_HANDLE:
			printf("SQLERROR() had invalid handle, RC=%d\n", rc);
			break;
		default:
			printf("SQLERROR() unknown return code, RC=%d\n", rc);
			break;
		}
		printf( "Error: %s", szErrorMsg );	/* display error message for user  */
	}
else
	printf("{error} STATE=%s, CODE=%ld, MSG=%s\n", szSqlState,
			(long int)pfNativeError, szErrorMsg);

return (1);                    /* TRUE is secondary return code   */
}                                /* end print_err function          */

/*********************************************************************
 *  The following function is included for completeness, but is not  *
 *  relevant for understanding the function of ODBC.                 *
 *********************************************************************/

#define MAX_NUM_PRECISION 15

/*********************************************************************
 *  Define max length of char string representation of number as:    *
 *    =  max(precision) + leading sign + E + exp sign + max exp leng *
 *    =  15             + 1            + 1 + 1        + 2            *
 *    =  15 + 5                                                      *
 *********************************************************************/

#define MAX_NUM_STRING_SIZE (MAX_NUM_PRECISION + 5)

UDWORD display_size (
               SWORD coltype,
               SQLULEN collen,
               UCHAR *colname)
{
    /*
    ** Refer to the ODBC Programmers Reference.
    ** There is a Display Size section which has the number of characters required for each type
    **
    ** NOTE:
    **      The literals defined below are used to determine the maximum length of the SQL Data Type
    **      and as a learning tool to show what format these different SQL Data Types are.
    */
    int  SQLDataType_BufferSize;
    const char DECIMAL_OR_NUMERIC_LITERAL[] = "-1234567890.12345";
    const char BIT_LITERAL[] = "1";
    const char TINYINT_LITERAL[] = "-123";
    const char SMALLINT_LITERAL[] = "-123456";
    const char INTEGER_LITERAL[] = "-12345678901";
    const char BIGINT_LITERAL[] = "-12345678901234567890";
    const char REAL_LITERAL[] = "-1234567.E+12";
    const char FLOAT_OR_DOUBLE_LITERAL[] = "-1.2345678901234567890123456789012345678E+123";  /* Also covers the case where the type is NUMBER */
    const char DATE_LITERAL[] = "yyyy-mm-dd";
    const char TIME_LITERAL[] = "hh:mm:ss.ffffff+hh:mi";  /* fractional portion would be 9 + fractional characters */
    const char TIMESTAMP_LITERAL[] = "yyyy-mm-dd hh:mm:ss.ffffff+hh:mi"; /* fractional portion would be 20 + fractional characters */
    const char PERIOD_DATE_LITERAL[] = "('2005-02-03', '2006-02-04')";
    const char PERIOD_TIME_LITERAL[] = "('12:12:12.340000+05:30', '13:12:12.340000+05:30')";
    const char PERIOD_TIMESTAMP_WITH_TIME_ZONE[] = "('2005-02-03 12:12:12.000045+08:00', '2005-02-03 12:12:12.000048+08:00')";
    int  INTERVAL_DISPLAY_SIZE = 55; /* would like to define literals as a learning tool */
    const int LARGE_TYPE_MAX_DISPLAY_SIZE = 1000;


    /*
    ** Assume native size
    */
    SQLDataType_BufferSize = collen;

    switch (coltype)
    {
    case SQL_CHAR:
    case SQL_VARCHAR:
        break;
    case SQL_WCHAR:
    case SQL_WVARCHAR:
        break;
    case SQL_LONGVARBINARY:
    case SQL_LONGVARCHAR:
    case SQL_WLONGVARCHAR:
            SQLDataType_BufferSize = LARGE_TYPE_MAX_DISPLAY_SIZE;
        break;
    case SQL_DECIMAL:
    case SQL_NUMERIC:
            SQLDataType_BufferSize = strlen(DECIMAL_OR_NUMERIC_LITERAL);
        break;
    case SQL_BIT:
            SQLDataType_BufferSize = strlen(BIT_LITERAL);
        break;
    case SQL_TINYINT:
            SQLDataType_BufferSize = strlen(TINYINT_LITERAL);
        break;
    case SQL_SMALLINT:
            SQLDataType_BufferSize = strlen(SMALLINT_LITERAL);
        break;
    case SQL_INTEGER:
            SQLDataType_BufferSize = strlen(INTEGER_LITERAL);
        break;
    case SQL_BIGINT:
            SQLDataType_BufferSize = strlen(BIGINT_LITERAL);
        break;
    case SQL_REAL:
            SQLDataType_BufferSize = strlen(REAL_LITERAL);
        break;
    case SQL_FLOAT:
    case SQL_DOUBLE:
            SQLDataType_BufferSize = strlen(FLOAT_OR_DOUBLE_LITERAL);
        break;
    case SQL_BINARY:
    case SQL_VARBINARY:
        return(max((int) 2 * collen, (int) strlen((char *) colname)));

    case SQL_DATE:               /* 2x spec */
    case SQL_TYPE_DATE:          /* 3x spec */
            SQLDataType_BufferSize = strlen(DATE_LITERAL);;
        break;
    case SQL_TIME:              /* 2x spec */
    case SQL_TYPE_TIME:         /* 3x spec */
            SQLDataType_BufferSize = strlen(TIME_LITERAL);
        break;
    case SQL_TYPE_TIMESTAMP:
            SQLDataType_BufferSize = strlen(TIMESTAMP_LITERAL);
        break;
    case SQL_INTERVAL_YEAR:
    case SQL_INTERVAL_MONTH:
    case SQL_INTERVAL_DAY:
    case SQL_INTERVAL_HOUR:
    case SQL_INTERVAL_MINUTE:
    case SQL_INTERVAL_SECOND:
    case SQL_INTERVAL_YEAR_TO_MONTH:
    case SQL_INTERVAL_DAY_TO_HOUR:
    case SQL_INTERVAL_DAY_TO_MINUTE:
    case SQL_INTERVAL_DAY_TO_SECOND:
    case SQL_INTERVAL_HOUR_TO_MINUTE:
    case SQL_INTERVAL_HOUR_TO_SECOND:
    case SQL_INTERVAL_MINUTE_TO_SECOND:
            SQLDataType_BufferSize = INTERVAL_DISPLAY_SIZE;
        break;
    case SQL_PERIOD_DATE:
            SQLDataType_BufferSize = strlen(PERIOD_DATE_LITERAL);
        break;
    case SQL_PERIOD_TIME:
    case SQL_PERIOD_TIME_WITH_TIME_ZONE:
            SQLDataType_BufferSize = strlen(PERIOD_TIME_LITERAL);
        break;
    case SQL_PERIOD_TIMESTAMP:
    case SQL_PERIOD_TIMESTAMP_WITH_TIME_ZONE:
            SQLDataType_BufferSize = strlen(PERIOD_TIMESTAMP_WITH_TIME_ZONE);
        break;
    case SQL_TD_XML:
            SQLDataType_BufferSize = LARGE_TYPE_MAX_DISPLAY_SIZE;
        break;
    default:
        printf("Unknown datatype, %d\n", coltype);
        return (0);

    } /* SWITCH:(coltype) */

    return( max( SQLDataType_BufferSize, (int) strlen((char *) colname) ) + 1 );

} /* display_size() */


/*********************************************************************
 *  Use K&R getline function to get an input line from stdin.        *
 *********************************************************************/

int getline (char *s, const int lim)
{
int c, i;

for (i=0; i < lim-1 && (c=getchar()) != EOF && c!='\n'; ++i)
	s[i] = c;
s[i] = '\0';
return i;
}


/**
 * nullCout() - This funcion prints out "NULL" when a
 *				NULL string pointer or an empty string
 *				is passed in.
**/
void nullCout( char *str)
{
	if (!str || !str[0])
	{
		cout << "<Null> ";
		return;
	}
	else
		cout << str << " ";
}
