/*****************************************************************************

Copyright 2009 - 2013 by Teradata Corporation.
All Rights Reserved.

Module:      error.c

Author:
             Teradata Corporation
             Teradata Warehouse Engineering
             ODBC

Created:

Description: This source file is responsible for retrieving ODBC Error information.
             It currently uses SQLError() which is part of ODBC API version 2.x

Functions:
    print_err()
    kr_getline()


Revision History:

   Revision    Date       DR/RFC  Userid   Comments
   ----------- --------   ------- -------- -----------------------------------
   03.00.00.00 06/11/01     52182 NDFSODBC Fixes for UNIX-Show-Table to output
                                            in the correct format
   13.00.00.00 01/23/08           jk121225 Updates with copyright and header
   13.00.00.00 07/27/09    134217 ff185017 Updates for Windows compatibility
   13.10f.00.00 11/11/09    86872 jk121225 moved the display_size to adhoc.c, a more logic place.
   13.10.00.06  05/04/11   150430 jk121225 renamed getline function to kr_getline due to conflict.
                                           Should use getline version from O/S library. RFC candidate 
  14e.00.00.00  06/17/11   151802 jk121225 Don't display password to console when entered by user.
   14.10.00.xx  06/11/13 ODBC-11303 st255032 Fix compilation issues on Mac OS X and warnings.
 *****************************************************************************/



/*---------------------------------------------------------------------*/
/*--- SECTION: Required Headers ---------------------------------------*/

#if defined(WIN32) || defined(_WIN64)
/*
 * windows.h has to be included before using any of 
 * the ODBC related header files in WINDOWS
 */
#    include <windows.h>
#else
/*
 * stdlib.h has to be included before using any of 
 * the ODBC related header files in UNIX
 */
#    include <stdlib.h>
#endif

/*
 * sqlext.h defines SQL extensions for ODBC.  It in turn includes
 * sql.h, which includes sqltypes.h.    Hence, only the inclusion
 * of sqlext.h is necessary for compiling ODBC based programs.
 */
#include <sqlext.h>

#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#include <tdsql.h>  /* TTU 13.0*/


/*--- END: Required Headers -------------------------------------------*/
/*---------------------------------------------------------------------*/



/* ============================== */
/*  - Defined Limits and Sizes -  */
/* ============================== */

#define MSG_LNG 1024                /* Maximum message's length         */



/* ============================================================== */
/*  - Different NULL-casts to avoid warnings of some compilers -  */
/* ============================================================== */
#define NULL_pUCHAR     (UCHAR *) 0
#define NULL_pchar      (char *) 0



/* ====================================================== */
/*  - Adhoc MACRO for allocation/reallocation attempts -  */
/* ====================================================== */

/***************************************************************************\
|
|   Macro:      ALLOC_BUFFER(buffname, buffsize, bufftype, returnerror)
|   Purpose:    Attempt to realloc(buffname, ((buffsize)+1)*sizeof(bufftype))
|               It realloc() fails, prints an error message including
|               the line where the error occurred and returns 'returnerror'
|
|   USE: [returning something]
|           int foo()
|           {
|               char * buffer;
|               size_t len = 124;
|               ALLOC_BUFFER(buffer, len, char, -1);
|               free(buffer);
|               return 0;
|           }
|        [not returning anything]
|           void foo()
|           {
|               char * buffer;
|               size_t len = 124;
|               ALLOC_BUFFER(buffer, len, char, );
|               free(buffer);
|           }
|
\***************************************************************************/
#define ALLOC_BUFFER(buffname, buffsize, bufftype, returnerror) \
    if (NULL == (buffname = (bufftype *) realloc(buffname, ((buffsize) + 1) * sizeof(bufftype)))) \
    { \
        printf("\nadhoc: %s=realloc(%d) failed in line %d\n", #buffname, (buffsize), __LINE__); \
        return returnerror; \
    } else memset(buffname, 0, ((buffsize) + 1) * sizeof(bufftype))


/*--- END: Defines, consts, and static declarations -------------------*/
/*---------------------------------------------------------------------*/





/*---------------------------------------------------------------------*/
/*--- SECTION: Implementation of this module's functions --------------*/

/***************************************************************************\
|
|   Method:     print_err()
|   Purpose:    Retreive and display ODBC status/error information
|
|   ODBC version: 2.x (using GetError() once)
|       NOTE: In ODBC 2.x a SINGLE CALL to SQLGetError() can be made,
|           If the returned message is truncated the lost data is lost
|           forever.
|
|   @param [in] argc  first argument from main()
|
|   @param [in] argv  second argument from main()
|
|   @return  the number of times 'argument' was found in 'argv' array
|
\***************************************************************************/
int print_err(HENV henv, HDBC hdbc, HSTMT hstmt)
{
    /*
    **  declaration of print_err()'s variables
    */
    RETCODE rc;                     /* general ODBC API return codes    */
    UCHAR * szSqlState;             /* SQL state string                 */
    SQLINTEGER pfNativeError;       /* Native error code                */
    UCHAR * szErrorMsg;             /* (Error msg text) Buffer          */
    SWORD pbcErrorMsg;              /* Length of the buffer szErrorMsg  */


    /*
    **  initialization of print_err()'s variables
    */
    szSqlState = NULL_pUCHAR;
    ALLOC_BUFFER(szSqlState, MSG_LNG, UCHAR, -1);  /* #ERROR# adhoc error codes need to be revised */

    pfNativeError = (SWORD) 0;

    szErrorMsg = NULL_pUCHAR;
    pbcErrorMsg = (SWORD) 0;
    ALLOC_BUFFER(szErrorMsg, MSG_LNG, UCHAR, -1);  /* #ERROR# adhoc error codes need to be revised */


    /*
    **  Retreive and display ODBC 2.x status/error information
    */
    rc = SQLError(henv, hdbc, hstmt,
        szSqlState, &pfNativeError,
        szErrorMsg, MSG_LNG, &pbcErrorMsg);

    if ((SQL_SUCCESS != rc) && (SQL_SUCCESS_WITH_INFO != rc))
    { /* IF-ELSE: Check SQLError() return code */
        switch (rc)
        {
        case SQL_NO_DATA_FOUND:
            printf("\nadhoc: SQLError() couldn't find text, RC=%d\n", rc);
            break;
        case SQL_ERROR:
            printf("\nadhoc: SQLError() couldn't access text, RC=%d\n", rc);
            break;
        case SQL_INVALID_HANDLE:
            printf("\nadhoc: SQLError() had invalid handle, RC=%d\n", rc);
            break;
        default:
            printf("\nadhoc: SQLError() unknown return code, RC=%d\n", rc);
            break;
        }

    }
    else
    { /* IF-ELSE: Check SQLError() return code */
        printf("\nadhoc: (SQL Diagnostics) STATE=%s, CODE=%ld, MSG=%s\n",
            szSqlState, (long int)pfNativeError, szErrorMsg);

    } /* IF-ELSE: Check SQLError() return code */


    /*
    **  Clean up and return
    */
    if (NULL_pUCHAR != szSqlState)
    {
        free(szSqlState);
        szSqlState = NULL_pUCHAR;
    }

    if (NULL_pUCHAR != szErrorMsg)
    {
        free(szErrorMsg);
        szErrorMsg = NULL_pUCHAR;
    }

    return (rc);                    /* return SQLError()'s return code  */

} /* print_err() */





/***************************************************************************\
|
|   Method:     kr_getline()
|   Purpose:    Obtain null-terminated string of characters from stdin until
|               a '\n' character is detected (this last character will not
|               be part of the string)
|
|       NOTE:   Using K&R getline function to get an input line from stdin
|
|   @param [in/out] s    pointer to a buffer of characters
|
|   @param [in]     lim  size of the buffer 's'
|
|   @return  the length of the string stored in the buffer s (excluding NULL)
|           or NEGATIVE ONE (-1) if EOF contition was reached before reading
|               any character into the buffer.
|
|   Remarks:
|   Requirements:
|   See Also:
|  
|   Fixes Documented :
|     Date         DR/RFC  Owner      Description
|     -----------------------------------------------------------
|     06/22/2011   151802  jk121225   reformatted for loop while debugging ECHO logic
|
\***************************************************************************/

int kr_getline (char s[], int lim)
{
    int c;                          /* value returned by getchar()      */
    int i;                          /* index used in for-loop. Serves as
                                        the position in buffer s where to
                                        store the next character        */

    for ( i = 0; 
          ( (i < (lim - 1)) && 
            ((c = getchar()) != EOF) && 
            (c != '\n')
          ); 
          ++i )
    {
        s[i] = c;
    }
    s[i] = '\0';    /* NULL-terminate the string stored in s */

    if ((EOF == c) && (0 == i))
        return -1;  /* EOF before any character could be read */

    return i;       /* return the length of the string stored in s
                        not including the NULL terminator */

} /* kr_getline() */


/*--- END: Implementation of this module's functions ------------------*/
/*---------------------------------------------------------------------*/





/*===========================================================================*
    End of File:    error.c
 *===========================================================================*/
