/*************************************************************************
*
* Title: TEL API DynamicLibrary Class Definition
*
*
* Copyright 2010, 2012-2014 by Teradata Corporation.
* All Rights Reserved.
* TERADATA CONFIDENTIAL AND TRADE SECRET
*
* This copyrighted material is the Confidential, Unpublished
* Property of the Teradata Corporation.  This copyright notice and
* any other copyright notices included in machine readable
* copies must be reproduced on all authorized copies.
*
* Description:
* The class is a singleton class to load library dynamically and creates
* objects out of the loaded library.
*
* Purpose:
* To load library dynamically once and provide a global point of access
* to it.
*
* History Information
*
* Revision    Date     DCR    DID      Comments
* ----------- -------- ------ -------- ---------------------------------------
* 15.00.00.00 24032014 TPT-21257 KB186006  dynlnk_sample compile errors in MVS
* 14.10.00.03 20022013 TPT-18039 kb186006  TPTAPI Dynamic link sample 
*                                          applications are failing on MACOS
* 14.10.00.02 07232012 TPT-14052 by255002  Compilation Error occurs 
*                                          while Compiling the samples
*                                          in dynlnk_sample
* 14.10.00.01 05172012 TPT-11431 vm185012  TPTAPI to add support for
                                           gcc 4.3 Linux compiler
* 14.00.00.01 09032010 144824 mv121803 Support MVS platform
* 13.00.00.01 05211010 142720 mv121803 Initial Creation
*
*************************************************************************/

// To get around the issue of converting parameter from 'const char *' to 
// 'LPCWSTR' in function LoadLibraryW when the application is built with
// the UNICODE option in the 64-bit Windows environment. 
#if defined(WIN64)
   #ifdef UNICODE
      #undef UNICODE
   #endif
#endif

#include <stdio.h>

#ifndef __MVS__                                      //DR144824
   #include "dynlib.h"
#else                                                //DR144824
   #include "PTCDLLIB.H"                             //DR144824
#endif                                               //DR144824

#if defined(__MVS__)
   #define TPTAPI_LIBRARY_NAME "TPTAPI"
   
#elif defined(WIN32)
   #define TPTAPI_LIBRARY_NAME "telapi.dll"

#elif defined(HPUX) && !defined(HPUX_IA64)
   #define TPTAPI_LIBRARY_NAME "libtelapi.sl"
   
#elif defined(__APPLE__)                            //TPT-18039
   #define TPTAPI_LIBRARY_NAME "libtelapi.dylib"	//TPT-18039

#else
   #define TPTAPI_LIBRARY_NAME "libtelapi.so"

#endif

using namespace teradata::client::API;

#ifdef WIN32
      HINSTANCE  DynamicLibrary::m_libhdl = 0;
#else
      void*      DynamicLibrary::m_libhdl = 0;
#endif

DynamicLibrary* DynamicLibrary::m_instance = 0;
TD_Boolean      DynamicLibrary::m_isloaded = FALSE;


DynamicLibrary::DynamicLibrary () {}

DynamicLibrary::~DynamicLibrary () {}

DynamicLibrary*  DynamicLibrary::Instance () {

    if ( DynamicLibrary::m_instance == 0 )
       DynamicLibrary::m_instance = new DynamicLibrary;

    DynamicLibrary::m_instance->LoadTptApiLibrary(TPTAPI_LIBRARY_NAME);

    return DynamicLibrary::m_instance;
}

void DynamicLibrary::Delete () {   

    if ( DynamicLibrary::m_libhdl ) {

       cout << "   UnLoad TPTAPI library \n";

       #if defined(WIN32)
        FreeLibrary(m_libhdl);

       #elif defined(__MVS__)
        dllfree((dllhandle *)m_libhdl);

       #else
        dlclose(m_libhdl);
       #endif

       DynamicLibrary::m_libhdl = NULL;
    }

    DynamicLibrary::m_isloaded = FALSE;

    if ( DynamicLibrary::m_instance ) {
       cout << "   Delete the DynamicLibrary object \n";
       delete DynamicLibrary::m_instance; 
       DynamicLibrary::m_instance = 0;
    }

}

void DynamicLibrary::LoadTptApiLibrary (const char *LibraryName) {

#if defined(__MVS__) 
    int  save_errno;
#endif

    if ( !m_isloaded) {

cout << "   Load TPTAPI library \n";

#if defined(__MVS__)
       m_libhdl = dllload(LibraryName);
       if ( !m_libhdl ) {
          save_errno = errno;
          printf ("Couldn't open %s: %s\n", LibraryName, strerror(save_errno));
          exit(-1);
       }
 
       pfn_newCon = (ConnectionX * (*)())dllqueryfn((dllhandle *)m_libhdl,
                                                   "newConnectionX");
       if ( pfn_newCon == NULL ) {
          save_errno = errno;
          cout << "Cannot load the function 'newConnectionX'.\n";
          cout << strerror(save_errno) << endl;
          exit (-1);   
       }

       pfn_newConE = (ConnectionX * (*)(TD_Encoding))dllqueryfn(
                                                      (dllhandle *)m_libhdl,
                                                      "newConnectionXEncoding");
       if ( pfn_newConE == NULL ) {
          save_errno = errno;
          cout << "Cannot load the function 'newConnectionXEncoding'.\n";
          cout << strerror(save_errno)  << endl;
          exit (-1);   
       }

       pfn_copyCon = 
            (ConnectionX * (*)(ConnectionX*, ConnectionX*))dllqueryfn(
                                     (dllhandle *)m_libhdl, "copyConnectionX");
       if ( pfn_copyCon == NULL ) {
          save_errno = errno;
          cout << "Cannot load the function 'copyConnectionX'.\n";
          cout << strerror(save_errno) << endl;
          exit (-1);   
       }

       pfn_delCon = (void (*)(ConnectionX *))dllqueryfn(
                                                      (dllhandle *)m_libhdl, 
                                                      "delConnectionX");
       if ( pfn_delCon == NULL ) {
          save_errno = errno;
          cout << "Cannot load the function 'delConnectionX'.\n";
          cout << strerror(save_errno)  << endl;
          exit (-1);   
       }
 
       pfn_newSchm = (SchemaX * (*)(const char*))dllqueryfn(
                                                     (dllhandle *)m_libhdl,
                                                     "newSchemaX"); //TPT-21257
       if ( pfn_newSchm == NULL ) {
          save_errno = errno;
          cout << "Cannot load the function 'newSchemaX'.\n";
          cout << strerror(save_errno) << endl;
          exit (-1);   
       }

       pfn_newSchmE = (SchemaX * (*)(TD_Encoding))dllqueryfn(
                                                        (dllhandle *)m_libhdl, 
                                                        "newSchemaXEncoding");
       if ( pfn_newSchmE == NULL ) {
          save_errno = errno;
          cout << "Cannot load the function 'newSchemaXEncoding'.\n";
          cout << strerror(save_errno) << endl;
          exit (-1);   
       }

       pfn_copySchm = (SchemaX * (*)(SchemaX*, SchemaX*))dllqueryfn(
                                                      (dllhandle *)m_libhdl,
                                                      "copySchemaX");
       if ( pfn_copySchm == NULL ) {
          save_errno = errno;
          cout << "Cannot load the function 'copySchemaX'.\n";
          cout << strerror(save_errno) << endl;
          exit (-1);   
       }

       pfn_delSchm = (void (*)(SchemaX *))dllqueryfn((dllhandle *)m_libhdl, 
                                                     "delSchemaX");
       if ( pfn_delSchm == NULL ) {
          save_errno = errno;
          cout << "Cannot load the function 'delSchemaX'.\n";
          cout << strerror(save_errno) << endl;
          exit (-1);   
       }

       pfn_newDMLGrp = (DMLGroupX * (*)())dllqueryfn((dllhandle *)m_libhdl, 
                                                     "newDMLGroupX");
       if ( pfn_newDMLGrp == NULL ) {
          save_errno = errno;
          cout << "Cannot load the function 'newDMLGroupX'.\n";
          cout << strerror(save_errno) << endl;
          exit (-1);   
       }

       pfn_newDMLGrpE = (DMLGroupX * (*)(TD_Encoding))dllqueryfn(
                                                        (dllhandle *)m_libhdl, 
                                                        "newDMLGroupXEncoding");
       if ( pfn_newDMLGrpE == NULL ) {
          save_errno = errno;
          cout << "Cannot load the function 'newDMLGroupXEncoding'.\n";
          cout << strerror(save_errno) << endl;
          exit (-1);   
       }

       pfn_copyDMLGrp = (DMLGroupX * (*)(DMLGroupX*, DMLGroupX*))dllqueryfn(
                                                     (dllhandle *)m_libhdl,
                                                     "copyDMLGroupX");
       if ( pfn_copyDMLGrp == NULL ) {
          save_errno = errno;
          cout << "Cannot load the function 'copyDMLGroupX'.\n";
          cout << strerror(save_errno) << endl;
          exit (-1);   
       }

       pfn_delDMLGrp = (void (*)(DMLGroupX *))dllqueryfn((dllhandle *)m_libhdl,
                                                         "delDMLGroupX");
       if ( pfn_delDMLGrp == NULL ) {
          save_errno = errno;
          cout << "Cannot load the function 'delDMLGroupX'.\n";
          cout << strerror(save_errno) << endl;
          exit (-1);   
       }

#elif defined(WIN32)
       m_libhdl = LoadLibrary(LibraryName); 
       if ( m_libhdl == NULL ) {
          LPVOID lpMsgBuf;
          FormatMessage ( 
                 FORMAT_MESSAGE_ALLOCATE_BUFFER |
                 FORMAT_MESSAGE_FROM_SYSTEM |
                 FORMAT_MESSAGE_IGNORE_INSERTS,
                 NULL,
                 GetLastError(),
                 MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
                 (LPTSTR) &lpMsgBuf,
                 0,
                 NULL
                );
         printf ("Couldn't open %s: %s\n", LibraryName, (LPCTSTR)lpMsgBuf);
         exit (-1);   
       } 
 
       pfn_newCon = (ConnectionX * (*)())GetProcAddress(m_libhdl,
                                                        "newConnectionX");
       if ( pfn_newCon == NULL ) {
          LPVOID lpMsgBuf;
          FormatMessage (
                 FORMAT_MESSAGE_ALLOCATE_BUFFER |
                 FORMAT_MESSAGE_FROM_SYSTEM |
                 FORMAT_MESSAGE_IGNORE_INSERTS,
                 NULL,
                 GetLastError(),
                 MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
                 (LPTSTR) &lpMsgBuf,
                 0,
                 NULL
                );
         printf ("Cannot load the function 'newConnectionX: %s\n",
                 (LPCTSTR)lpMsgBuf);
         exit (-1);   
       }

       pfn_newConE = (ConnectionX * (*)(TD_Encoding))GetProcAddress(m_libhdl,
                                                      "newConnectionXEncoding");
       if ( pfn_newConE == NULL ) {
          LPVOID lpMsgBuf;
          FormatMessage (
                 FORMAT_MESSAGE_ALLOCATE_BUFFER |
                 FORMAT_MESSAGE_FROM_SYSTEM |
                 FORMAT_MESSAGE_IGNORE_INSERTS,
                 NULL,
                 GetLastError(),
                 MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
                 (LPTSTR) &lpMsgBuf,
                 0,
                 NULL
                );
         printf ("Cannot load the function 'newConnectionXEncoding: %s\n",
                 (LPCTSTR)lpMsgBuf);
         exit (-1);   
       }

       pfn_copyCon = 
            (ConnectionX * (*)(ConnectionX*, ConnectionX*))GetProcAddress(
                                                 m_libhdl, "copyConnectionX");
       if ( pfn_copyCon == NULL ) {
          LPVOID lpMsgBuf;
          FormatMessage (
                 FORMAT_MESSAGE_ALLOCATE_BUFFER |
                 FORMAT_MESSAGE_FROM_SYSTEM |
                 FORMAT_MESSAGE_IGNORE_INSERTS,
                 NULL,
                 GetLastError(),
                 MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
                 (LPTSTR) &lpMsgBuf,
                 0,
                 NULL
                );
         printf ("Cannot load the function 'copyConnectionX: %s\n",
                 (LPCTSTR)lpMsgBuf);
         exit (-1);   
       }

       pfn_delCon = (void (*)(ConnectionX *))GetProcAddress(m_libhdl,
                                                            "delConnectionX");
       if ( pfn_delCon == NULL ) {
          LPVOID lpMsgBuf;
          FormatMessage (
                 FORMAT_MESSAGE_ALLOCATE_BUFFER |
                 FORMAT_MESSAGE_FROM_SYSTEM |
                 FORMAT_MESSAGE_IGNORE_INSERTS,
                 NULL,
                 GetLastError(),
                 MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
                 (LPTSTR) &lpMsgBuf,
                 0,
                 NULL
                );
         printf ("Cannot load the function 'delConnectionX': %s\n",
                 (LPCTSTR)lpMsgBuf);
         exit (-1);   
       }
    
       pfn_newSchm = (SchemaX * (*)(const char*))GetProcAddress(m_libhdl, 
                                                          "newSchemaX");
                                                          //TPT-14052    
       if ( pfn_newSchm == NULL ) {
          LPVOID lpMsgBuf;
          FormatMessage ( 
                 FORMAT_MESSAGE_ALLOCATE_BUFFER |
                 FORMAT_MESSAGE_FROM_SYSTEM |
                 FORMAT_MESSAGE_IGNORE_INSERTS,
                 NULL,
                 GetLastError(),
                 MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
                 (LPTSTR) &lpMsgBuf,
                 0,
                 NULL
                );
         printf ("Cannot load the function 'newSchemaX: %s\n", 
                 (LPCTSTR)lpMsgBuf);
         exit (-1);   
       }

       pfn_newSchmE = (SchemaX * (*)(TD_Encoding))GetProcAddress(m_libhdl,
                                                          "newSchemaXEncoding");
       if ( pfn_newSchmE == NULL ) {
          LPVOID lpMsgBuf;
          FormatMessage (
                 FORMAT_MESSAGE_ALLOCATE_BUFFER |
                 FORMAT_MESSAGE_FROM_SYSTEM |
                 FORMAT_MESSAGE_IGNORE_INSERTS,
                 NULL,
                 GetLastError(),
                 MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
                 (LPTSTR) &lpMsgBuf,
                 0,
                 NULL
                );
         printf ("Cannot load the function 'newSchemaXEncoding: %s\n",
                 (LPCTSTR)lpMsgBuf);
         exit (-1);   
       }

       pfn_copySchm =
              (SchemaX * (*)(SchemaX*, SchemaX*))GetProcAddress(m_libhdl,
                                                               "copySchemaX");
       if ( pfn_copySchm == NULL ) {
          LPVOID lpMsgBuf;
          FormatMessage (
                 FORMAT_MESSAGE_ALLOCATE_BUFFER |
                 FORMAT_MESSAGE_FROM_SYSTEM |
                 FORMAT_MESSAGE_IGNORE_INSERTS,
                 NULL,
                 GetLastError(),
                 MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
                 (LPTSTR) &lpMsgBuf,
                 0,
                 NULL
                );
         printf ("Cannot load the function 'copySchemaX: %s\n",
                 (LPCTSTR)lpMsgBuf);
         exit (-1);   
       }

       pfn_delSchm = (void (*)(SchemaX *))GetProcAddress(m_libhdl, 
                                                         "delSchemaX");
       if ( pfn_delSchm == NULL ) {
          LPVOID lpMsgBuf;
          FormatMessage (
                 FORMAT_MESSAGE_ALLOCATE_BUFFER |
                 FORMAT_MESSAGE_FROM_SYSTEM |
                 FORMAT_MESSAGE_IGNORE_INSERTS,
                 NULL,
                 GetLastError(),
                 MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
                 (LPTSTR) &lpMsgBuf,
                 0,
                 NULL
                );
         printf ("Cannot load the function 'delSchemaX': %s\n", 
                 (LPCTSTR)lpMsgBuf);
         exit (-1);   
       }

       pfn_newDMLGrp = (DMLGroupX * (*)())GetProcAddress(m_libhdl, 
                                                         "newDMLGroupX");
       if ( pfn_newDMLGrp == NULL ) {
          LPVOID lpMsgBuf;
          FormatMessage (
                 FORMAT_MESSAGE_ALLOCATE_BUFFER |
                 FORMAT_MESSAGE_FROM_SYSTEM |
                 FORMAT_MESSAGE_IGNORE_INSERTS,
                 NULL, 
                 GetLastError(),
                 MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
                 (LPTSTR) &lpMsgBuf,
                 0,
                 NULL
                );
         printf ("Cannot load the function 'newDMLGroupX: %s\n",
                 (LPCTSTR)lpMsgBuf);
         exit (-1);   
       }

       pfn_newDMLGrpE = (DMLGroupX * (*)(TD_Encoding))GetProcAddress(m_libhdl,
                                                       "newDMLGroupXEncoding");
       if ( pfn_newDMLGrpE == NULL ) {
          LPVOID lpMsgBuf;
          FormatMessage (
                 FORMAT_MESSAGE_ALLOCATE_BUFFER |
                 FORMAT_MESSAGE_FROM_SYSTEM |
                 FORMAT_MESSAGE_IGNORE_INSERTS,
                 NULL,
                 GetLastError(),
                 MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
                 (LPTSTR) &lpMsgBuf,
                 0,
                 NULL
                );
         printf ("Cannot load the function 'newDMLGroupXEncoding: %s\n",
                 (LPCTSTR)lpMsgBuf);
         exit (-1);   
       }

       pfn_copyDMLGrp = 
              (DMLGroupX * (*)(DMLGroupX*, DMLGroupX*))GetProcAddress(m_libhdl,
                                                               "copyDMLGroupX");
       if ( pfn_copyDMLGrp == NULL ) {
          LPVOID lpMsgBuf;
          FormatMessage (
                 FORMAT_MESSAGE_ALLOCATE_BUFFER |
                 FORMAT_MESSAGE_FROM_SYSTEM |
                 FORMAT_MESSAGE_IGNORE_INSERTS,
                 NULL, 
                 GetLastError(),
                 MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
                 (LPTSTR) &lpMsgBuf,
                 0,
                 NULL
                );
         printf ("Cannot load the function 'copyDMLGroupX: %s\n",
                 (LPCTSTR)lpMsgBuf);
         exit (-1);   
       }

       pfn_delDMLGrp = (void (*)(DMLGroupX *))GetProcAddress(m_libhdl,
                                                          "delDMLGroupX");
       if ( pfn_delDMLGrp == NULL ) {
          LPVOID lpMsgBuf;
          FormatMessage (
                 FORMAT_MESSAGE_ALLOCATE_BUFFER |
                 FORMAT_MESSAGE_FROM_SYSTEM |
                 FORMAT_MESSAGE_IGNORE_INSERTS,
                 NULL,
                 GetLastError(),
                 MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
                 (LPTSTR) &lpMsgBuf,
                 0,
                 NULL
                );
         printf ("Cannot load the function 'delDMLGroupX': %s\n",
                 (LPCTSTR)lpMsgBuf);
         exit (-1);   
       }

#else
       m_libhdl = dlopen(LibraryName, RTLD_LAZY);
       if ( !m_libhdl ) {
          printf ("Couldn't open %s: %s\n", LibraryName, dlerror());
          exit (-1);  
       }

       // Get function symbols
       dlerror();
       pfn_newCon = (ConnectionX * (*)())dlsym(m_libhdl, "newConnectionX");
       if ( pfn_newCon == NULL ) {
          cout << "Cannot load the function 'newConnectionX'.\n";
          cout << dlerror() << endl;
          exit (-1);   
       }

       dlerror();
       pfn_newConE = (ConnectionX * (*)(TD_Encoding))dlsym(m_libhdl, 
                                                      "newConnectionXEncoding");
       if ( pfn_newConE == NULL ) {
          cout << "Cannot load the function 'newConnectionXEncoding'.\n";
          cout << dlerror() << endl;
          exit (-1);   
       }

       dlerror();
       pfn_copyCon = (ConnectionX * (*)(ConnectionX*, ConnectionX*))dlsym(
                                                   m_libhdl, "copyConnectionX");
       if ( pfn_copyCon == NULL ) {
          cout << "Cannot load the function 'copyConnectionX'.\n";
          cout << dlerror() << endl;
          exit (-1);   
       }

       dlerror();
       pfn_delCon = (void (*)(ConnectionX *))dlsym(m_libhdl, "delConnectionX");
       if ( pfn_delCon == NULL ) {
          cout << "Cannot load the function 'delConnectionX'.\n";
          cout << dlerror() << endl;
          exit (-1);   
       }

       dlerror();
       pfn_newSchm = (SchemaX * (*)(const char*))dlsym(m_libhdl, "newSchemaX");
       if ( pfn_newSchm == NULL ) {
          cout << "Cannot load the function 'newSchemaX'.\n";
          cout << dlerror() << endl;
          exit (-1);   
       }

       dlerror();
       pfn_newSchmE = (SchemaX * (*)(TD_Encoding))dlsym(m_libhdl, 
                                                        "newSchemaXEncoding");
       if ( pfn_newSchmE == NULL ) {
          cout << "Cannot load the function 'newSchemaXEncoding'.\n";
          cout << dlerror() << endl;
          exit (-1);   
       }

       dlerror();
       pfn_copySchm = (SchemaX * (*)(SchemaX*, SchemaX*))dlsym(m_libhdl, 
                                                               "copySchemaX");
       if ( pfn_copySchm == NULL ) {
          cout << "Cannot load the function 'copySchemaX'.\n";
          cout << dlerror() << endl;
          exit (-1);   
       }

       dlerror();
       pfn_delSchm = (void (*)(SchemaX *))dlsym(m_libhdl, "delSchemaX");
       if ( pfn_delSchm == NULL ) {
          cout << "Cannot load the function 'delSchemaX'.\n";
          cout << dlerror() << endl;
          exit (-1);   
       }
    
       dlerror(); 
       pfn_newDMLGrp = (DMLGroupX * (*)())dlsym(m_libhdl, "newDMLGroupX");
       if ( pfn_newDMLGrp == NULL ) {
          cout << "Cannot load the function 'newDMLGroupX'.\n";
          cout << dlerror() << endl;
          exit (-1);   
       }
    
       dlerror();
       pfn_newDMLGrpE = (DMLGroupX * (*)(TD_Encoding))dlsym(m_libhdl, 
                                                       "newDMLGroupXEncoding");
       if ( pfn_newDMLGrpE == NULL ) {
          cout << "Cannot load the function 'newDMLGroupXEncoding'.\n";
          cout << dlerror() << endl;
          exit (-1);   
       }

       dlerror();
       pfn_copyDMLGrp = 
              (DMLGroupX * (*)(DMLGroupX*, DMLGroupX*))dlsym(m_libhdl, 
                                                             "copyDMLGroupX");
       if ( pfn_copyDMLGrp == NULL ) {
          cout << "Cannot load the function 'copyDMLGroupX'.\n";
          cout << dlerror() << endl;
          exit (-1);   
       }

       dlerror(); 
       pfn_delDMLGrp = (void (*)(DMLGroupX *))dlsym(m_libhdl, "delDMLGroupX");
       if ( pfn_delDMLGrp == NULL ) {
          cout << "Cannot load the function 'delDMLGroupX'.\n";
          cout << dlerror() << endl;
          exit (-1);   
       }

#endif

        m_isloaded = TRUE;
    }
}

void DynamicLibrary::delConnectionX (ConnectionX* obj) {
     pfn_delCon(obj);
}

void DynamicLibrary::delDMLGroupX (DMLGroupX* obj) {
     pfn_delDMLGrp(obj);
}

void DynamicLibrary::delSchemaX (SchemaX* obj) {
     pfn_delSchm(obj);
}

ConnectionX* DynamicLibrary::newConnectionX () {
     return ((ConnectionX *)pfn_newCon());
}

ConnectionX* DynamicLibrary::newConnectionX (TD_Encoding encoding) {
     return ((ConnectionX *)pfn_newConE(encoding));
}

DMLGroupX* DynamicLibrary::newDMLGroupX () {
     return ((DMLGroupX *)pfn_newDMLGrp());
}

DMLGroupX* DynamicLibrary::newDMLGroupX (TD_Encoding encoding) {
     return ((DMLGroupX *)pfn_newDMLGrpE(encoding));
}

SchemaX* DynamicLibrary::newSchemaX (const char *stype) {
     return ((SchemaX *)pfn_newSchm(stype));
}

SchemaX* DynamicLibrary::newSchemaX (TD_Encoding encoding) {
     return ((SchemaX *)pfn_newSchmE(encoding));
}

