/*************************************************************************
*
* Title: TEL API DynamicLibrary Class Definition
*
*
* Copyright 2010-2012, 2014 by Teradata Corporation.
* All Rights Reserved.
* TERADATA CONFIDENTIAL AND TRADE SECRET
*
* This copyrighted material is the Confidential, Unpublished
* Property of the Teradata Corporation.  This copyright notice and
* any other copyright notices included in machine readable
* copies must be reproduced on all authorized copies.
*
* Description:
* The class is a singleton class to load library dynamically and creates
* objects out of the loaded library. 
*
* Purpose:
* To load library dynamically once and provide a global point of access
* to it.
*
* History Information
*
* Revision    Date     DCR    DID      Comments
* ----------- -------- ------ -------- ---------------------------------------
* 14.10.00.01 05172012 TPT-11431 vm185012  TPTAPI to add support for
                                           gcc 4.3 Linux compiler
* 14.00.00.02 08222011 153267 km255009 Correcting build errors for HPUX 
* 14.00.00.01 10252010 144824 mv121803 Support MVS
* 13.00.00.01 05211010 142720 mv121803 Initial Creation
*
*************************************************************************/
 
#ifndef DYNAMICLIBRARY_H
#define DYNAMICLIBRARY_H

#ifndef __MVS__                                     //DR144824
   #include "connectionX.h"
   #include "schemaX.h"
   #include "DMLGroupX.h"
#else                                              //DR144824
   #include "CONNECTX.H"                           //DR144824
   #include "SCHEMAX.H"                            //DR144824
   #include "DMLGRPX.H"                            //DR144824
#endif                                             //DR144824

#if defined(__MVS__)
   #include <dll.h>
   #include <errno.h>
#elif defined(HPUX)
   #include <dl.h>
   #include <errno.h>
   #include <dlfcn.h>                              //DR153267 
#elif defined(WIN32)  
   #include <windows.h>
#else
   #include <dlfcn.h>
#endif 

#define FALSE 0
#define TRUE  1

namespace ncr {
namespace teradata {
namespace client {
namespace API {

class DynamicLibrary {
   public:
      static DynamicLibrary*  Instance ();
      static void Delete ();  
      
      // Create objects out of the loaded library
      ConnectionX*  newConnectionX ();
      ConnectionX*  newConnectionX (TD_Encoding encoding);
      DMLGroupX*    newDMLGroupX ();
      DMLGroupX*    newDMLGroupX (TD_Encoding encoding);
      SchemaX*      newSchemaX (const char* stype);        // TPT-11431
      SchemaX*      newSchemaX (TD_Encoding encoding);
   
      // Delete objects created out of the loaded library
      void  delConnectionX (ConnectionX *);
      void  delDMLGroupX (DMLGroupX *);
      void  delSchemaX (SchemaX *);

   protected:
      DynamicLibrary();
      virtual ~DynamicLibrary();

      // Since an instance of DynamicLibrary manages lifetime of a loaded
      // library, it's important to ensure that the object isn't copied.
      DynamicLibrary (const DynamicLibrary&) {}
      DynamicLibrary& operator=(const DynamicLibrary&) {return *this;}

      // Load TPTAPI library
      void LoadTptApiLibrary (const char *);
     
   private:
      static DynamicLibrary*  m_instance;
      static TD_Boolean       m_isloaded;

#ifdef WIN32      
      static HINSTANCE  m_libhdl;
#else     
      static void      *m_libhdl;
#endif 

      teradata::client::API::ConnectionX* (*pfn_newCon)(); 
      teradata::client::API::ConnectionX* (*pfn_newConE)(TD_Encoding); 
      teradata::client::API::DMLGroupX*   (*pfn_newDMLGrp)();
      teradata::client::API::DMLGroupX*   (*pfn_newDMLGrpE)(TD_Encoding);
      teradata::client::API::SchemaX*     (*pfn_newSchm)(const char *); 
                                                             // TPT-11431
      teradata::client::API::SchemaX*     (*pfn_newSchmE)(TD_Encoding);

      teradata::client::API::ConnectionX* (*pfn_copyCon)(
                                       teradata::client::API::ConnectionX*,
                                       teradata::client::API::ConnectionX*);
      teradata::client::API::SchemaX* (*pfn_copySchm)(
                                           teradata::client::API::SchemaX*,
                                           teradata::client::API::SchemaX*);
      teradata::client::API::DMLGroupX* (*pfn_copyDMLGrp)(
                                           teradata::client::API::DMLGroupX*,
                                           teradata::client::API::DMLGroupX*);

      void (*pfn_delCon)(teradata::client::API::ConnectionX *);
      void (*pfn_delDMLGrp)(teradata::client::API::DMLGroupX *);
      void (*pfn_delSchm)(teradata::client::API::SchemaX *);

}; // end of class definition

} // namespace API
} // namespace client
} // namespace teradata
} // namespace ncr     
namespace teradata = ncr::teradata;

#endif
