/*************************************************************************
*
* Copyright 2010-2011, 2013-2014 by Teradata Corporation.
* All Rights Reserved.
* TERADATA CONFIDENTIAL AND TRADE SECRET
*
* This copyrighted material is the Confidential, Unpublished
* Property of the Teradata Corporation.  This copyright notice and
* any other copyright notices included in machine readable
* copies must be reproduced on all authorized copies.
*
* History Information:
*
* Revision     Date      DR      DID       Comments
* -------------------------------------------------------------------------
* 13.10.00.01  08172010  142720  mv121803  Support TELAPI dynamic loading
*
*****************************************************************************/

#include "GetBuffer.h"
#include "OptionsManager.h"
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "string.h"

#include "dynlib.h"                                        //DR142720

using namespace teradata::client::API;

GenericDriver::GenericDriver(OptionsManager * om)
{
  inputStream = NULL;
  outputStream = NULL;
  inputFileName = NULL;
  outputFileName = NULL;
  m_om = om;
}

GenericDriver::~GenericDriver(void)
{
}


/***************************************************/
/* testLoad                                        */
/*                                                 */
/* Tests transferring data from Export's GetBuffer */
/* to Load's PutBuffer. Uses the following steps:  */
/*                                                 */
/* 1. Setup and Initiate Load driver               */
/* 2. Query buffer layout from Load                */
/* 3. Pass Load buffer layout to Export driver     */
/* 4. Initiate Export driver                       */
/* 5. Transfer buffers from GetBuffer to PutBuffer */
/* 6. Call Load's EndAcquisition & ApplyRows       */
/* 7. Terminate Load and Export                    */
/***************************************************/
void GenericDriver::testExportToLoad()
{
  DynamicLibrary  *lib_hdl = DynamicLibrary::Instance();     //DR142720

  time_t now;
  time(&now);

  int ldStatus = 0;
  int expStatus = 0;
  int queryStatus = 0;
  char* errorMsg = NULL;
  TD_ErrorType errorType;
  BufferLayout layout; /* BufferLayout struct defined in GetBuffer.h */
  RowCounts ldCount;   /* RowCounts struct defined in GetBuffer.h */
  char *dataptr;
  TD_Length datalen;
  char* msg = (char*)malloc(256);
 
  /**********************************************
  * Retrieve Options From Option File
  **********************************************/

  int numSessions           = m_om -> getNumSessions();
  char *tdp_id              = strdup(m_om -> getTdpId().c_str());
  char *user_name           = strdup(m_om -> getUserName().c_str());
  char *user_password       = strdup(m_om -> getUserPassword().c_str());
  char *target_table        = strdup(m_om -> getTargetTable().c_str());
  char *target_table_export = strdup(m_om -> getTargetTableExport().c_str());
  char *log_table           = strdup(m_om -> getLogTable().c_str());
  char *error_table_1       = strdup(m_om -> getErrorTable1().c_str());
  char *error_table_2       = strdup(m_om -> getErrorTable2().c_str());

  char *ins_statement       = (char*)malloc(1024);
  char *sel_statement       = (char*)malloc(1024);
  sprintf(ins_statement, "INSERT INTO %s( :col1, :col2);", target_table);
  sprintf(sel_statement, "SELECT * FROM %s;", target_table_export);

  ConnectionX *ldConn  = lib_hdl -> newConnectionX();           //DR142720
  ConnectionX *expConn = lib_hdl -> newConnectionX();           //DR142720

  cout << "######################### OUTPUT START #########################" \
  << endl;
  cout << "\nTest Description     :GetBuffer sample" << endl;
  cout << "Date                 :" << ctime(&now);

  cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
  cout << "      LOAD OPERATOR CONNECTION INFO        " << endl;
  cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

  cout << "Operator type     :LOAD" << endl;
  cout << "TDP ID            :" << tdp_id << endl;
  cout << "TDP username      :" << user_name << endl;
  cout << "TDP password      :<not displayable>" << endl;
  cout << "Target Table      :" << target_table << endl;
  cout << "DML Statement     :" << ins_statement << endl;


  /**********************************************
  * Set Operator Type and Trace/Log Levels
  **********************************************/
  ldConn -> AddAttribute(TD_SYSTEM_OPERATOR, TD_LOAD);
  expConn -> AddAttribute(TD_SYSTEM_OPERATOR, TD_EXPORT);
  //expConn -> AddAttribute(TD_TRACE_OUTPUT, "export.txt");
  //expConn -> AddArrayAttribute(TD_TRACE_LEVEL, 2, TD_OPER_ALL,\
  //TD_GENERAL, NULL);

  /**********************************************
  * Add Attributes
  **********************************************/
  ldConn -> AddAttribute(TD_TDP_ID, tdp_id);
  ldConn -> AddAttribute(TD_USER_NAME, user_name);
  ldConn -> AddAttribute(TD_USER_PASSWORD, user_password);
  ldConn -> AddAttribute(TD_TARGET_TABLE, target_table);
  ldConn -> AddAttribute(TD_LOG_TABLE, log_table);
  ldConn -> AddAttribute(TD_ERROR_TABLE_1, error_table_1);
  ldConn -> AddAttribute(TD_ERROR_TABLE_2, error_table_2);
  ldConn -> AddAttribute(TD_MAX_SESSIONS, numSessions);
  ldConn -> AddAttribute(TD_MIN_SESSIONS, numSessions);
  ldConn -> AddAttribute(TD_BUFFER_MODE, "Yes");

  expConn -> AddAttribute(TD_TDP_ID, tdp_id);
  expConn -> AddAttribute(TD_USER_NAME, user_name);
  expConn -> AddAttribute(TD_USER_PASSWORD, user_password);
  expConn -> AddAttribute(TD_SELECT_STMT, sel_statement);
  expConn -> AddAttribute(TD_MAX_SESSIONS, numSessions);
  expConn -> AddAttribute(TD_MIN_SESSIONS, numSessions);
  expConn -> AddAttribute(TD_BUFFER_MODE, "Yes");
 
  /**********************************************
  * Add Schema
  **********************************************/
  SchemaX *ldSchema = lib_hdl -> newSchemaX("input");      //DR142720
  ldSchema -> AddColumn((char*)"col1", TD_INTEGER, 4);  //TPT-17718
  ldSchema -> AddColumn((char*)"col2", TD_VARCHAR, 32000);  //TPT-17718
  ldConn -> AddSchema(ldSchema);

  SchemaX *expSchema = lib_hdl -> newSchemaX("output");    //DR142720
  expSchema -> AddColumn((char*)"col1", TD_INTEGER, 4);  //TPT-17718
  expSchema -> AddColumn((char*)"col2", TD_VARCHAR, 32000);  //TPT-17718
  expConn -> AddSchema(expSchema);

  /**********************************************
  * Add DMLGroups
  **********************************************/
  TD_Index dmlGroupIndex = 0;

  DMLGroupX* dmlGr = lib_hdl -> newDMLGroupX();           //DR142720
  dmlGr -> AddStatement(ins_statement);
  ldConn -> AddDMLGroup(dmlGr, &dmlGroupIndex);
  
  /**********************************************
  * Initiate Load Driver
  **********************************************/
  cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
  cout << "               INITIATE PHASE               " << endl;
  cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

  ldStatus = ldConn -> Initiate();
  if ( ldStatus < TD_Error )
  {
    cout << "Load Driver initiated Successfully" << endl;

    /**********************************************
    * Query Version Number
    **********************************************/
    queryStatus = ldConn -> GetEvent(TD_Evt_Version, &dataptr, &datalen);

    cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
    cout << "              VERSION INFO                  " << endl;
    cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

    if ( queryStatus < TD_Error && queryStatus != TD_Unavailable )
    {
      cout << "TPTAPI Version  :";
      cout << *((char**)dataptr) << endl;
      cout << "Operator Version:";
      cout << *((char**)(dataptr+sizeof(char**))) << endl;
    }
    else
    {
      printStatus("TD_Evt_Version event failed with status ", queryStatus);
    }
   
    /**********************************************
    * Query Connect Status
    **********************************************/
    queryStatus = ldConn -> GetEvent(TD_Evt_ConnectStatus, &dataptr, &datalen);

    cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
    cout << "              CONNECT STATUS                 " << endl;
    cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

    if ( queryStatus < TD_Error && queryStatus != TD_Unavailable )
    {
      cout << "sessions requested:";
      cout << *((int*)dataptr) << endl;
      cout << "sessions connected:";
      cout << *((int*)(dataptr+4)) << endl;
      if( *((int*)(dataptr+8)) == 0 )
        cout << "no CLI error occured" << endl;
      else
      {
        cout << "CLI error: ";
        cout << *((int*)(dataptr+8)) << endl;
      }
    }
    else
    {
      printStatus("TD_Evt_ConnectStatus event failed with status", \
      queryStatus);
    }

    /**********************************************
    * Query Buffer Layout
    **********************************************/
    queryStatus = ldConn -> GetEvent(TD_Evt_BufferLayout, &dataptr, &datalen);

    if ( queryStatus < TD_Error && queryStatus != TD_Unavailable )
    {
      layout = *((BufferLayout*)dataptr);
      cout << "Max Buffer Size    :";
      cout << layout.MaxMsgSize << endl;
      cout << "Row Header Size    :";
      cout << layout.RowHeaderSize << endl;
      cout << "Row Length Size    :";
      cout << layout.RowLengthSize << endl;
      cout << "Buffer Trailer Size:";
      cout << layout.BufTrailerSize << endl;
    }
    else
    {
      printStatus("TD_Evt_BufferLayout event failed with status ", \
      queryStatus);
    }
    expConn -> AddAttribute(TD_BUFFER_MAX_SIZE, layout.MaxMsgSize);
    expConn -> AddAttribute(TD_BUFFER_LENGTH_SIZE, layout.RowLengthSize);
    expConn -> AddAttribute(TD_BUFFER_HEADER_SIZE, layout.RowHeaderSize);
    expConn -> AddAttribute(TD_BUFFER_TRAILER_SIZE, layout.BufTrailerSize);

    cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
    cout << "       EXPORT OPERATOR CONNECTION INFO       " << endl;
    cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

    cout << "Operator type      :EXPORT" << endl;
    cout << "TDP ID             :" << tdp_id  << endl;
    cout << "TDP username       :" << user_name << endl;
    cout << "TDP password       :<not displayable>" << endl;
    cout << "TargetTable        :" << target_table_export << endl;
    cout << "DML Statement      :" << sel_statement << endl;

    /**********************************************
    * Initiate Export Driver
    **********************************************/
    cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
    cout << "             INITIATE PHASE                " << endl;
    cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

    expStatus = expConn -> Initiate();
    if ( expStatus < TD_Error )
    {
      cout << "Export Driver initiated Successfully" << endl;

     /**********************************************
     * Query Version Number
     **********************************************/
     queryStatus = expConn -> GetEvent(TD_Evt_Version, &dataptr, &datalen);

     cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
     cout << "              VERSION INFO                  " << endl;
     cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

     if ( queryStatus < TD_Error && queryStatus != TD_Unavailable )
     {
       cout << "TPTAPI Version  :";
       cout << *((char**)dataptr) << endl;
       cout << "Operator Version:";
       cout << *((char**)(dataptr+sizeof(char**))) << endl;
     }
     else
     {
       printStatus("TD_Evt_Version event failed with status ", queryStatus);
     }

     /**********************************************
     * Query Connect Status
     **********************************************/
     queryStatus = expConn -> GetEvent(TD_Evt_ConnectStatus, &dataptr,\
     &datalen);

     cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
     cout << "              CONNECT STATUS                 " << endl;
     cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

     if ( queryStatus < TD_Error && queryStatus != TD_Unavailable )
     {
       cout << "sessions requested:";
       cout << *((int*)dataptr) << endl;
       cout << "sessions connected:";
       cout << *((int*)(dataptr+4)) << endl;
       if( *((int*)(dataptr+8)) == 0 )
         cout << "no CLI error occured" << endl;
       else
       {
         cout << " CLI error ";
         cout << *((int*)(dataptr+8)) << endl;
       }
     }
     else
     {
       printStatus("TD_Evt_ConnectStatus event failed with status ", \
       queryStatus);
     }
    
     /**********************************************
     * Acquisition
     **********************************************/
     cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
     cout << "             ACQUISITION PHASE            " << endl;
     cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

     char* dataBuffer;
     TD_Length dataLen = 0;
     int count = 0;
     int transferDone = 0;

     while(!transferDone)
     {
       /* Get a buffer from Export */
       expStatus = expConn -> GetBuffer(&dataBuffer, &dataLen);

       if ( expStatus >= TD_Error )
       {
         sprintf(msg,"GetBuffer failed on buffer %d with status ", count+1);
         printStatus(msg, expStatus);
         transferDone = 1;
       }
       else if ( expStatus == TD_END_Method )
       {
         cout << "End of Data Reached" << endl;
         transferDone = 1;
       }
       else
       {
         /* Pass Buffer to Load */
         ldStatus = ldConn -> PutBuffer(dataBuffer, dataLen, 1);

         if ( ldStatus != TD_Success )
         {
           sprintf(msg,"PutBuffer failed on buffer %d with status ", count+1);
           printStatus(msg, ldStatus);
           transferDone = 1;
         }
         else
         {
           count++;
           /* Get Load Row Counts */

           queryStatus = ldConn -> GetEvent(TD_Evt_RowCounts, \
           &dataptr, &datalen);
           if ( queryStatus < TD_Error && queryStatus != TD_Unavailable )
           {
             ldCount = *((RowCounts*)dataptr);
             cout << "Rows Received :";
             cout << ldCount.RowsReceived << endl;
             cout << "Rows Sent     :";
             cout << ldCount.RowsSent << endl;
             cout << "Rows Applied  :";
             cout << ldCount.RowsApplied << endl;
           }
           else
           {
             printStatus("TD_Evt_RowCounts event failed with status ", \
             queryStatus);
           }
         }
       }
     }
     if ( ldStatus < TD_Error && expStatus < TD_Error )
     {
       
       /**********************************************
       * End Acquisition
       **********************************************/

       ldStatus = ldConn -> EndAcquisition();

       if ( ldStatus < TD_Error )
       {
         cout << "Acquisition completed Successfully" << endl;

         /* Get Load Row Counts */

         queryStatus = ldConn -> GetEvent(TD_Evt_RowCounts, &dataptr, \
         &datalen);

         if ( queryStatus < TD_Error && queryStatus != TD_Unavailable )
         {
           ldCount = *((RowCounts*)dataptr);
           cout << "Rows Received :";
           cout << ldCount.RowsReceived << endl;
           cout << "Rows Sent     :";
           cout << ldCount.RowsSent << endl;
           cout << "Rows Applied  :";
           cout << ldCount.RowsApplied << endl;
         }
         else
         {
           cout << "TD_Evt_RowCounts Info Not Available" << \
           queryStatus << endl;
         }
            
         /**********************************************
         * Application
         **********************************************/
         cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
         cout << "             APPLICATION PHASE             " << endl;
         cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

         ldStatus = ldConn -> ApplyRows();

         if ( ldStatus < TD_Error )
         {
           cout << "Rows Applied Successfully" << endl;
           cout << "Transfer completed successfully" << endl;
           cout << "Transferred " << count << " buffers" << endl;

           /* Get Export Row Count */
           queryStatus = expConn -> GetEvent(TD_Evt_ExportCount,\
           &dataptr, &datalen);

           if ( queryStatus < TD_Error && queryStatus != TD_Unavailable )
           {
             cout << "TD_Evt_ExportCount:";
             cout << *((int*)dataptr) << endl;
           }
           else
           {
             printStatus("TD_Evt_ExportCount event failed with status", \
             queryStatus);
           }

           /* Get Load Apply Count */
           queryStatus = ldConn -> GetEvent(TD_Evt_ApplyCount, &dataptr, \
           &datalen);

           if ( queryStatus < TD_Error && queryStatus != TD_Unavailable )
           {
             cout << "TD_Evt_ApplyCount :";
             cout << *((int*)dataptr) << endl;
           }
           else
           {
             printStatus("TD_Evt_ApplyCount event failed with status", \
             queryStatus);
           }
         }
         else
         {
           //Get Error Information

           cout << "Error occured during Application" << endl;
           ldConn -> GetErrorInfo(&errorMsg, &errorType);

           if ( errorMsg != NULL )
           {
             cout << errorMsg << endl;
             cout << "Type: " << errorType << endl;
           }
           else
           {
             cout << "No Error Info Available" << endl;
           }
         }
       }
       else
       {
         //Get Error Information

         cout << "Error occured during EndAcquisition" << endl;
         ldConn -> GetErrorInfo(&errorMsg, &errorType);

         if ( errorMsg != NULL )
         {
           cout << errorMsg << endl;
           cout << "Type: " << errorType << endl;
         }
         else
         {
           cout << "No Error Info Available" << endl;
         }
       }
     }
     else
     {
       //Get Error Information
       if ( ldStatus >= TD_Error )
       {
         cout << "Load Error occured during Acquisition" << endl;
         ldConn -> GetErrorInfo(&errorMsg, &errorType);

         if ( errorMsg != NULL )
         {
           cout << errorMsg << endl;
           cout << "Type: " << errorType << endl;
         }
         else
         {
           cout << "No Error Info Available" << endl;
         }
       }
       if ( expStatus >= TD_Error )
       {
         cout << "Export Error occured during Acquisition" << endl;
         expConn -> GetErrorInfo(&errorMsg, &errorType);
         if ( errorMsg != NULL )
         {
           cout << errorMsg << endl;
           cout << "Type: " << errorType << endl;
         }
         else
         {
           cout << "No Error Info Available" << endl;
         }
        } 
      }
    }
    else
    {
      //Get Error Information

      cout << "Error occured during Export Initiate" << endl;
      expConn -> GetErrorInfo(&errorMsg, &errorType);

      if ( errorMsg != NULL )
      {
        cout << errorMsg << endl;
        cout << "Type: " << errorType << endl;
      }
      else
      {
        cout << "No Error Info Available" << endl;
      }
    }
  }
  else
  {
    //Get Error Information
    cout << "Error occured during Load Initiate" << endl;

    ldConn -> GetErrorInfo(&errorMsg, &errorType);
    if ( errorMsg != NULL )
    {
      cout << errorMsg << endl;
      cout << "Type: " << errorType << endl;
    }
    else
    {
      cout << "No Error Info Available" << endl;
    }
  }
  
  /**********************************************
  * Terminate Drivers
  **********************************************/
  cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
  cout << "              TERMINATE PHASE                " << endl;
  cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

  ldStatus = ldConn -> Terminate();
  expStatus = expConn -> Terminate();

  if ( ldStatus >= TD_Error || expStatus >= TD_Error )
  {
    /* Get Error Information */

    if ( ldStatus >= TD_Error )
    {
      cout << "Error occured during Load Terminate" << endl;
      ldConn -> GetErrorInfo(&errorMsg, &errorType);

      if ( errorMsg != NULL )
      {
        cout << errorMsg << endl;
        cout << "Type: " << errorType << endl;
      }
      else
      {
        cout << "No Error Info Available" << endl;
      }
    }
    else
    {
      cout << "Load Terminated Successfully" << endl;
    }

    if ( expStatus >= TD_Error )
    {
      cout << "Error occured during Export Terminate" << endl;
      expConn -> GetErrorInfo(&errorMsg, &errorType);

      if ( errorMsg != NULL )
      {
        cout << errorMsg << endl;
        cout << "Type: " << errorType << endl;
      }
      else
      {
        cout << "No Error Info Available" << endl;
      }
    }
    else
    {
      cout << "Export Terminated Successfully" << endl;
    }
  }

  /**********************************************
  * Clean Up
  **********************************************/

  cout << "Deleting objects" << endl;

  lib_hdl -> delDMLGroupX(dmlGr);           //DR142720
  lib_hdl -> delSchemaX(ldSchema);          //DR142720
  lib_hdl -> delSchemaX(expSchema);         //DR142720
  lib_hdl -> delConnectionX(ldConn);        //DR142720
  lib_hdl -> delConnectionX(expConn);       //DR142720
  lib_hdl -> Delete();                      //DR142720

  free(user_name);
  free(user_password);
  free(target_table);
  free(target_table_export);
  free(tdp_id);
  free(log_table);
  free(error_table_1);
  free(error_table_2);
  free(ins_statement);
  free(sel_statement);
  free(msg);

  cout << endl << "*** Export to Load Complete ***" << endl;
  cout <<"\n######################## OUTPUT END ####################" << endl;
}

/***************************************************/
/* testExport                                      */
/*                                                 */
/* Tests transferring data from Export's GetBuffer */
/* to file.                                        */
/*                                                 */
/* 1. Set TD_BUFFER_MODE to "Yes" or "Y"           */
/* 2. Set Buffer layout attributes                 */
/* 3. Initiate Export driver                       */
/* 4. Get buffers of data from GetBuffer           */
/* 5. Write buffer to file                         */
/* 6. Terminate                                    */
/***************************************************/
void GenericDriver::testExportToFile()
{
  DynamicLibrary  *lib_hdl = DynamicLibrary::Instance();      //DR142720

  time_t now;
  time(&now);

  int returnValue = 0;
  int queryStatus = 0;
  char* errorMsg = NULL;
  TD_ErrorType errorType;
  BufferLayout layout; /* BufferLayout struct defined in GetBuffer.h */

  char *dataptr;
  TD_Length datalen;
  char* msg = (char*)malloc(256);

  /**********************************************
  * Retrieve Options From Option File
  **********************************************/
  int numSessions     = m_om -> getNumSessions();
  char *tdp_id        = strdup(m_om -> getTdpId().c_str());
  char *user_name     = strdup(m_om -> getUserName().c_str());
  char *user_password = strdup(m_om -> getUserPassword().c_str());
  char *target_table  = strdup(m_om -> getTargetTableExport().c_str());

  char *sel_statement = (char*)malloc(1024);
  sprintf(sel_statement, "SELECT * FROM %s;", target_table);

  ConnectionX *conn = lib_hdl -> newConnectionX();           //DR142720

  cout << "######################### OUTPUT START #########################" \
  << endl;
  cout << "\nTest Description     :GetBuffer sample" << endl;
  cout << "Operator type        :EXPORT" << endl;
  cout << "Date                 :" << ctime(&now);
  cout << "TDP ID               :" << tdp_id << endl;
  cout << "TDP username         :" << user_name << endl;
  cout << "TDP password         :<not displayable>" << endl;
  cout << "Target Table         :" << target_table << endl;
  cout << "Output File          :" << outputFileName << endl;
  cout << "DML Statement        :" << sel_statement << endl;

  /**********************************************
  * Set Operator Type and Trace/Log Levels
  **********************************************/
  conn -> AddAttribute(TD_SYSTEM_OPERATOR, TD_EXPORT);
  //conn -> AddAttribute(TD_TRACE_OUTPUT, "export.txt");
  //conn -> AddArrayAttribute(TD_TRACE_LEVEL, 2, TD_OPER_ALL,\
  //TD_GENERAL, NULL);

  /**********************************************
  * Add Attributes
  **********************************************/
  conn -> AddAttribute(TD_TDP_ID, tdp_id);
  conn -> AddAttribute(TD_USER_NAME, user_name);
  conn -> AddAttribute(TD_USER_PASSWORD, user_password);
  conn -> AddAttribute(TD_SELECT_STMT, sel_statement);
  conn -> AddAttribute(TD_MAX_SESSIONS, numSessions);
  conn -> AddAttribute(TD_MIN_SESSIONS, numSessions);

  /* Must be set to "Yes" or "Y" to use GetBuffer */
  conn -> AddAttribute(TD_BUFFER_MODE, "Yes");

  /**********************************************
  * Set Buffer Layout
  **********************************************/
  cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
  cout << "              BUFFER LAYOUT                  " << endl;
  cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

  /* Values set here will be added using AddAttribute below */
  layout.MaxMsgSize = 64260;
  layout.RowHeaderSize = 8;
  layout.RowLengthSize = 2;
  layout.BufTrailerSize = 0;

  cout << "TD_BUFFER_MAX_SIZE    :" << layout.MaxMsgSize << endl;
  cout << "TD_BUFFER_HEADER_SIZE :" << layout.RowHeaderSize << endl;
  cout << "TD_BUFFER_LENGTH_SIZE :" << layout.RowLengthSize << endl;
  cout << "TD_BUFFER_TRAILER_SIZE:" << layout.BufTrailerSize << endl;

  /* Buffer layout attributes may be set in any order */
  /* Maximum buffer size may be as large as memory can handle but */
  /* must be at least big enough to hold 1 row                    */
  conn -> AddAttribute(TD_BUFFER_MAX_SIZE, layout.MaxMsgSize);

  /*Buffer length size may be 4,2, or 0 */
  conn -> AddAttribute(TD_BUFFER_LENGTH_SIZE, layout.RowLengthSize);

  /*Buffer header size must be at least as big as buffer length size */
  conn -> AddAttribute(TD_BUFFER_HEADER_SIZE, layout.RowHeaderSize);

  /*Buffer trailer size may be 0 or bigger */
  conn -> AddAttribute(TD_BUFFER_TRAILER_SIZE, layout.BufTrailerSize);

  /**********************************************
  * Add Schema
  **********************************************/
  SchemaX *schema = lib_hdl -> newSchemaX("output");    //DR142720
  schema -> AddColumn((char*)"col1", TD_INTEGER, 4);  //TPT-17718
  schema -> AddColumn((char*)"col2", TD_VARCHAR, 32000);  //TPT-17718
  conn -> AddSchema(schema);
  
  /**********************************************
  * Initiate
  **********************************************/
  cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
  cout << "              INITIATE PHASE               " << endl;
  cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

  returnValue = conn -> Initiate();

  if ( returnValue < TD_Error )
  {
    cout << "Driver initiated Successfully" << endl;

    /**********************************************
    * Query Version Number
    **********************************************/
    queryStatus = conn -> GetEvent(TD_Evt_Version, &dataptr, &datalen);

    cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
    cout << "              VERSION INFO                  " << endl;
    cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

    if ( queryStatus < TD_Error && queryStatus != TD_Unavailable )
    {
      cout << "TPTAPI Version  :";
      cout << *((char**)dataptr) << endl;
      cout << "Operator Version:";
      cout << *((char**)(dataptr+sizeof(char**))) << endl;
    }
    else
    {
      printStatus("TD_Evt_Version event failed with status ", queryStatus);
    }
    
    /**********************************************
    * Query Connect Status
    **********************************************/

    queryStatus = conn -> GetEvent(TD_Evt_ConnectStatus, &dataptr, &datalen);

    cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
    cout << "               CONNECT STATUS                " << endl;
    cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

    if ( queryStatus < TD_Error && queryStatus != TD_Unavailable )
    {
      //cout << "TD_Evt_ConnectStatus: sessions requested: ";
      cout << "sessions requested:";
      cout << *((int*)dataptr) << endl;
      cout << "sessions connected:";
      cout << *((int*)(dataptr+4)) << endl;
      if( *((int*)(dataptr+8)) == 0 )
        cout << "no CLI error occured" << endl;
      else
      {
        cout << " CLI error ";
        cout << *((int*)(dataptr+8)) << endl;
      }
    }
    else
    {
      printStatus("TD_Evt_ConnectStatus event failed with status ", \
      queryStatus);
    }
   
    /**********************************************
    * Acquisition
    **********************************************/
    cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
    cout << "             ACQUISITION PHASE            " << endl;
    cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

    int count = 0;
    char* data;
    TD_Length dataLen = 0;
    int exportStatus = 0;

    while( exportStatus != -1)
    {
      /* Get a buffer from Export */
      returnValue = conn -> GetBuffer(&data, &dataLen);
      if ( returnValue >= TD_Error )
      {
        sprintf(msg,"GetBuffer failed on buffer %d with status ", count+1);
        printStatus(msg, returnValue);
        exportStatus = WriteBufferData(NULL, 0); //Abort write
      }
      else if ( returnValue == TD_END_Method )
      {
        cout << "End of Data Reached" << endl;
        exportStatus = WriteBufferData(NULL, 0); //end write
      }
      else
      {
        /* Write buffer out to file */
        exportStatus = WriteBufferData(data, dataLen);
        count++;
      }

      /**********************************************
      * Query Block Count after 1st call to GetBuffer
      **********************************************/
      if ( returnValue < TD_Error && count <= 1)
      {
        queryStatus = conn -> GetEvent(TD_Evt_BlockCount, &dataptr, &datalen);
        if ( queryStatus < TD_Error && queryStatus != TD_Unavailable )
        {
          cout << "TD_Evt_BlockCount :";
          cout << *((int*)dataptr) << endl;
        }
        else
        {
          printStatus("TD_Evt_BlockCount event failed with status ", \
          queryStatus);
        }
      }

      /**********************************************
      * Query Export Count
      **********************************************/
      if ( returnValue < TD_Error)
      {
        queryStatus = conn -> GetEvent(TD_Evt_ExportCount, &dataptr, \
        &datalen);

        if ( queryStatus < TD_Error && queryStatus != TD_Unavailable )
        {
          cout << "TD_Evt_ExportCount:";
          cout << *((int*)dataptr) << endl;
        }
        else
        {
          printStatus("TD_Evt_ExportCount event failed with status ", \
          queryStatus);
        }
      }
    }
    cout << "Exported " << count << " Buffers" << endl;

    if ( returnValue < TD_Error )
    {
      cout << "Export completed successfully" << endl;
    }
    else
    {
      //Get Error Information

      cout << "Error occured during Export" << endl;
      conn -> GetErrorInfo(&errorMsg, &errorType);

      if ( errorMsg != NULL )
      {
        cout << errorMsg << endl;
        cout << "Type: " << errorType << endl;
      }
      else
      {
        cout << "No Error Info Available" << endl;
      }
    }
  }
  else
  {
    //Get Error Information

    cout << "Error occured during Initiate" << endl;
    conn -> GetErrorInfo(&errorMsg, &errorType);

    if ( errorMsg != NULL )
    {
     cout << errorMsg << endl;
      cout << "Type: " << errorType << endl;
    }
    else
    {
      cout << "No Error Info Available" << endl;
    }
  }
 
  /**********************************************
  * Terminate
  **********************************************/

  cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
  cout << "              TERMINATE PHASE                " << endl;
  cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

  returnValue = conn -> Terminate();
  if ( returnValue >= TD_Error )
  {
    //Get Error Information

    cout << "Error occured during Terminate" << endl;
    conn -> GetErrorInfo(&errorMsg, &errorType);

    if ( errorMsg != NULL )
    {
      cout << errorMsg << endl;
      cout << "Type: " << errorType << endl;
    }
    else
    {
      cout << "No Error Info Available" << endl;
    }
  }
  else
  {
    cout << "Driver Terminated Successfully" << endl;
  }

  /**********************************************
  * Clean Up
  **********************************************/

  cout << "Deleting Objects" << endl;
        
  lib_hdl -> delSchemaX(schema);          //DR142720
  lib_hdl -> delConnectionX(conn);        //DR142720
  lib_hdl -> Delete();                    //DR142720
	
  free(user_name);
  free(user_password);
  free(target_table);
  free(tdp_id);
  free(sel_statement);
  free(msg);

  cout << endl << "*** Export To File Complete ***" << endl;
  cout <<"\n######################## OUTPUT END ####################" << endl;
}

/****************************************************/
/* testUpdate                                       */
/*                                                  */
/* Tests transferring data from Export's GetBuffer  */
/* to Updates's PutBuffer. Uses the following steps:*/
/*                                                  */
/* 1. Setup and Initiate Update driver              */
/* 2. Query buffer layout from Update               */
/* 3. Pass Update buffer layout to Export driver    */
/* 4. Initiate Export driver                        */
/* 5. Transfer buffers from GetBuffer to PutBuffer  */
/* 6. Call Update's EndAcquisition & ApplyRows      */
/* 7. Terminate Update and Export                   */
/****************************************************/
void GenericDriver::testExportToUpdate()
{
  DynamicLibrary  *lib_hdl = DynamicLibrary::Instance();     //DR142720

  time_t now;
  time(&now);

  int updStatus = 0;
  int expStatus = 0;
  int queryStatus = 0;
  char* errorMsg = NULL;
  TD_ErrorType errorType;
  BufferLayout layout; /* BufferLayout struct defined in GetBuffer.h */
  RowCounts updCount;  /* RowCounts struct defined in GetBuffer.h */

  char *dataptr;
  TD_Length datalen;
  char* msg = (char*)malloc(256);

  /**********************************************
  * Retrieve Options From Option File
  **********************************************/
  int numSessions           = m_om -> getNumSessions();
  char *tdp_id              = strdup(m_om -> getTdpId().c_str());
  char *user_name           = strdup(m_om -> getUserName().c_str());
  char *user_password       = strdup(m_om -> getUserPassword().c_str());
  char *target_table        = strdup(m_om -> getTargetTable().c_str());
  char *target_table_export = strdup(m_om -> getTargetTableExport().c_str());
  char *log_table           = strdup(m_om -> getLogTable().c_str());
  char *work_table          = strdup(m_om -> getWorkTable().c_str());
  char *error_table_1       = strdup(m_om -> getErrorTable1().c_str());
  char *error_table_2       = strdup(m_om -> getErrorTable2().c_str());

  char *ins_statement       = (char*)malloc(1024);
  char *sel_statement       = (char*)malloc(1024);
  sprintf(ins_statement, "INSERT INTO %s( :col1, :col2);", target_table);
  sprintf(sel_statement, "SELECT * FROM %s;", target_table_export);


  cout << "######################### OUTPUT START #########################" \
  << endl;
  cout << "\nTest Description     :GetBuffer sample" << endl;
  cout << "Date                 :" << ctime(&now);

  cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
  cout << "       UPDATE OPERATOR CONNECTION INFO       " << endl;
  cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
  cout << "Operator type         :UPDATE" << endl;
  cout << "TDP ID                :" << tdp_id << endl;
  cout << "TDP username          :" << user_name << endl;
  cout << "TDP password          :<not displayable>" << endl;
  cout << "Target Table          :" << target_table << endl;
  cout << "DML Statement         :" << ins_statement << endl;

  ConnectionX *updConn = lib_hdl -> newConnectionX();           //DR142720
  ConnectionX *expConn = lib_hdl -> newConnectionX();           //DR142720

  /**********************************************
  * Set Operator Type and Trace/Log Levels
  **********************************************/
  updConn -> AddAttribute(TD_SYSTEM_OPERATOR, TD_UPDATE);
  expConn -> AddAttribute(TD_SYSTEM_OPERATOR, TD_EXPORT);

  //updConn -> AddAttribute(TD_TRACE_OUTPUT, "update.txt");
  //updConn -> AddArrayAttribute(TD_TRACE_LEVEL, 2, TD_OPER_ALL,\
  //TD_GENERAL, NULL);

  //expConn -> AddAttribute(TD_TRACE_OUTPUT, "export.txt");
  //expConn -> AddArrayAttribute(TD_TRACE_LEVEL, 2, TD_OPER_ALL,\
  //TD_GENERAL, NULL);

  /**********************************************
  * Add Attributes
  **********************************************/
  updConn -> AddAttribute(TD_TDP_ID, tdp_id);
  updConn -> AddAttribute(TD_USER_NAME, user_name);
  updConn -> AddAttribute(TD_USER_PASSWORD, user_password);
  updConn -> AddAttribute(TD_TARGET_TABLE, target_table);
  updConn -> AddAttribute(TD_LOG_TABLE, log_table);
  updConn -> AddAttribute(TD_WORK_TABLE, work_table);
  updConn -> AddAttribute(TD_ERROR_TABLE_1, error_table_1);
  updConn -> AddAttribute(TD_ERROR_TABLE_2, error_table_2);
  updConn -> AddAttribute(TD_MAX_SESSIONS, numSessions);
  updConn -> AddAttribute(TD_MIN_SESSIONS, numSessions);
  updConn -> AddAttribute(TD_BUFFER_MODE, "Yes");

  expConn -> AddAttribute(TD_TDP_ID, tdp_id);
  expConn -> AddAttribute(TD_USER_NAME, user_name);
  expConn -> AddAttribute(TD_USER_PASSWORD, user_password);
  expConn -> AddAttribute(TD_SELECT_STMT, sel_statement);
  expConn -> AddAttribute(TD_MAX_SESSIONS, numSessions);
  expConn -> AddAttribute(TD_MIN_SESSIONS, numSessions);
  expConn -> AddAttribute(TD_BUFFER_MODE, "Yes");

  /**********************************************
  * Add Schema
  **********************************************/
  SchemaX *updSchema = lib_hdl -> newSchemaX("input");    //DR142720
  updSchema -> AddColumn((char*)"col1", TD_INTEGER, 4);  //TPT-17718
  updSchema -> AddColumn((char*)"col2", TD_VARCHAR, 32000);  //TPT-17718
  updConn -> AddSchema(updSchema);

  SchemaX *expSchema = lib_hdl -> newSchemaX("output");   //DR142720
  expSchema -> AddColumn((char*)"col1", TD_INTEGER, 4);  //TPT-17718
  expSchema -> AddColumn((char*)"col2", TD_VARCHAR, 32000);  //TPT-17718
  expConn -> AddSchema(expSchema);

  /**********************************************
  * Add DMLGroups
  **********************************************/
  TD_Index dmlGroupIndex = 0;

  DMLGroupX *dmlGr = lib_hdl -> newDMLGroupX();        //DR142720
  dmlGr -> AddStatement(ins_statement);
  updConn -> AddDMLGroup(dmlGr, &dmlGroupIndex);

  /**********************************************
  * Initiate Update Driver
  **********************************************/
  cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
  cout << "                INITIATE PHASE               " << endl;
  cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

  updStatus = updConn -> Initiate();
  if ( updStatus < TD_Error )
  {
    cout << "Update Driver initiated Successfully" << endl;

    /**********************************************
    * Query Version Number
    **********************************************/
    queryStatus = updConn -> GetEvent(TD_Evt_Version, &dataptr, &datalen);

    cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
    cout << "              VERSION INFO                  " << endl;
    cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

    if ( queryStatus < TD_Error && queryStatus != TD_Unavailable )
    {
      cout << "TPTAPI Version  :";
      cout << *((char**)dataptr) << endl;
      cout << "Operator Version:";
      cout << *((char**)(dataptr+sizeof(char**))) << endl;
    }
    else
    {
      printStatus("TD_Evt_Version event failed with status ", queryStatus);
    }

    /**********************************************
    * Query Connect Status
    **********************************************/
    queryStatus = updConn -> GetEvent(TD_Evt_ConnectStatus, \
    &dataptr, &datalen);

    cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
    cout << "              CONNECT STATUS                 " << endl;
    cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

    if ( queryStatus < TD_Error && queryStatus != TD_Unavailable )
    {
      cout << "sessions requested:";
      cout << *((int*)dataptr) << endl;
      cout << "sessions connected:";
      cout << *((int*)(dataptr+4)) << endl;
      if( *((int*)(dataptr+8)) == 0 )
        cout << "no CLI error occured" << endl;
      else
       {
        cout << " CLI error ";
        cout << *((int*)(dataptr+8)) << endl;
      }
    }
    else
    {
      printStatus("TD_Evt_ConnectStatus event failed with status ",\
      queryStatus);
    }
	
    /**********************************************
    * Query Buffer Layout
    **********************************************/
    queryStatus = updConn -> GetEvent(TD_Evt_BufferLayout, &dataptr, &datalen);

    if ( queryStatus < TD_Error && queryStatus != TD_Unavailable )
    {
      layout = *((BufferLayout*)dataptr);
      cout << "Max Buffer Size    :";
      cout << layout.MaxMsgSize << endl;
      cout << "Row Header Size    :";
      cout << layout.RowHeaderSize << endl;
      cout << "Row Length Size    :";
      cout << layout.RowLengthSize << endl;
      cout << "Buffer Trailer Size:";
      cout << layout.BufTrailerSize << endl;
    }
    else
    {
      printStatus("TD_Evt_BufferLayout event failed with status ", \
      queryStatus);
    }

    /* Give Buffer Layout to Export Driver */
    expConn -> AddAttribute(TD_BUFFER_MAX_SIZE, layout.MaxMsgSize);
    expConn -> AddAttribute(TD_BUFFER_LENGTH_SIZE, layout.RowLengthSize);
    expConn -> AddAttribute(TD_BUFFER_HEADER_SIZE, layout.RowHeaderSize);
    expConn -> AddAttribute(TD_BUFFER_TRAILER_SIZE, layout.BufTrailerSize);

    cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
    cout << "       EXPORT OPERATOR CONNECTION INFO       " << endl;
    cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

    cout << "Operator type      :EXPORT" << endl;
    cout << "TDP ID             :" << tdp_id << endl;
    cout << "TDP username       :" << user_name << endl;
    cout << "TDP password       :<not displayable>" << endl;
    cout << "TargetTable        :" << target_table_export << endl;
    cout << "DML Statement      :" << sel_statement << endl;
   
    /**********************************************
    * Initiate Export Driver
    **********************************************/
    cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
    cout << "        INITIATE PHASE FOR EXPORT            " << endl;
    cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

    expStatus = expConn -> Initiate();

    if ( expStatus < TD_Error )
    {
      cout << "Export Driver initiated Successfully" << endl;

      /**********************************************
      * Query Version Number
      **********************************************/
      queryStatus = expConn -> GetEvent(TD_Evt_Version, &dataptr, &datalen);

      cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
      cout << "              VERSION INFO                  " << endl;
      cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
      if ( queryStatus < TD_Error && queryStatus != TD_Unavailable )
      {
        cout << "TPTAPI Version  :";
        cout << *((char**)dataptr) << endl;
        cout << "Operator Version:";
        cout << *((char**)(dataptr+sizeof(char**))) << endl;
      }
      else
      {
        printStatus("TD_Evt_Version event failed with status ", queryStatus);
      }
      /**********************************************
      * Query Connect Status
      **********************************************/
      queryStatus = expConn -> GetEvent(TD_Evt_ConnectStatus, \
      &dataptr, &datalen);

      cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
      cout << "              CONNECT STATUS                 " << endl;
      cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

      if ( queryStatus < TD_Error && queryStatus != TD_Unavailable )
      {
        cout << "sessions requested:";
        cout << *((int*)dataptr) << endl;
        cout << "sessions connected:";
        cout << *((int*)(dataptr+4)) << endl;
        if( *((int*)(dataptr+8)) == 0 )
              cout << "no CLI error occured" << endl;
        else
        {
         cout << " CLI error ";
              cout << *((int*)(dataptr+8)) << endl;
        }
      }
      else
      {
        printStatus("TD_Evt_ConnectStatus event failed with status ", \
        queryStatus);
      }

      /**********************************************
      * Acquisition
      **********************************************/
      cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
      cout << "             ACQUISITION PHASE            " << endl;
      cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

      char* dataBuffer;
      TD_Length dataLen = 0;
      int count = 0;
      int transferDone = 0;

      while(!transferDone)
      {
        /* Get a buffer from Export */
        expStatus = expConn -> GetBuffer(&dataBuffer, &dataLen);

        if ( expStatus >= TD_Error )
        {
          sprintf(msg,"GetBuffer failed on buffer %d with status ", count+1);
          printStatus(msg, expStatus);
          transferDone = 1;
        }
        else if ( expStatus == TD_END_Method )
        {
          cout << "End of Data Reached" << endl;
          transferDone = 1;
        }
        else
        {
          /* Pass buffer to Update */

          updStatus = updConn -> PutBuffer(dataBuffer, dataLen, 1);
          if ( updStatus != TD_Success )
          {
            sprintf(msg,"PutBuffer failed on buffer %d with status ", count+1);
            printStatus(msg, updStatus);
            transferDone = 1;
          }
          else
          {
            count++;
            /* Get Update Row Counts */

            queryStatus = updConn -> GetEvent(TD_Evt_RowCounts, \
            &dataptr, &datalen);

            if ( queryStatus < TD_Error && queryStatus != TD_Unavailable )
            {
              updCount = *((RowCounts*)dataptr);
              cout << "Rows Received : ";
              cout << updCount.RowsReceived << endl;
              cout << "Rows Sent     : ";
              cout << updCount.RowsSent << endl;
              cout << "Rows Applied  : ";
              cout << updCount.RowsApplied << endl;
            }
            else
            { 
              printStatus("TD_Evt_RowCounts event failed with status ", \
              queryStatus);
            }
          }
        }
      }
      if ( updStatus < TD_Error && expStatus < TD_Error )
      {

        /**********************************************
        * End Acquisition
        **********************************************/
        updStatus = updConn -> EndAcquisition();
        cout << "Acquisition completed Successfully" << endl;

        if ( updStatus < TD_Error )
        {
          /* Get Update Row Counts */

          queryStatus = updConn -> GetEvent(TD_Evt_RowCounts, \
          &dataptr, &datalen);

          if ( queryStatus < TD_Error && queryStatus != TD_Unavailable )
          {
            updCount = *((RowCounts*)dataptr);
            cout << "Rows Received : ";
            cout << updCount.RowsReceived << endl;
            cout << "Rows Sent     : ";
            cout << updCount.RowsSent << endl;
            cout << "Rows Applied  : ";
            cout << updCount.RowsApplied << endl;
          }
          else
          {
            printStatus("TD_Evt_RowCounts event failed with status ", \
            queryStatus);
          }

          /**********************************************
          * Application
          **********************************************/
          cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
          cout << "             APPLICATION PHASE             " << endl;
          cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

          updStatus = updConn -> ApplyRows();

          if ( updStatus < TD_Error )
          {
            cout << "Rows Applied Successfully" << endl;
            cout << "Transfer completed successfully" << endl;
            cout << "Transferred " << count << " buffers" << endl;

            /* Get Export Row Count */
            queryStatus = expConn -> GetEvent(TD_Evt_ExportCount, \
            &dataptr, &datalen);

            if ( queryStatus < TD_Error && queryStatus != TD_Unavailable )
            {
              cout << "TD_Evt_ExportCount : ";
              cout << *((int*)dataptr) << endl;
            }
            else
            {
              printStatus("TD_Evt_ExportCount event failed with status ",\
              queryStatus);
            }

            /* Get Update Apply Count */
            queryStatus = updConn -> GetEvent(TD_Evt_ApplyCount, &dataptr,\
            &datalen,1);

            if ( queryStatus < TD_Error && queryStatus != TD_Unavailable )
            {
              cout << "TD_Evt_ApplyCount  : ";
              cout << *((int*)dataptr) << endl;
            }
            else
            {
              printStatus("TD_Evt_ApplyCount event failed with status ",\
              queryStatus);
            }
          }
          else
          {
            //Get Error Information

            cout << "Error occured during Application" << endl;
            updConn -> GetErrorInfo(&errorMsg, &errorType);
            if ( errorMsg != NULL )
            {
              cout << errorMsg << endl;
              cout << "Type: " << errorType << endl;
            }
            else
            {
              cout << "No Error Info Available" << endl;
            }
          }
        }
        else
        {
          //Get Error Information

          cout << "Error occured during EndAcquisition" << endl;
          updConn -> GetErrorInfo(&errorMsg, &errorType);

          if ( errorMsg != NULL )
          {
            cout << errorMsg << endl;
            cout << "Type: " << errorType << endl;
          }
          else 
          {
            cout << "No Error Info Available" << endl;
          }
        }
      }
      else
      {
        //Get Error Information
        if ( updStatus >= TD_Error )
        {
          cout << "Update Error occured during Acquisition" << endl;
          updConn -> GetErrorInfo(&errorMsg, &errorType);

          if ( errorMsg != NULL )
          {
            cout << errorMsg << endl;
            cout << "Type: " << errorType << endl;
          }
          else
          {
            cout << "No Error Info Available" << endl;
          }
        }
        if ( expStatus >= TD_Error )
        {
          cout << "Export Error occured during Acquisition" << endl;
          expConn -> GetErrorInfo(&errorMsg, &errorType);

          if ( errorMsg != NULL )
          {
            cout << errorMsg << endl;
            cout << "Type: " << errorType << endl;
          }
          else
          {
            cout << "No Error Info Available" << endl;
          }
        }
      }
    }
    else
    {
      //Get Error Information

      cout << "Error occured during Export Initiate" << endl;
      expConn -> GetErrorInfo(&errorMsg, &errorType);

      if ( errorMsg != NULL )
      {
        cout << errorMsg << endl;
        cout << "Type: " << errorType << endl;
      }
      else
      {
        cout << "No Error Info Available" << endl;
      }
    }
  }
  else
  {
    //Get Error Information

    cout << "Error occured during Update Initiate" << endl;
    updConn -> GetErrorInfo(&errorMsg, &errorType);

    if ( errorMsg != NULL )
    {
      cout << errorMsg << endl;
      cout << "Type: " << errorType << endl;
    }
    else
    {
      cout << "No Error Info Available" << endl;
    }
  }

  /**********************************************
  * Terminate Drivers
  **********************************************/

  cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
  cout << "              TERMINATE PHASE                " << endl;
  cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

  updStatus = updConn -> Terminate();
  expStatus = expConn -> Terminate();

  if ( updStatus >= TD_Error || expStatus >= TD_Error )
  {
    //Get Error Information

    if ( updStatus >= TD_Error )
    {
      cout << "Error occured during Update Terminate" << endl;
      updConn -> GetErrorInfo(&errorMsg, &errorType);

      if ( errorMsg != NULL )
      {
        cout << errorMsg << endl;
        cout << "Type: " << errorType << endl;
      }
      else
      {
        cout << "No Error Info Available" << endl;
      }
    }
    else
    {
      cout << "Update Terminated Successfully" << endl;
    }

    if ( expStatus >= TD_Error )
    {
      cout << "Error occured during Export Terminate" << endl;
      expConn -> GetErrorInfo(&errorMsg, &errorType);

      if ( errorMsg != NULL )
      {
        cout << errorMsg << endl;
        cout << "Type: " << errorType << endl;
      }
      else
      {
        cout << "No Error Info Available" << endl;
      }
    }
    else
    {
      cout << "Export Terminated Successfully" << endl;
    }
  }

  /**********************************************
  * Clean Up
  **********************************************/

  cout << "Deleting objects" << endl;

  lib_hdl -> delDMLGroupX(dmlGr);           //DR142720
  lib_hdl -> delSchemaX(updSchema);         //DR142720
  lib_hdl -> delSchemaX(expSchema);         //DR142720
  lib_hdl -> delConnectionX(updConn);       //DR142720
  lib_hdl -> delConnectionX(expConn);       //DR142720
  lib_hdl -> Delete();                      //DR142720

  free(user_name);
  free(user_password);
  free(target_table);
  free(target_table_export);
  free(tdp_id);
  free(log_table);
  free(work_table);
  free(error_table_1);
  free(error_table_2);
  free(ins_statement);
  free(sel_statement);
  free(msg);

  cout << endl << "*** Export To Update Complete ***" << endl;
  cout <<"\n######################## OUTPUT END ####################" << endl;

}

/****************************************************/
/* testStream                                       */
/*                                                  */
/* Tests transferring data from Export's GetBuffer  */
/* to Stream's PutBuffer. Uses the following steps: */
/*                                                  */
/* 1. Setup and Initiate Stream driver              */
/* 2. Query buffer layout from Stream               */
/* 3. Pass Stream buffer layout to Export driver    */
/* 4. Initiate Export driver                        */
/* 5. Transfer buffers from GetBuffer to PutBuffer  */
/* 6. Call Stream's EndAcquisition                  */
/* 7. Terminate Stream and Export                   */
/****************************************************/
void GenericDriver::testExportToStream()
{
  DynamicLibrary  *lib_hdl = DynamicLibrary::Instance();     //DR142720
  
  time_t now;
  time(&now);
  int strStatus = 0;
  int expStatus = 0;
  int queryStatus = 0;
  char* errorMsg = NULL;
  TD_ErrorType errorType;
  BufferLayout layout; /* BufferLayout struct defined in GetBuffer.h */
  RowCounts strCount;  /* RowCounts struct defined in GetBuffer.h */
  RunStats strStats; /* RunStats struct defined in GetBuffer.h */

  char *dataptr;
  TD_Length datalen;
  char* msg = (char*)malloc(256);

   /**********************************************
  * Retrieve Options From Option File
  **********************************************/
  int numSessions           = m_om -> getNumSessions();
  char *tdp_id              = strdup(m_om -> getTdpId().c_str());
  char *user_name           = strdup(m_om -> getUserName().c_str());
  char *user_password       = strdup(m_om -> getUserPassword().c_str());
  char *target_table        = strdup(m_om -> getTargetTable().c_str());
  char *target_table_export = strdup(m_om -> getTargetTableExport().c_str());
  char *log_table           = strdup(m_om -> getLogTable().c_str());
  char *error_table         = strdup(m_om -> getErrorTable().c_str());

  char *ins_statement = (char*)malloc(1024);
  char *sel_statement = (char*)malloc(1024);
  sprintf(ins_statement, "INSERT INTO %s( :col1, :col2);", target_table);
  sprintf(sel_statement, "SELECT * FROM %s;", target_table_export);

  cout << "######################### OUTPUT START #########################" \
  << endl;
  cout << "\nTest Description     :GetBuffer sample" << endl;
  cout << "Date                 :" << ctime(&now);

  cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
  cout << "       STREAM OPERATOR CONNECTION INFO       " << endl;
  cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

  cout << "Operator type     :STREAM" << endl;
  cout << "TDP ID            :" << tdp_id << endl;
  cout << "TDP username      :" << user_name << endl;
  cout << "TDP password      :<not displayable>" << endl;
  cout << "Target Table      :" << target_table << endl;
  cout << "DML Statement     :" << ins_statement << endl;

  ConnectionX *strConn = lib_hdl->newConnectionX();           //DR142720
  ConnectionX *expConn = lib_hdl->newConnectionX();           //DR142720

  /**********************************************
  * Set Operator Type and Trace/Log Levels
  **********************************************/
  strConn -> AddAttribute(TD_SYSTEM_OPERATOR, TD_STREAM);
  expConn -> AddAttribute(TD_SYSTEM_OPERATOR, TD_EXPORT);

  //strConn -> AddAttribute(TD_TRACE_OUTPUT, "stream.txt");
  //strConn -> AddArrayAttribute(TD_TRACE_LEVEL, 2, TD_OPER_ALL,
  //TD_GENERAL, NULL);

  //expConn -> AddAttribute(TD_TRACE_OUTPUT, "export.txt");
  //expConn -> AddArrayAttribute(TD_TRACE_LEVEL, 2, TD_OPER_ALL,
  //TD_GENERAL, NULL);

  /**********************************************
  * Add Attributes
  **********************************************/
  strConn -> AddAttribute(TD_TDP_ID, tdp_id);
  strConn -> AddAttribute(TD_USER_NAME, user_name);
  strConn -> AddAttribute(TD_USER_PASSWORD, user_password);
  strConn -> AddAttribute(TD_TARGET_TABLE, target_table);
  strConn -> AddAttribute(TD_LOG_TABLE, log_table);
  strConn -> AddAttribute(TD_ERROR_TABLE, error_table);
  strConn -> AddAttribute(TD_MAX_SESSIONS, numSessions);
  strConn -> AddAttribute(TD_MIN_SESSIONS, numSessions);
  strConn -> AddAttribute(TD_BUFFER_MODE, "Yes");

  expConn -> AddAttribute(TD_TDP_ID, tdp_id);
  expConn -> AddAttribute(TD_USER_NAME, user_name);
  expConn -> AddAttribute(TD_USER_PASSWORD, user_password);
  expConn -> AddAttribute(TD_SELECT_STMT, sel_statement);
  expConn -> AddAttribute(TD_MAX_SESSIONS, numSessions);
  expConn -> AddAttribute(TD_MIN_SESSIONS, numSessions);
  expConn -> AddAttribute(TD_BUFFER_MODE, "Yes");

  /**********************************************
  * Add Schema
  **********************************************/
  SchemaX *strSchema = lib_hdl->newSchemaX("input");    //DR142720
  strSchema -> AddColumn((char*)"col1", TD_INTEGER, 4);  //TPT-17718
  strSchema -> AddColumn((char*)"col2", TD_VARCHAR, 32000);  //TPT-17718
  strConn -> AddSchema(strSchema);

  SchemaX *expSchema = lib_hdl->newSchemaX("output");   //DR142720
  expSchema -> AddColumn((char*)"col1", TD_INTEGER, 4);  //TPT-17718
  expSchema -> AddColumn((char*)"col2", TD_VARCHAR, 32000);  //TPT-17718
  expConn -> AddSchema(expSchema);

  /**********************************************
  * Add DMLGroups
  **********************************************/
  TD_Index dmlGroupIndex = 0;

  DMLGroupX *dmlGr = lib_hdl -> newDMLGroupX();        //DR142720
  dmlGr -> AddStatement(ins_statement);
  strConn -> AddDMLGroup(dmlGr, &dmlGroupIndex);

  /**********************************************
  * Initiate Load Driver
  **********************************************/
  cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
  cout << "         INITIATE PHASE FOR STREAM          " << endl;
  cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

  strStatus = strConn -> Initiate();

  if ( strStatus < TD_Error )
  {
    cout << "Stream Driver initiated Successfully" << endl;

    /**********************************************
    * Query Version Number
    **********************************************/

    queryStatus = strConn -> GetEvent(TD_Evt_Version, &dataptr, &datalen);

    cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
    cout << "              VERSION INFO                  " << endl;
    cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

    if ( queryStatus < TD_Error && queryStatus != TD_Unavailable )
    {
      cout << "TPTAPI Version  :";
      cout << *((char**)dataptr) << endl;
      cout << "Operator Version:";
      cout << *((char**)(dataptr+sizeof(char**))) << endl;
    }
    else
    {
      printStatus("TD_Evt_Version event failed with status ", queryStatus);
    }

    /**********************************************
    * Query Connect Status
    **********************************************/

    queryStatus = strConn -> GetEvent(TD_Evt_ConnectStatus, \
    &dataptr, &datalen);

    cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
    cout << "              CONNECT STATUS                 " << endl;
    cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

    if ( queryStatus < TD_Error && queryStatus != TD_Unavailable )
    {
      cout << "sessions requested:";
      cout << *((int*)dataptr) << endl;
      cout << "sessions connected:";
      cout << *((int*)(dataptr+4)) << endl;
      if( *((int*)(dataptr+8)) == 0 )
        cout << "no CLI error occured" << endl;
      else
      {
        cout << "CLI error";
        cout << *((int*)(dataptr+8)) << endl;
      }
    }
    else
    {
      printStatus("TD_Evt_ConnectStatus event failed with status ", \
      queryStatus);
    }

    /**********************************************
    * Query Buffer Layout
    **********************************************/

    queryStatus = strConn -> GetEvent(TD_Evt_BufferLayout, &dataptr, &datalen);

    if ( queryStatus < TD_Error && queryStatus != TD_Unavailable )
    {
      layout = *((BufferLayout*)dataptr);
      cout << "Max Buffer Size    :";
      cout << layout.MaxMsgSize << endl;
      cout << "Row Header Size    :";
      cout << layout.RowHeaderSize << endl;
      cout << "Row Length Size    :";
      cout << layout.RowLengthSize << endl;
      cout << "Buffer Trailer Size:";
      cout << layout.BufTrailerSize << endl;
    }
    else
    {
      printStatus("TD_Evt_BufferLayout event failed with status ", \
      queryStatus);
    }

    /* Give Buffer Layout to Export Driver */
    expConn -> AddAttribute(TD_BUFFER_MAX_SIZE, layout.MaxMsgSize);
    expConn -> AddAttribute(TD_BUFFER_LENGTH_SIZE, layout.RowLengthSize);
    expConn -> AddAttribute(TD_BUFFER_HEADER_SIZE, layout.RowHeaderSize);
    expConn -> AddAttribute(TD_BUFFER_TRAILER_SIZE, layout.BufTrailerSize);

    cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
    cout << "       EXPORT OPERATOR CONNECTION INFO       " << endl;
    cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

    cout << "Operator type      :EXPORT" << endl;
    cout << "TDP ID             :" << tdp_id << endl;
    cout << "TDP username       :" << user_name << endl;
    cout << "TDP password       :<not displayable>" << endl;
    cout << "TargetTable        :" << target_table_export << endl;
    cout << "DML Statement      :" << sel_statement << endl;

    /**********************************************
    * Initiate Export Driver
    **********************************************/
    cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
    cout << "        INITIATE PHASE FOR EXPORT            " << endl;
    cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

    expStatus = expConn -> Initiate();

    if ( expStatus < TD_Error )
    {
      cout << "Export Driver initiated Successfully" << endl;

      /**********************************************
      * Query Version Number
      **********************************************/

      queryStatus = expConn -> GetEvent(TD_Evt_Version, &dataptr, &datalen);

      cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
      cout << "              VERSION INFO                  " << endl;
      cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

      if ( queryStatus < TD_Error && queryStatus != TD_Unavailable )
      {
        cout << "TPTAPI Version  :";
        cout << *((char**)dataptr) << endl;
        cout << "Operator Version:";
        cout << *((char**)(dataptr+sizeof(char**))) << endl;
      }
      else
      {
        printStatus("TD_Evt_Version event failed with status ", queryStatus);
      }

      /**********************************************
      * Query Connect Status
      **********************************************/
      queryStatus = expConn -> GetEvent(TD_Evt_ConnectStatus, &dataptr,\
      &datalen);

      cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
      cout << "              CONNECT STATUS                 " << endl;
      cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

      if ( queryStatus < TD_Error && queryStatus != TD_Unavailable )
      {
        cout << "sessions requested:";
        cout << *((int*)dataptr) << endl;
        cout << "sessions connected:";
        cout << *((int*)(dataptr+4)) << endl;
        if( *((int*)(dataptr+8)) == 0 )
          cout << "no CLI error occured" << endl;
        else
        {
          cout << "CLI error ";
          cout << *((int*)(dataptr+8)) << endl;
        }
      }
      else
      {
        printStatus("TD_Evt_ConnectStatus event failed with status ",\
        queryStatus);
      }
      
      /**********************************************
      * Acquisition
      **********************************************/
      cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
      cout << "             ACQUISITION PHASE               " << endl;
      cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

      char* dataBuffer;
      TD_Length dataLen = 0;
      int count = 0;
      int transferDone = 0;

      while(!transferDone)
      {
        /* Get a buffer from Export */
        expStatus = expConn -> GetBuffer(&dataBuffer, &dataLen);
        if ( expStatus >= TD_Error )
        {
          sprintf(msg,"GetBuffer failed on buffer %d with status ", count+1);
          printStatus(msg, expStatus);
          transferDone = 1;
        }
        else if ( expStatus == TD_END_Method )
        {
          cout << "End of Data Reached" << endl;
          transferDone = 1;
        }
        else
        {
          /* Pass buffer to Stream */
          strStatus = strConn -> PutBuffer(dataBuffer, dataLen, 1);
          if ( strStatus != TD_Success )
          {
            sprintf(msg,"PutBuffer failed on buffer %d with status ", count+1);
            printStatus(msg, strStatus);
            transferDone = 1;
          }
          else
          {
            count++;
            /* Get Stream Row Counts */

            queryStatus = strConn -> GetEvent(TD_Evt_RowCounts,\
            &dataptr, &datalen);

            if ( queryStatus < TD_Error && queryStatus != TD_Unavailable )
            {
              strCount = *((RowCounts*)dataptr);
              cout << endl << "Rows Received               :";
              cout << strCount.RowsReceived << endl;
              cout << "Rows Sent                   :";
              cout << strCount.RowsSent << endl;
              cout << "Rows That Caused DBS Errors :";
              cout << strCount.RowsApplied << endl;
            }
            else
            {
              printStatus("TD_Evt_RowCounts event failed with status ", \
              queryStatus);
            }
          }
        }
      }

      if ( strStatus < TD_Error && expStatus < TD_Error )
      {
        /**********************************************
        * End Acquisition
        **********************************************/

        strStatus = strConn -> EndAcquisition();

        if ( strStatus < TD_Error )
        {

          /* Get Stream Row Counts */
          cout << "Acquisition completed Successfully" << endl;
          queryStatus = strConn -> GetEvent(TD_Evt_RowCounts, \
          &dataptr, &datalen);

          if ( queryStatus < TD_Error && queryStatus != TD_Unavailable )
          {
            strCount = *((RowCounts*)dataptr);
            cout << endl << "Rows Received             :";
            cout << strCount.RowsReceived << endl;
            cout << "Rows Sent                   :";
            cout << strCount.RowsSent << endl;
            cout << "Rows That Caused DBS Errors :";
            cout << strCount.RowsApplied << endl;
          }
          else
          {
            printStatus("TD_Evt_RowCounts event failed with status ", \
            queryStatus);
          }

          /**********************************************
          * Application - No Application For Stream
          **********************************************/

          if ( strStatus < TD_Error )
          {
            cout << "Transfer completed successfully" << endl;
            cout << "Transferred " << count << " buffers" << endl;

            /* Get Export Row Count */
            queryStatus = expConn -> GetEvent(TD_Evt_ExportCount, \
            &dataptr, &datalen);

            if ( queryStatus < TD_Error && queryStatus != TD_Unavailable )
            {
              cout << "TD_Evt_ExportCount: ";
              cout << *((int*)dataptr) << endl;
            }
            else
            {
              printStatus("TD_Evt_ExportCount event failed with status ", \
              queryStatus);
            }

            /* Get Stream Run Stats */
            queryStatus = strConn -> GetEvent(TD_Evt_RunStats, \
            &dataptr, &datalen);

            if ( queryStatus < TD_Error && queryStatus != TD_Unavailable )
            {
              strStats = *((RunStats*)dataptr);
              cout << "SQL Statements Sent         :";
              cout << strStats.SQLStmtsSent << endl;
              cout << "DBS Requests Sent           :";
              cout << strStats.RequestsSent << endl;
              cout << "Rows Received               :";
              cout << strStats.RowsReceived << endl;
              cout << "Rows Sent                   :";
              cout << strStats.RowsSent << endl;
              cout << "Rows That Caused DBS Errors :";
              cout << strStats.DBSErrorRows << endl;
            }
            else
            {
              printStatus("TD_Evt_RunStats event failed with status", \
              queryStatus);
            }
          }
          else
          {
            //Get Error Information

            cout << "Error occured during Application" << endl;
            strConn -> GetErrorInfo(&errorMsg, &errorType);

            if ( errorMsg != NULL )
            {
              cout << errorMsg << endl;
              cout << "Type: " << errorType << endl;
            }
            else
            {
              cout << "No Error Info Available" << endl;
            }
          }
        }
        else
        {
          //Get Error Information

          cout << "Error occured during EndAcquisition" << endl;
          strConn -> GetErrorInfo(&errorMsg, &errorType);

          if ( errorMsg != NULL )
          {
            cout << errorMsg << endl;
            cout << "Type: " << errorType << endl;
          }
          else
          {
            cout << "No Error Info Available" << endl;
          }
        }
      }
      else
      {
        //Get Error Information
        if ( strStatus >= TD_Error )
        {
          cout << "Stream Error occured during Acquisition" << endl;
          strConn -> GetErrorInfo(&errorMsg, &errorType);

          if ( errorMsg != NULL )
          {
            cout << errorMsg << endl;
            cout << "Type: " << errorType << endl;
          }
          else
          {
            cout << "No Error Info Available" << endl;
          }
        }
        if ( expStatus >= TD_Error )
        {
          cout << "Export Error occured during Acquisition" << endl;
          expConn -> GetErrorInfo(&errorMsg, &errorType);

          if ( errorMsg != NULL )
          {
            cout << errorMsg << endl;
            cout << "Type: " << errorType << endl;
          }
          else
          {
            cout << "No Error Info Available" << endl;
          }
        }
      }
    }
    else
    {
      //Get Error Information

      cout << "Error occured during Export Initiate" << endl;
      expConn -> GetErrorInfo(&errorMsg, &errorType);

      if ( errorMsg != NULL )
      {
        cout << errorMsg << endl;
        cout << "Type: " << errorType << endl;
      }
      else
      {
        cout << "No Error Info Available" << endl;
      }
    }
  }
  else
  {
    //Get Error Information

    cout << "Error occured during Stream Initiate" << endl;
    strConn -> GetErrorInfo(&errorMsg, &errorType);

    if ( errorMsg != NULL )
    {
      cout << errorMsg << endl;
      cout << "Type: " << errorType << endl;
    }
    else
    {
      cout << "No Error Info Available" << endl;
    }
  }
  /**********************************************
  * Terminate Drivers
  **********************************************/
  cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
  cout << "              TERMINATE PHASE                " << endl;
  cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

  strStatus = strConn -> Terminate();
  expStatus = expConn -> Terminate();

  if ( strStatus >= TD_Error || expStatus >= TD_Error )
  {
    //Get Error Information
    if ( strStatus >= TD_Error )
    {
      cout << "Error occured during Stream Terminate" << endl;
      strConn -> GetErrorInfo(&errorMsg, &errorType);

      if ( errorMsg != NULL )
      {
        cout << errorMsg << endl;
        cout << "Type: " << errorType << endl;
      }
      else
      {
        cout << "No Error Info Available" << endl;
      }
    }
    else
    {
      cout << "Stream Terminated Successfully" << endl;
    }

    if ( expStatus >= TD_Error )
    {
      cout << "Error occured during Export Terminate" << endl;
      expConn -> GetErrorInfo(&errorMsg, &errorType);

      if ( errorMsg != NULL )
      {
        cout << errorMsg << endl;
        cout << "Type: " << errorType << endl;
      }
      else
      {
        cout << "No Error Info Available" << endl;
      }
    }
    else
    {
      cout << "Export Terminated Successfully" << endl;
    }
  }

  /**********************************************
  * Clean Up
  **********************************************/
  cout << "Deleting objects" << endl;
	
  lib_hdl -> delDMLGroupX(dmlGr);           //DR142720
  lib_hdl -> delSchemaX(strSchema);         //DR142720
  lib_hdl -> delSchemaX(expSchema);         //DR142720
  lib_hdl -> delConnectionX(strConn);       //DR142720
  lib_hdl -> delConnectionX(expConn);       //DR142720
  lib_hdl -> Delete();                      //DR142720

  free(user_name);
  free(user_password);
  free(target_table);
  free(target_table_export);
  free(tdp_id);
  free(log_table);
  free(error_table);
  free(ins_statement);
  free(sel_statement);
  free(msg);

  cout << endl << "*** Export To Stream Complete ***" << endl;
  cout <<"\n######################## OUTPUT END ####################" << endl;
}

/***************************************************/
/* testExportToPutRow                              */
/*                                                 */
/* Tests transferring data from Export's GetBuffer */
/* to Load's PutRow. Uses the following steps:     */
/*                                                 */
/* 1. Setup and Initiate Load driver               */
/* 2. Setup and Initiate Export driver             */
/* 3. Get buffer using GetBuffer                   */
/* 4. Parse through buffer to get rows             */
/* 5. Load rows using PutRow                       */
/* 6. Call Load's EndAcquisition & ApplyRows       */
/* 7. Terminate Load and Export                    */
/***************************************************/
void GenericDriver::testExportToPutRow()
{
  DynamicLibrary  *lib_hdl = DynamicLibrary::Instance();    //DR142720
 
  time_t now;
  time(&now); 
  int ldStatus = 0;
  int expStatus = 0;
  int queryStatus = 0;
  char* errorMsg = NULL;
  TD_ErrorType errorType;
  BufferLayout layout; /* BufferLayout struct defined in GetBuffer.h */
  RowCounts ldCount;   /* RowCounts struct defined in GetBuffer.h */

  char *dataptr;
  char *layoutptr;
  TD_Length datalen;
  char* msg = (char*)malloc(256);
 
  /**********************************************
  * Retrieve Options From Option File
  **********************************************/
  int numSessions           = m_om -> getNumSessions();
  char *tdp_id              = strdup(m_om -> getTdpId().c_str());
  char *user_name           = strdup(m_om -> getUserName().c_str());
  char *user_password       = strdup(m_om -> getUserPassword().c_str());
  char *target_table        = strdup(m_om -> getTargetTable().c_str());
  char *target_table_export = strdup(m_om -> getTargetTableExport().c_str());
  char *log_table           = strdup(m_om -> getLogTable().c_str());
  char *error_table_1       = strdup(m_om -> getErrorTable1().c_str());
  char *error_table_2       = strdup(m_om -> getErrorTable2().c_str());

  char *ins_statement       = (char*)malloc(1024);
  char *sel_statement       = (char*)malloc(1024);
  sprintf(ins_statement, "INSERT INTO %s( :col1, :col2);", target_table);
  sprintf(sel_statement, "SELECT * FROM %s;", target_table_export);

  cout << "######################### OUTPUT START #########################" \
  << endl;
  cout << "\nTest Description     :GetBuffer sample" << endl;
  cout << "Date                 :" << ctime(&now);

  cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
  cout << "       EXPORT OPERATOR CONNECTION INFO       " << endl;
  cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

  cout << "Operator type      :EXPORT" << endl;
  cout << "TDP ID             :" << tdp_id << endl;
  cout << "TDP username       :" << user_name << endl;
  cout << "TDP password       :<not displayable>" << endl;
  cout << "TargetTable        :" << target_table_export << endl;
  cout << "DML Statement      :" << sel_statement << endl;

  cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
  cout << "            BUFFER LAYOUT INFO              " << endl;
  cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

  layout.MaxMsgSize = 100000;
  layout.RowHeaderSize = 2;
  layout.RowLengthSize = 2; /* We'll need the length when calling PutRow */
  layout.BufTrailerSize = 0;

  layoutptr = (char*)&layout;
  cout << endl;
  cout << "TD_BUFFER_MAX_SIZE    :" << layout.MaxMsgSize << endl;
  cout << "TD_BUFFER_HEADER_SIZE :" << layout.RowHeaderSize << endl;
  cout << "TD_BUFFER_LENGTH_SIZE :" << layout.RowLengthSize << endl;
  cout << "TD_BUFFER_TRAILER_SIZE:" << layout.BufTrailerSize << endl;

  ConnectionX *ldConn  = lib_hdl -> newConnectionX();           //DR142720
  ConnectionX *expConn = lib_hdl -> newConnectionX();           //DR142720

  /**********************************************
  * Set Operator Type and Trace/Log Levels
  **********************************************/
  ldConn -> AddAttribute(TD_SYSTEM_OPERATOR, TD_LOAD);
  expConn -> AddAttribute(TD_SYSTEM_OPERATOR, TD_EXPORT);
  //expConn -> AddAttribute(TD_TRACE_OUTPUT, "export.txt");
  //expConn -> AddArrayAttribute(TD_TRACE_LEVEL, 2, TD_OPER_ALL,
  //TD_GENERAL, NULL);

  /**********************************************
  * Add Attributes
  **********************************************/
  ldConn -> AddAttribute(TD_TDP_ID, tdp_id);
  ldConn -> AddAttribute(TD_USER_NAME, user_name);
  ldConn -> AddAttribute(TD_USER_PASSWORD, user_password);
  ldConn -> AddAttribute(TD_TARGET_TABLE, target_table);
  ldConn -> AddAttribute(TD_LOG_TABLE, log_table);
  ldConn -> AddAttribute(TD_ERROR_TABLE_1, error_table_1);
  ldConn -> AddAttribute(TD_ERROR_TABLE_2, error_table_2);
  ldConn -> AddAttribute(TD_MAX_SESSIONS, numSessions);
  ldConn -> AddAttribute(TD_MIN_SESSIONS, numSessions);

  expConn -> AddAttribute(TD_TDP_ID, tdp_id);
  expConn -> AddAttribute(TD_USER_NAME, user_name);
  expConn -> AddAttribute(TD_USER_PASSWORD, user_password);
  expConn -> AddAttribute(TD_SELECT_STMT, sel_statement);
  expConn -> AddAttribute(TD_MAX_SESSIONS, numSessions);
  expConn -> AddAttribute(TD_MIN_SESSIONS, numSessions);
  expConn -> AddAttribute(TD_BUFFER_MODE, "Yes");

  expConn -> AddAttribute(TD_BUFFER_MAX_SIZE, layout.MaxMsgSize);
  expConn -> AddAttribute(TD_BUFFER_LENGTH_SIZE, layout.RowLengthSize);
  expConn -> AddAttribute(TD_BUFFER_HEADER_SIZE, layout.RowHeaderSize);
  expConn -> AddAttribute(TD_BUFFER_TRAILER_SIZE, layout.BufTrailerSize);

  /**********************************************
  * Add Schema
  **********************************************/
  SchemaX *ldSchema = lib_hdl -> newSchemaX("input");           //DR142720
  ldSchema -> AddColumn((char*)"col1", TD_INTEGER, 4);  //TPT-17718
  ldSchema -> AddColumn((char*)"col2", TD_VARCHAR, 32000);  //TPT-17718
  ldConn -> AddSchema(ldSchema);

  SchemaX *expSchema = lib_hdl->newSchemaX("output");         //DR142720
  expSchema -> AddColumn((char*)"col1", TD_INTEGER, 4);  //TPT-17718
  expSchema -> AddColumn((char*)"col2", TD_VARCHAR, 32000);  //TPT-17718
  expConn -> AddSchema(expSchema);

  /**********************************************
  * Add DMLGroups
  **********************************************/
  TD_Index dmlGroupIndex = 0;

  DMLGroupX* dmlGr = lib_hdl->newDMLGroupX();           //DR142720
  dmlGr -> AddStatement(ins_statement);
  ldConn -> AddDMLGroup(dmlGr, &dmlGroupIndex);

  /**********************************************
  * Initiate Export Driver
  **********************************************/
  cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
  cout << "        INITIATE PHASE FOR EXPORT            " << endl;
  cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

  expStatus = expConn -> Initiate();

  if ( expStatus < TD_Error)
  {
    cout << "Export Driver initiated Successfully" << endl;

    /**********************************************
    * Query Version Number
    **********************************************/
    queryStatus = expConn -> GetEvent(TD_Evt_Version, &dataptr, &datalen);

    cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
    cout << "              VERSION INFO                  " << endl;
    cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

    if ( queryStatus < TD_Error && queryStatus != TD_Unavailable )
    {
      cout << "TPTAPI Version  :";
      cout << *((char**)dataptr) << endl;
      cout << "Operator Version:";
      cout << *((char**)(dataptr+sizeof(char**))) << endl;
    }
    else
    {
      printStatus("TD_Evt_Version event failed with status ", queryStatus);
    }

    /**********************************************
    * Query Connect Status
    **********************************************/
    queryStatus = expConn -> GetEvent(TD_Evt_ConnectStatus, \
    &dataptr, &datalen);

    cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
    cout << "              CONNECT STATUS                 " << endl;
    cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

    if ( queryStatus < TD_Error && queryStatus != TD_Unavailable )
    {
      cout << "sessions requested:";
      cout << *((int*)dataptr) << endl;
      cout << "sessions connected:";
      cout << *((int*)(dataptr+4)) << endl;
      if( *((int*)(dataptr+8)) == 0 )
            cout << "no CLI error occured" << endl;
      else
      {
        cout << "CLI error ";
        cout << *((int*)(dataptr+8)) << endl;
      }
    }
    else
    {
      printStatus("TD_Evt_ConnectStatus event failed with status ", \
      queryStatus);
    }
    
    cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
    cout << "    LOAD OPERATOR CONNECTION INFO " << endl;
    cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

    cout << "Operator type     :LOAD(using PUTROW)" << endl;
    cout << "TDP ID            :" << tdp_id << endl;
    cout << "TDP username      :" << user_name << endl;
    cout << "TDP password      :<not displayable>" << endl;
    cout << "Target Table      :" << target_table << endl;
    cout << "DML Statement     :" << ins_statement << endl;

    /**********************************************
    * Initiate Load Driver
    **********************************************/
    cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
    cout << "         INITIATE PHASE FOR LOAD             " << endl;
    cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

    ldStatus = ldConn -> Initiate();
    if ( ldStatus < TD_Error )
    {
      cout << "Load Driver initiated Successfully" << endl;

      /**********************************************
      * Query Version Number
      **********************************************/
      queryStatus = ldConn -> GetEvent(TD_Evt_Version, &dataptr, &datalen);

      cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
      cout << "              VERSION INFO                  " << endl;
      cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

      if ( queryStatus < TD_Error && queryStatus != TD_Unavailable )
      {
        cout << "TPTAPI Version  : ";
        cout << *((char**)dataptr) << endl;
        cout << "Operator Version: ";
        cout << *((char**)(dataptr+sizeof(char**))) << endl;
      }
      else
      {
        printStatus("TD_Evt_Version event failed with status ", queryStatus);
      }

      /**********************************************
      * Query Connect Status
      **********************************************/
      queryStatus = ldConn -> GetEvent(TD_Evt_ConnectStatus, &dataptr,\
      &datalen);

      cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
      cout << "              CONNECT STATUS                 " << endl;
      cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

      if ( queryStatus < TD_Error && queryStatus != TD_Unavailable )
      {
        cout << "sessions requested:";
        cout << *((int*)dataptr) << endl;
        cout << "sessions connected:";
        cout << *((int*)(dataptr+4)) << endl;
        if( *((int*)(dataptr+8)) == 0 )
          cout << "no CLI error occured" << endl;
        else
        {
          cout << " CLI error ";
          cout << *((int*)(dataptr+8)) << endl;
        }
      }
      else
      {
        printStatus("TD_Evt_ConnectStatus event failed with status", \
        queryStatus);
      }
     
      /**********************************************
      * Acquisition
      **********************************************/
      cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
      cout << "             ACQUISITION PHASE            " << endl;
      cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

      char* dataBuffer;
      char* curBufPos;
      int curBufLen;
      TD_Length recordLen = 0;
      unsigned short recordLenShort = 0;
      int parcelLen = layout.RowHeaderSize - layout.RowLengthSize;
      TD_Length dataLen = 0;
      int count = 0;
      int bufCount = 0;
      int transferDone = 0;
      int eob = 0;
      int byteCount = 0;

      while(!transferDone)
      {
        /* Get a buffer from Export */
        expStatus = expConn -> GetBuffer(&dataBuffer, &dataLen);

        if ( expStatus >= TD_Error )
        {
          sprintf(msg,"GetBuffer failed on buffer %d with status ", bufCount+1);
          printStatus(msg, expStatus);
          transferDone = 1;
        }
        else if ( expStatus == TD_END_Method )
        {
          cout << "End of Data Reached" << endl;
          transferDone = 1;
        }
        else
        {
          /* Parse Buffer */
          eob = 0;
          curBufLen = 0;
          curBufPos = dataBuffer;
          bufCount++;
          byteCount += dataLen;

          while(!eob)
          {
            /* Quit if at end of buffer */
            if( curBufLen >= ( dataLen - layout.BufTrailerSize ) )
            {
              eob = 1;
              break;
            }
            /* Skip Parcel */
            curBufPos += parcelLen;
            curBufLen += parcelLen;
            /* Read Length - Length size could be 4 or 2 */
            if( layout.RowLengthSize == 2 )
            {
              /* DR124305 */
              memcpy(&recordLenShort, curBufPos, sizeof(unsigned short));
              recordLen = recordLenShort;
              curBufPos += sizeof(unsigned short);
              curBufLen += sizeof(unsigned short);
            }
            else if ( layout.RowLengthSize == 4 )
            {
              /* DR124305 */
              memcpy(&recordLen, curBufPos, sizeof(unsigned int));
              curBufPos += sizeof(unsigned int);
              curBufLen += sizeof(unsigned int);
            }
            else
            {
              cout << "Invalid row length size: " << layout.RowLengthSize << endl;
            }

            /* PutRow */
            ldStatus = ldConn -> PutRow(curBufPos, recordLen);

            if ( ldStatus != TD_Success )
            {
              sprintf(msg,"PutRow failed on row %d with status ", count+1);
              printStatus(msg, ldStatus);
              transferDone = 1;
              eob = 1;
              break;
            }

            /* Increment to Next Record */
            curBufPos += recordLen;
            curBufLen += recordLen;

            /* Increment count */
            count++;
          }

          /* Get Load Row Counts */
          queryStatus = ldConn -> GetEvent(TD_Evt_RowCounts, &dataptr,\
          &datalen);

          if ( queryStatus < TD_Error && queryStatus != TD_Unavailable )
          {
            ldCount = *((RowCounts*)dataptr);
            cout << "Rows Received :";
            cout << ldCount.RowsReceived << endl;
            cout << "Rows Sent     :";
            cout << ldCount.RowsSent << endl;
            cout << "Rows Applied  :";
            cout << ldCount.RowsApplied << endl;
          }
          else
          {
            printStatus("TD_Evt_RowCounts event failed with status", \
            queryStatus);
          }
        }
      }
      if ( ldStatus < TD_Error && expStatus < TD_Error )
      {
    
        /**********************************************
        * End Acquisition
        **********************************************/

        ldStatus = ldConn -> EndAcquisition();
        if ( ldStatus < TD_Error )
        {
          cout << "Acquisition complected Successfully" << endl;

          /* Get Load Row Counts */

          queryStatus = ldConn -> GetEvent(TD_Evt_RowCounts, \
          &dataptr, &datalen);

          if ( queryStatus < TD_Error && queryStatus != TD_Unavailable )
          {
            ldCount = *((RowCounts*)dataptr);
            cout << "Rows Received :";
            cout << ldCount.RowsReceived << endl;
            cout << "Rows Sent     :";
            cout << ldCount.RowsSent << endl;
            cout << "Rows Applied  :";
            cout << ldCount.RowsApplied << endl;
          }
          else
          {
            printStatus("TD_Evt_RowCounts event failed with status", \
            queryStatus);
          }

          /**********************************************
          * Application
          **********************************************/
          cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
          cout << "             APPLICATION PHASE             " << endl;
          cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

          ldStatus = ldConn -> ApplyRows();

          if ( ldStatus < TD_Error )
          {
            cout << "Rows Applied Successfully" << endl;
            cout << "Transfer completed successfully" << endl;
            cout << endl;
            cout << "Transferred " << bufCount << " buffers" << endl;
            cout << "Transferred " << byteCount << " bytes" << endl;
            cout << "Transferred " << count << " rows" << endl;

            /* Get Export Row Count */
            queryStatus = expConn -> GetEvent(TD_Evt_ExportCount, &dataptr,\
            &datalen);

            if ( queryStatus < TD_Error && queryStatus != TD_Unavailable )
            {
              cout << "TD_Evt_ExportCount :";
              cout << *((int*)dataptr) << endl;
            }
            else
            {
              printStatus("TD_Evt_ExportCount event failed with status ",\
              queryStatus);
            }

            /* Get Load Apply Count */
            queryStatus = ldConn -> GetEvent(TD_Evt_ApplyCount, &dataptr,\
            &datalen);

            if ( queryStatus < TD_Error && queryStatus != TD_Unavailable )
            {
              cout << "TD_Evt_ApplyCount  :";
              cout << *((int*)dataptr) << endl;
            } 
            else
            {
              printStatus("TD_Evt_ApplyCount event failed with status",\
              queryStatus);
            }
            cout << endl;
          }
          else
          {
            //Get Error Information
            cout << "Error occured during Application" << endl;
            ldConn -> GetErrorInfo(&errorMsg, &errorType);

            if ( errorMsg != NULL )
            {
              cout << errorMsg << endl;
              cout << "Type: " << errorType << endl;
            }
            else
            {
              cout << "No Error Info Available" << endl;
            }
          }
        }
        else
        {
          //Get Error Information

          cout << "Error occured during EndAcquisition" << endl;
          ldConn -> GetErrorInfo(&errorMsg, &errorType);

          if ( errorMsg != NULL )
          {
            cout << errorMsg << endl;
            cout << "Type: " << errorType << endl;
          }
          else
          {
            cout << "No Error Info Available" << endl;
          }
        }
      }
      else
      {
        //Get Error Information
        if ( ldStatus >= TD_Error )
        {
          cout << "Load Error occured during Acquisition" << endl;
          ldConn -> GetErrorInfo(&errorMsg, &errorType);

          if ( errorMsg != NULL )
          {
            cout << errorMsg << endl;
            cout << "Type: " << errorType << endl;
          }
          else
          {
            cout << "No Error Info Available" << endl;
          }
        }
        if ( expStatus >= TD_Error )
        {
           cout << "Export Error occured during Acquisition" << endl;
          expConn -> GetErrorInfo(&errorMsg, &errorType);

          if ( errorMsg != NULL )
          {
            cout << errorMsg << endl;
            cout << "Type: " << errorType << endl;
          }
          else
          {
            cout << "No Error Info Available" << endl;
          }
        }
      }
    }
    else
    {
      //Get Error Information

      cout << "Error occured during Load Initiate" << endl;
      ldConn -> GetErrorInfo(&errorMsg, &errorType);

      if ( errorMsg != NULL )
      {
        cout << errorMsg << endl;
        cout << "Type: " << errorType << endl;
      }
      else
      {
        cout << "No Error Info Available" << endl;
      }
    }
  }
  else
  {
    //Get Error Information

    cout << "Error occured during Export Initiate" << endl;
    expConn -> GetErrorInfo(&errorMsg, &errorType);

    if ( errorMsg != NULL )
    {
      cout << errorMsg << endl;
      cout << "Type: " << errorType << endl;
    }
    else
    {
      cout << "No Error Info Available" << endl;
    }
  }

  /**********************************************
  * Terminate Drivers
  **********************************************/
  cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
  cout << "              TERMINATE PHASE                " << endl;
  cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

  ldStatus = ldConn -> Terminate();
  expStatus = expConn -> Terminate();

  if ( ldStatus >= TD_Error || expStatus >= TD_Error )
  {
    //Get Error Information
    if ( ldStatus >= TD_Error )
    {
      cout << "Error occured during Load Terminate" << endl;
      ldConn -> GetErrorInfo(&errorMsg, &errorType);
      if ( errorMsg != NULL )
      {
        cout << errorMsg << endl;
        cout << "Type: " << errorType << endl;
      }
      else
      {
        cout << "No Error Info Available" << endl;
      }
    }
    else
    {
      cout << "Load Terminated Successfully" << endl;
    }

    if ( expStatus >= TD_Error )
    {
      cout << "Error occured during Export Terminate" << endl;
      expConn -> GetErrorInfo(&errorMsg, &errorType);

      if ( errorMsg != NULL )
      {
        cout << errorMsg << endl;
        cout << "Type: " << errorType << endl;
      }
      else
      {
        cout << "No Error Info Available" << endl;
      }
    }
    else
    {
      cout << "Export Terminated Successfully" << endl;
    }
  }

  /**********************************************
  * Clean Up
  **********************************************/

  cout << "Deleting objects" << endl;

  lib_hdl -> delDMLGroupX(dmlGr);           //DR142720
  lib_hdl -> delSchemaX(ldSchema);          //DR142720
  lib_hdl -> delSchemaX(expSchema);         //DR142720
  lib_hdl -> delConnectionX(ldConn);        //DR142720
  lib_hdl -> delConnectionX(expConn);       //DR142720
  lib_hdl -> Delete();                      //DR142720

  free(user_name);
  free(user_password);
  free(target_table);
  free(target_table_export);
  free(tdp_id);
  free(log_table);
  free(error_table_1);
  free(error_table_2);
  free(ins_statement);
  free(sel_statement);
  free(msg);

  cout << endl << "*** Export to PutRow Complete ***" << endl;
  cout <<"\n######################## OUTPUT END ####################" << endl;

}


/*************************** Utility Functions ****************************************/

/***********************************************
* Function prints string version of status code
************************************************/
void GenericDriver::printStatus(const char* msg, int code)    //TPT-17718
{
  cout << msg;

  switch(code)
  {
    case TD_Success:
         cout << "TD_Success";
         break;
    case TD_SYNC_Barrier:
         cout << "TD_SYNC_Barrier";
         break;
    case TD_SYNC_TELINFO:
         cout << "TD_SYNC_TELINFO";
         break;
    case TD_END_Method:
         cout << "TD_END_Method";
         break;
    case TD_Unavailable:
         cout << "TD_Unavailable";
         break;
    case TD_Error:
         cout << "Error Code: TD_Error";
         break;
    default:
     cout << "Error Code: " << code;
         break;
  }
  cout << endl;
}

/***************************************
* Function sets input file name
***************************************/
void GenericDriver::setInput(char* fname)
{
  if( inputFileName != NULL )
  {
    delete inputFileName;
  }
  inputFileName = strdup(fname);
}

/***************************************
* Function sets output file name
***************************************/
void GenericDriver::setOutput(char* fname)
{
  if( outputFileName != NULL )
  {
    delete outputFileName;
  }
  outputFileName = strdup(fname);
}

/***************************************
* Function reads in a row of data
***************************************/
int GenericDriver::GetRowData(char* rowBuffer, size_t rowLength)
{
  unsigned short rlength;

  //Close File if End Signal Received
  if ( rowLength <= 0 )
  {
    if ( inputStream != NULL )
    {
      fclose(inputStream);
    }
    return -1;
  }

  //Open File if not already open
  if( inputStream == NULL )
  {
    inputStream = fopen(inputFileName, "rb");

    if( inputStream == NULL )
    {
      cout << "Error opening file " << inputFileName << endl;
      return -1;
    }
  }

  //Read  the lenght of each record..read first two bytes
  size_t recLen =fread(rowBuffer, 1, (size_t)2, inputStream);

  if ( recLen < 1 )
  {
    if ( feof(inputStream) )
    {
      cout << "End of file reached" << endl;
      if( inputStream != NULL )
      {
        fclose(inputStream);
      }
      return -1;
    }
    else
    {
      /* return some read error here and exit */
      cout << "read error" << endl;
      if( inputStream != NULL )
      {
        fclose(inputStream);
      }
      return -1;
    }
  }
  //read row of data with size rlength

  memcpy(&rlength, rowBuffer, sizeof(unsigned short));   /* DR124305 */
  size_t recoLen = fread(rowBuffer+2,1,(size_t)rlength+1,inputStream);
  return 0;
}


/*****************************************************
* Function writes out a row of data in indicator mode
******************************************************/
int GenericDriver::WriteRowData(char* data, TD_Length dataLen)
{
 if( data != NULL )
  {
    //Open File if not already open
    if( outputStream == NULL )
    {
      outputStream = fopen(outputFileName, "wb");

      if( outputStream == NULL )
      {
            cout << "Error opening file " << outputFileName << endl;
            return -1;
      }
    }

    unsigned long i = 0;
    unsigned short shortdataLen = (unsigned short)dataLen;
    char* rowBuffer = (char*)malloc(dataLen+20);

    memcpy(rowBuffer, &shortdataLen, 2);

    for(i = 0; i < dataLen; i++)
    {
      rowBuffer[i+2] = data[i];
    }
    rowBuffer[i+2] = 0x0a;

    //fprintf (stream, "%s\n",rowBuffer+1); // account for indicator byte
    fwrite(rowBuffer, dataLen+3, 1, outputStream);

    free(rowBuffer);
  }
  else
  {
    if( outputStream != NULL )
    {
      fclose(outputStream);
    }
    return -1;
  }
  return 0;
}

/*****************************************************
* Function writes out a buffer of data in indicator mode
******************************************************/
int GenericDriver::WriteBufferData(char* data, TD_Length dataLen)
{
  if( data != NULL )
  {
    //Open File if not already open
    if( outputStream == NULL )
    {
      outputStream = fopen(outputFileName, "wb");

      if( outputStream == NULL )
      {
        cout << "Error opening file " << outputFileName << endl;
        return -1;
      }
    }
    fwrite(data, dataLen, 1, outputStream);
  }
  else
  {

    {
      fclose(outputStream);
    }
    return -1;
  }
  return 0;
}



