/*************************************************************************
*
* Copyright 2010, 2013-2014 by Teradata Corporation.
* All Rights Reserved.
* TERADATA CONFIDENTIAL AND TRADE SECRET
*
* This copyrighted material is the Confidential, Unpublished
* Property of the Teradata Corporation.  This copyright notice and
* any other copyright notices included in machine readable
* copies must be reproduced on all authorized copies.
*
* History Information:
*
* Revision     Date      DR      DID       Comments
* ---------------------------------------------------------------------
* 15.00.00.00  12262013 TPT20211 kb186006  Copyright year change
* 14.00.00.01  10292010  144824  mv121803  Suppoprt MVS
* 13.00.00.01  06182010  142720  mv121803  Dynamically load TELAPI
*
*************************************************************************/

#ifndef __MVS__
  #include "ThreadTestParameters.h"
  #include "dynlib.h"                         //DR142720
#else
  #include "PTCTHTPD.H"                       //DR144824
  #include "PTCDLLIB.H"                       //DR144824
#endif//__MVS__


ThreadTestParameters::ThreadTestParameters(TD_OperatorType oper, 
int numInstances)
{
	//Initialize Variables
	mSourceFileName = NULL;
	mRowSize = 0;
	mRestartAttempts = 0;
	mCheckpointInterval = 0;
	mNumThreads = numInstances;
	mOperType = oper;
	mBufferModeEnabled = 0;
	mBufferRowCountLimit = 0;

	//Add Instance Attributes
	char* maxThreads = (char*)malloc(32);
	sprintf(maxThreads,"%d",numInstances);
	AddAttribute(TD_MAX_INSTANCES,1,maxThreads);
	mTraceLevel = TD_OFF;
	mCreateNewConnection = 0;
	free(maxThreads);
}

ThreadTestParameters::~ThreadTestParameters()
{
        DynamicLibrary  *lib_hdl = DynamicLibrary::Instance();     //DR142720

	if ( mSourceFileName != NULL )
	{
		free(mSourceFileName);
	}

	if ( !mAttributeValues.empty() )
	{
		vector<char*>::iterator iter = mAttributeValues.begin();

		for(; iter != mAttributeValues.end(); iter++ )
		{	
			char* value = *iter;
			free(value);
		}
	}

	if ( !mSchemaList.empty() )
	{
		vector<SchemaX*>::iterator iter = mSchemaList.begin();

		for(; iter != mSchemaList.end(); iter++ )
		{	
			SchemaX* schema = *iter;           //DR142720
			//delete schema;
                        lib_hdl->delSchemaX(schema);       //DR142720
		}
	}

	if ( !mDMLGroupList.empty() )
	{
		vector<DMLGroupX*>::iterator iter = mDMLGroupList.begin();

		for(; iter != mDMLGroupList.end(); iter++ )
		{	
			DMLGroupX* dmlGr = *iter;          //DR142720
			//delete dmlGr;
                        lib_hdl->delDMLGroupX(dmlGr);      //DR142720 
		}
	}

}

/**
 * Store a single attribute and all of its values
 */
void ThreadTestParameters::AddAttribute(TD_Attribute attrType,
 int valCount, const char* val1, ... )                    //TPT-17718
{
		va_list args;
		va_start(args,val1);
		char* value = strdup(val1);

		mAttributeTypes.push_back(attrType);
		mAttributeValueCount.push_back(valCount);
		mAttributeValues.push_back(value);

		for( int i = 1; i < valCount; i++)
		{
			value = strdup(va_arg(args,char*));
			mAttributeValues.push_back(value);
		}

		va_end(args);
}

/**
 * Store all attributes dealing with the logon
 */
void ThreadTestParameters::SetLogon(char* userName, char* userpwd,
 char* tdpid)
{
	AddAttribute(TD_USER_NAME,1,userName);
	AddAttribute(TD_USER_PASSWORD,1,userpwd);
	AddAttribute(TD_TDP_ID,1,tdpid);
}

/**
 * Store all attributes dealing with external files
 ( source/destination )
 */
void ThreadTestParameters::SetFile(char* name)
{
	mSourceFileName = (char *)malloc(256);
	strcpy(mSourceFileName,name);
	mRowSize = 0;
}


/**
 * Store all attributes dealing with Tables
 *	- Target Tables
 *	- Error Tables
 *	- Log Tables
 *	- Work Tables
 */
void ThreadTestParameters::SetTable(int tableCount,
 char* tableName,...)
{
	va_list args;
	va_start(args,tableName);
	vector<char*> tableNames;

	// Temporarily store arguments in a vector
	tableNames.push_back(tableName);

	for( int i = 1; i < tableCount; i++)
	{
		tableNames.push_back(va_arg(args,char*));
	}

	va_end(args);

	vector<char*>::iterator iter;

	// Add Table Target
	mAttributeTypes.push_back(TD_TARGET_TABLE);
	mAttributeValueCount.push_back(tableCount);

	for( iter = tableNames.begin(); iter !=
 tableNames.end(); iter++ )
	{
		char* tname = strdup(*iter);
		mAttributeValues.push_back(tname);
	}

    // Only Stream operator use TD_ERROR_TABLE
    if ( mOperType == TD_STREAM ){
        // Add Error Table for STREAM operator
        mAttributeTypes.push_back(TD_ERROR_TABLE);
        mAttributeValueCount.push_back(tableCount);
        char* errorStream = NULL;

        for( iter = tableNames.begin(); iter != 
tableNames.end(); iter++ )
        {
            // Add "_error" to the end of table name
            char* tname = *iter;
            errorStream = (char*)malloc(256);
            sprintf(errorStream,"%s_error",tname);
            mAttributeValues.push_back(errorStream);
        }
    }
    else {
        // Add Error Table 1
        mAttributeTypes.push_back(TD_ERROR_TABLE_1);
        mAttributeValueCount.push_back(tableCount);
        char* error1 = NULL;

        for( iter = tableNames.begin(); iter != 
tableNames.end(); iter++ )
        {
            // Add "_e1" to the end of table name
            char* tname = *iter;
            error1 = (char*)malloc(256);
            sprintf(error1,"%s_e1",tname);
            mAttributeValues.push_back(error1);
        }

        // Add Error Table 2
        mAttributeTypes.push_back(TD_ERROR_TABLE_2);
        mAttributeValueCount.push_back(tableCount);
        char* error2 = NULL;

        for( iter = tableNames.begin(); iter != 
tableNames.end(); iter++ )
        {
            // Add "_e2" to the end of table name
            char* tname = *iter;
            error2 = (char*)malloc(256);
            sprintf(error2,"%s_e2",tname);
            mAttributeValues.push_back(error2);
        }
    }

	// Add Log Table
	// We'll only add deriving the log table
	// name from the first table name
	mAttributeTypes.push_back(TD_LOG_TABLE);
	mAttributeValueCount.push_back(1);
	iter = tableNames.begin();

	// Add "_log" to the end of table name
	char* log = (char*)malloc(256);
	char* tname = *iter;
	sprintf(log,"%s_log",tname);
	mAttributeValues.push_back(log);
	

	// Add Work Table - Update Only
	if ( mOperType == TD_UPDATE ){

		mAttributeTypes.push_back(TD_WORK_TABLE);
		mAttributeValueCount.push_back(tableCount);
		char* wt = NULL;

		for( iter = tableNames.begin(); iter !=
 tableNames.end(); iter++ )
		{
			// Add "_wt" to the end of table name
			char* tname = *iter;
			wt = (char*)malloc(256);
			sprintf(wt,"%s_wt",tname);
			mAttributeValues.push_back(wt);
		}
	}

}


/**
 * Store all attributes dealing with sessions
 */
void ThreadTestParameters::SetSessions(int max, int min)
{
	char* maxSessions = (char*)malloc(32);
	char* minSessions = (char*)malloc(32);

	sprintf(maxSessions,"%d",max);
	sprintf(minSessions,"%d",min);

	AddAttribute(TD_MAX_SESSIONS,1,maxSessions);
	AddAttribute(TD_MIN_SESSIONS,1,minSessions);

	free(maxSessions);
	free(minSessions);
}

/**
 * Store all attributes dealing with sessions
 */
void ThreadTestParameters::SetCheckpoint(int checkPointInterval,
 int restartAttempts )
{
	//Set Checkpoint Interval
	mCheckpointInterval = checkPointInterval;

	//Set Restart Attempts
	mRestartAttempts = restartAttempts;

	mCreateNewConnection = 1;

	if( mRestartAttempts > 0 && !mCreateNewConnection ){
		AddAttribute(TD_RESTARTMODE,1,"1");
	}
}

void ThreadTestParameters::SetTracing(TD_TraceLevel level)
{
	mTraceLevel = level;
}

void ThreadTestParameters::SetFail(int instance, int rowCount)
{
	mInstanceFail.push_back(instance);
	mRowCountFail.push_back(rowCount);
	mDidFail.push_back(0);
}

void ThreadTestParameters::SetRowLimit(int instance, int rowCount)
{
	mInstanceLimit.push_back(instance);
	mRowCountLimit.push_back(rowCount);
}

void ThreadTestParameters::SetBufferMode(int enable, int rowsPerBuffer)
{
	mBufferModeEnabled = enable;
	mBufferRowCountLimit = rowsPerBuffer;
}








