/*******************************************
*
* Copyright 2006-2009, 2013-2014 by Teradata Corporation.
* All Rights Reserved.
* TERADATA CONFIDENTIAL AND TRADE SECRET
*
* This copyrighted material is the Confidential, Unpublished
* Property of the Teradata Corporation.  This copyright notice and
* any other copyright notices included in machine readable
* copies must be reproduced on all authorized copies.
*
* Description:
*  
* GenericDriver handles the real logic for the XSP. It
* is very similar to the application GenericDriver sample.
* Please run through that to get familiar with Teradata
* PT API. Specific difference is the way schema is constructed
* by OptionsReader instead of OptionsManager, to reduce the need
* for recompilation. Make sure to read OptionsManager where
* the schema is parsed, and modify as appropiate for a different
* schema if the number of columns are different from this sample.
* Another difference is the way error is relayed to user. 
* There is no standard output when running as an XSP. One can
* either write to own file, write error string into error_message
* parameter or write to trace table. 
* The testExport2X functions are similar to the GetBuffer sample.
* See the GetBuffer for more detailed explanation. This XSP 
* sample uses the GetBuffer feature to load data from one DBS to
* another
*
*/

#ifndef __BUILD_APP
#define SQL_TEXT Latin_Text
#include <sqltypes_td.h>

#include <stdlib.h>
#include <string.h>
#include <errno.h>
#endif

#include "GenericDriver.h"

#include "connection.h"

extern "C"
void debug_trace(const char * instring);

using namespace teradata::client::API;

GenericDriver::GenericDriver(OptionsReader *om)
: inputStream(NULL), inputFileName(NULL), 
  outputFileName(NULL), outputStream(NULL), m_om(om)
{
}


GenericDriver::GenericDriver(OptionsReader *om, 
			     OptionsReader *omload)
: inputStream(NULL), inputFileName(NULL), 
  outputFileName(NULL), outputStream(NULL), 
  m_om(om), m_omload(omload)
{
}


GenericDriver::~GenericDriver()
{
	if (inputFileName != NULL) {
		free(inputFileName);
	}
	if (outputFileName != NULL) {
		free(outputFileName);
	}
}


void GenericDriver::testLoad(char *sqlstate, SQL_TEXT *error_message)
{
	int returnValue = 0;
	char * errorMsg = NULL;
	TD_ErrorType errorType;

	debug_trace("*** Load Driver Example ***");

	TD_Index dmlGroupIndex=0;

	// create api objects
	Connection * conn = new Connection();
	Schema *schema = new Schema((char*)"input");
	DMLGroup *dmlGr = new DMLGroup();

	// option file contains connection attributes,
	// table schema definition, dml statements
	// option reader reads all that and add to the api objects
	if (m_om->readOptionsFromFile(conn, schema, dmlGr, 
				      sqlstate, error_message) == -100) {
	  delete dmlGr;
	  delete schema;
	  delete conn;
	  return;
	}

	conn->AddAttribute(TD_SYSTEM_OPERATOR,TD_LOAD);
	// uncomment to create trace file
	//conn->AddAttribute(TD_TRACE_OUTPUT,(char*)"/xsptest/load.txt");
	//conn->AddArrayAttribute(TD_TRACE_LEVEL,2,TD_OPER_ALL,TD_GENERAL,NULL);
	conn->AddSchema(schema);
	conn->AddDMLGroup(dmlGr,&dmlGroupIndex);

	returnValue = conn->Initiate();

	//cout << "Driver initiated with status " << returnValue << endl;

	if ( returnValue < TD_Error )
	{
		// Acquistion
		char rowBuffer[104];

		int loadStatus = 0;
		TD_Length bytes;
		int count = 0;

		while (loadStatus != -1)
		{
			loadStatus = GetRowData(rowBuffer, 104);

			if ( loadStatus != -1 )
			{
				// pick up the first 2 bytes as the row length of the indicator mode record
				bytes = *((unsigned short *)rowBuffer);
				returnValue = conn->PutRow( rowBuffer + 2, bytes );

				if (returnValue >= TD_Error)
				{
					debug_trace("PutRow failed");
					loadStatus = GetRowData(NULL, 0);// abort load
				}
				else
				{
					count++;
				}
			}

		}

		if (returnValue < TD_Error) {

			// End Acquisition
			returnValue = conn->EndAcquisition();
			debug_trace("End Acquistion");

			if (returnValue < TD_Error) {

				// Application
				returnValue = conn->ApplyRows();
				debug_trace("Apply Rows");

				if (returnValue < TD_Error) {
					debug_trace("Load completed successfully");
				}
				else {
					debug_trace("Error occured during Application");
					conn->GetErrorInfo(&errorMsg,&errorType);
					strcpy(sqlstate,"U0005");
					if ( errorMsg != NULL ){
					  //cout << "Type: " << errorType << endl;
					  if (strlen(errorMsg) > 256) {
					    strncpy((char *)error_message,errorMsg,256);
					    sqlstate[256] = '\0';
					  }
					  else
					    strcpy((char *)error_message,errorMsg);
					  
					}else{
					  strcpy((char *)error_message,"No Error Info Available");
					}
				}
			}
			else {
				debug_trace("Error occured during EndAcquisition");
				strcpy(sqlstate,"U0004");
				conn->GetErrorInfo(&errorMsg,&errorType);
				if ( errorMsg != NULL ){
				  //cout << "Type: " << errorType << endl;
				  if (strlen(errorMsg) > 256) {
				    strncpy((char *)error_message,errorMsg,256);
				    sqlstate[256] = '\0';
				  }
				  else
				    strcpy((char *)error_message,errorMsg);
				  
				}else{
				  strcpy((char *)error_message,"No Error Info Available");
				}
			}
		}
		else {
			debug_trace("Error occured during Acquisition");
			strcpy(sqlstate,"U0003");
			conn->GetErrorInfo(&errorMsg,&errorType);
			if ( errorMsg != NULL ){
			  //cout << "Type: " << errorType << endl;
			  if (strlen(errorMsg) > 256) {
			    strncpy((char *)error_message,errorMsg,256);
			    sqlstate[256] = '\0';
			  }
			  else
			    strcpy((char *)error_message,errorMsg);
			  
			}else{
			  strcpy((char *)error_message,"No Error Info Available");
			}
		}

	}
	else
	{
		debug_trace("Error occured during Initiate");
		strcpy(sqlstate,"U0002");
		conn->GetErrorInfo(&errorMsg,&errorType);
		if ( errorMsg != NULL ){
			//cout << "Type: " << errorType << endl;
			if (strlen(errorMsg) > 256) {
				strncpy((char *)error_message,errorMsg,256);
				sqlstate[256] = '\0';
			}
			else
				strcpy((char *)error_message,errorMsg);

		}else{
			strcpy((char *)error_message,"No Error Info Available");
		}
	}


	returnValue = conn->Terminate();

	//cout << "Driver Terminated with status " << returnValue << endl;

	if ( returnValue >= TD_Error )
	{  
		//Get Error Information
		debug_trace("Error occured during Terminate");
         	strcpy(sqlstate,"U0006");
		conn->GetErrorInfo(&errorMsg,&errorType);
		if ( errorMsg != NULL ){
		  //cout << "Type: " << errorType << endl;
		  if (strlen(errorMsg) > 256) {
		    strncpy((char *)error_message,errorMsg,256);
		    sqlstate[256] = '\0';
		  }
		  else
		    strcpy((char *)error_message,errorMsg);
		  
		}else{
		  strcpy((char *)error_message,"No Error Info Available");
		}
	}

	delete dmlGr;
	delete schema;
	delete conn;

}

void GenericDriver::testExport(char *sqlstate, SQL_TEXT *error_message)
{
	int returnValue = 0;
	char * errorMsg = NULL;
	TD_ErrorType errorType;

	debug_trace("*** Export Driver Example ***");

	// create api objects, no dml group needed 
	Connection *conn = new Connection();
	Schema *schema = new Schema((char*)"output");  //TPT-17718

	// option file contains connection attributes,
	// table schema definition, dml statements
	// option reader reads all that and add to the api objects
	// note input file select statement is an attribute for export
	m_om->readOptionsFromFile(conn, schema, NULL, sqlstate, error_message, NULL);

	conn->AddAttribute(TD_SYSTEM_OPERATOR,TD_EXPORT);
	// uncomment to create trace file
	//conn->AddAttribute(TD_TRACE_OUTPUT,(char*)"/xsptest/export.txt");  //TPT-17718
	//conn->AddArrayAttribute(TD_TRACE_LEVEL,2,TD_OPER_ALL,TD_GENERAL);

	conn->AddSchema(schema);


	returnValue = conn->Initiate();

	//cout << "Driver initiated with status " << returnValue << endl;

	if ( returnValue < TD_Error )
	{
		// Acquistion
		char *data;
		int exportStatus = 0;
		TD_Length dataLen = 0;
		int count = 0;

		while (exportStatus != -1)
		{
			returnValue = conn->GetRow(&data, &dataLen);

			if ( returnValue >= TD_Error )
			{
				//cout << "GetRow failed on row " << count+1 << " with status " << returnValue << endl;
				debug_trace("GetRow failed");
				exportStatus = WriteRowData(NULL,0); //Abort write

			}else if ( returnValue == TD_END_Method ){
				//cout << "End of Data Reached" << endl;
				debug_trace("End of Data Reached");
				exportStatus = WriteRowData(NULL,0); //end write
				count++;
			}else{
				exportStatus = WriteRowData(data,dataLen);
				count++;
			}

		}

		if (returnValue < TD_Error) {
			debug_trace("Export completed successfully");
		}
		else {
			debug_trace("Error occured during Export");
			strcpy(sqlstate,"U0007");
			conn->GetErrorInfo(&errorMsg,&errorType);
			if ( errorMsg != NULL ){
			  //cout << "Type: " << errorType << endl;
			  if (strlen(errorMsg) > 256) {
			    strncpy((char *)error_message,errorMsg,256);
			    sqlstate[256] = '\0';
			  }
			  else
			    strcpy((char *)error_message,errorMsg);
			  
			}else{
			  strcpy((char *)error_message,"No Error Info Available");
			}
		}

	}
	else
	{
		debug_trace("Error occured during Initiate");
		strcpy(sqlstate,"U0002");
		conn->GetErrorInfo(&errorMsg,&errorType);
		if ( errorMsg != NULL ){
		  //cout << "Type: " << errorType << endl;
		  if (strlen(errorMsg) > 256) {
		    strncpy((char *)error_message,errorMsg,256);
		    sqlstate[256] = '\0';
		  }
		  else
		    strcpy((char *)error_message,errorMsg);
		  
		}else{
		  strcpy((char *)error_message,"No Error Info Available");
		}
	}


	returnValue = conn->Terminate();

	//cout << "Driver Terminated with status " << returnValue << endl;

	if ( returnValue >= TD_Error )
	{  
		//Get Error Information
	        debug_trace("Error occured during Terminate");
	        strcpy(sqlstate,"U0006");
		conn->GetErrorInfo(&errorMsg,&errorType);
		if ( errorMsg != NULL ){
		  //cout << "Type: " << errorType << endl;
		  if (strlen(errorMsg) > 256) {
		    strncpy((char *)error_message,errorMsg,256);
		    sqlstate[256] = '\0';
		  }
		  else
		    strcpy((char *)error_message,errorMsg);
		  
		}else{
		  strcpy((char *)error_message,"No Error Info Available");
		}
	}

	delete schema;
	delete conn;

	debug_trace("*** Export Complete ***");
}


void GenericDriver::testUpdate(char *sqlstate, SQL_TEXT *error_message)
{
	int returnValue = 0;
	char * errorMsg = NULL;
	TD_ErrorType errorType;

	// update sample does things a little differently to 
	// demostrate what happens with duplicate rows
	char * dml_statmentA = strdup("INSERT INTO tdexportA( :Associate_Id, :Associate_Name, :Salary, :DOJ, :Designation, :Loan_Amount, :Martial_Status, :No_Of_Dependents, CAST (:update_date AS TIMESTAMP(6) FORMAT 'YYYY-MM-DD hh:mi:ss'));");
	char * dml_statmentB = strdup("INSERT INTO tdexportB( :Associate_Id, :Associate_Name, :Salary, :DOJ, :Designation, :Loan_Amount, :Martial_Status, :No_Of_Dependents, CAST (:update_date AS TIMESTAMP(6) FORMAT 'YYYY-MM-DD hh:mi:ss'));");

	debug_trace("*** Update Driver Example ***");

	TD_Index dmlGroupIndex[2];

	// create api objects
	Connection * conn = new Connection();
	Schema *schema = new Schema((char*)"input");  //TPT-17718
	DMLGroup *dmlGr = new DMLGroup();

	// option file contains connection attributes,
	// table schema definition
	// option reader reads all that and add to the api objects
	// since this sample is a little different, the dml statement
	// is kept in code. note the input file is different from others
	// than others to define 2 target tables
	m_om->readOptionsFromFile(conn, schema, NULL, sqlstate, error_message, NULL);

	conn->AddAttribute(TD_SYSTEM_OPERATOR,TD_UPDATE);
	// uncomment to create trace file
	//conn->AddArrayAttribute(TD_TRACE_LEVEL,2,TD_OPER_ALL,TD_GENERAL);
	//conn->AddAttribute(TD_TRACE_OUTPUT,(char*)"/xsptest/update.txt");

	conn->AddSchema(schema);

	// note here that one table is set to mark duplicate rows
	// when this sample is done, there will be an error left
	// with the duplicate rows
	dmlGr->AddStatement(dml_statmentA);
	dmlGr->AddDMLOption(MARK_DUPLICATE_ROWS);
	conn->AddDMLGroup(dmlGr,&dmlGroupIndex[0]);
	delete dmlGr;

	dmlGr = new DMLGroup();
	dmlGr->AddStatement(dml_statmentB);
	dmlGr->AddDMLOption(IGNORE_DUPLICATE_ROWS);
	returnValue = conn->AddDMLGroup(dmlGr,&dmlGroupIndex[1]);
	delete dmlGr;


	returnValue = conn->Initiate();

	//cout << "Driver initiated with status " << returnValue << endl;

	if ( returnValue < TD_Error )
	{
		// Acquistion
		char rowBuffer[104];
		int loadStatus = 0;
		TD_Length bytes;
		int count = 0;

		while (loadStatus != -1)
		{
			loadStatus = GetRowData(rowBuffer, 104);
			//debug_trace("Get Row!");

			if ( loadStatus != -1 )
			{
				// pick up the first 2 bytes as the row length of the indicator mode record
				bytes = *((unsigned short *)rowBuffer);

				// same row is sent to both tables
				for (int i = 0; i < 2; i++) {
					returnValue = conn->UseDMLGroups(&dmlGroupIndex[i],1);
					if (returnValue >= TD_Error) {
						break;
					}
					debug_trace("Sending First Row");
					returnValue = conn->PutRow( rowBuffer + 2, bytes );
					if (returnValue >= TD_Error) {
						break;
					}
					debug_trace("Sending Duplicate Row");
					returnValue = conn->PutRow( rowBuffer + 2, bytes );

					if (returnValue >= TD_Error)
					{
						debug_trace("PutRow failed");
						loadStatus = GetRowData(NULL, 0);// abort load
					}
					else
					{
						count++;
					}
				}
			}

		}

		if (returnValue < TD_Error) {

			// End Acquisition
			returnValue = conn->EndAcquisition();
			debug_trace("End Acquistion");

			if (returnValue < TD_Error) {

				// Application
				returnValue = conn->ApplyRows();
				debug_trace("Apply Rows");

				if (returnValue < TD_Error) {
					debug_trace("Update completed successfully");
				}
				else {
					debug_trace("Error occured during Application");
					strcpy(sqlstate,"U0005");
					conn->GetErrorInfo(&errorMsg,&errorType);
					if ( errorMsg != NULL ){
					  //cout << "Type: " << errorType << endl;
					  if (strlen(errorMsg) > 256) {
					    strncpy((char *)error_message,errorMsg,256);
					    sqlstate[256] = '\0';
					  }
					  else
					    strcpy((char *)error_message,errorMsg);
					  
					}else{
					  strcpy((char *)error_message,"No Error Info Available");
					}
				}
			}
			else {
				debug_trace("Error occured during EndAcquisition");
				strcpy(sqlstate,"U0004");
				conn->GetErrorInfo(&errorMsg,&errorType);
				if ( errorMsg != NULL ){
				  //cout << "Type: " << errorType << endl;
				  if (strlen(errorMsg) > 256) {
				    strncpy((char *)error_message,errorMsg,256);
				    sqlstate[256] = '\0';
				  }
				  else
				    strcpy((char *)error_message,errorMsg);
				  
				}else{
				  strcpy((char *)error_message,"No Error Info Available");
				}
			}
		}
		else {
			debug_trace("Error occured during Acquisition");
			strcpy(sqlstate,"U0003");
			conn->GetErrorInfo(&errorMsg,&errorType);
			if ( errorMsg != NULL ){
			  //cout << "Type: " << errorType << endl;
			  if (strlen(errorMsg) > 256) {
			    strncpy((char *)error_message,errorMsg,256);
			    sqlstate[256] = '\0';
			  }
			  else
			    strcpy((char *)error_message,errorMsg);
			  
			}else{
			  strcpy((char *)error_message,"No Error Info Available");
			}
		}
	}
	else
	{
		debug_trace("Error occured during Initiate");
		strcpy(sqlstate,"U0002");
		conn->GetErrorInfo(&errorMsg,&errorType);
		if ( errorMsg != NULL ){
			//cout << "Type: " << errorType << endl;
			if (strlen(errorMsg) > 256) {
				strncpy((char *)error_message,errorMsg,256);
				sqlstate[256] = '\0';
			}
			else
				strcpy((char *)error_message,errorMsg);

		}else{
			strcpy((char *)error_message,"No Error Info Available");
		}
	}


	returnValue = conn->Terminate();

	//cout << "Driver Terminated with status " << returnValue << endl;

	if ( returnValue >= TD_Error )
	{  
		//Get Error Information
		debug_trace("Error occured during Terminate");
		conn->GetErrorInfo(&errorMsg,&errorType);
		strcpy(sqlstate,"U0006");
		if ( errorMsg != NULL ){
		  //cout << "Type: " << errorType << endl;
		  if (strlen(errorMsg) > 256) {
		    strncpy((char *)error_message,errorMsg,256);
		    sqlstate[256] = '\0';
		  }
		  else
		    strcpy((char *)error_message,errorMsg);
		  
		}else{
		  strcpy((char *)error_message,"No Error Info Available");
		}
	}

	delete schema;
	delete conn;

}


void GenericDriver::testStream(char *sqlstate, SQL_TEXT *error_message)
{
	int returnValue = 0;
	char * errorMsg = NULL;
	TD_ErrorType errorType;
	TD_Index dmlGroupIndex=0;

	debug_trace("*** Stream Driver Example ***");

	// create api objects
	Connection * conn = new Connection();
	Schema *schema = new Schema((char*)"input");  //TPT-17718
	DMLGroup* dmlGr = new DMLGroup();

	// option file contains connection attributes,
	// table schema definition, dml statements
	// option reader reads all that and add to the api objects
	// note the extra SERIALIZE_ON and USE_LIST for dml group
	m_om->readOptionsFromFile(conn,schema,dmlGr,sqlstate,error_message);

	conn->AddAttribute(TD_SYSTEM_OPERATOR,TD_STREAM);
	// uncomment to create trace file
	//conn->AddAttribute(TD_TRACE_OUTPUT,(char*)"/xsptest/stream.txt");
	//conn->AddArrayAttribute(TD_TRACE_LEVEL,2,TD_OPER_ALL,TD_GENERAL,NULL);

	conn->AddSchema(schema);
	conn->AddDMLGroup(dmlGr,&dmlGroupIndex);

	returnValue = conn->Initiate();

	//cout << "Driver initiated with status " << returnValue << endl;

	if ( returnValue < TD_Error )
	{
		// Acquistion
		char rowBuffer[104];
		int loadStatus = 0;
		TD_Length bytes;
		int count = 0;

		while (loadStatus != -1)
		{
			loadStatus = GetRowData(rowBuffer, 104);

			if ( loadStatus != -1 )
			{
				// pick up the first 2 bytes as the row length of the indicator mode record
				bytes = *((unsigned short *)rowBuffer);
				returnValue = conn->PutRow( rowBuffer + 2, bytes );

				if (returnValue >= TD_Error)
				{
					debug_trace("PutRow failed");
					loadStatus = GetRowData(NULL, 0);// abort load
				}
				else
				{
					count++;
				}
			}

		}


		if (returnValue < TD_Error) {

			// End Acquisition
			returnValue = conn->EndAcquisition();

			// no ApplyRows for Stream

			if (returnValue < TD_Error) {
				debug_trace("Stream completed successfully");
			}
			else {
				debug_trace("Error occured during EndAcquisition");
				strcpy(sqlstate,"U0004");
				conn->GetErrorInfo(&errorMsg,&errorType);
				if ( errorMsg != NULL ){
				  //cout << "Type: " << errorType << endl;
				  if (strlen(errorMsg) > 256) {
				    strncpy((char *)error_message,errorMsg,256);
				    sqlstate[256] = '\0';
				  }
				  else
				    strcpy((char *)error_message,errorMsg);
				  
				}else{
				  strcpy((char *)error_message,"No Error Info Available");
				}
			}
		}
		else {
			debug_trace("Error occured during Acquisition");
			strcpy(sqlstate,"U0003");
			conn->GetErrorInfo(&errorMsg,&errorType);
			if ( errorMsg != NULL ){
			  //cout << "Type: " << errorType << endl;
			  if (strlen(errorMsg) > 256) {
			    strncpy((char *)error_message,errorMsg,256);
			    sqlstate[256] = '\0';
			  }
			  else
			    strcpy((char *)error_message,errorMsg);
			  
			}else{
			  strcpy((char *)error_message,"No Error Info Available");
			}
		}
	}
	else
	{
		debug_trace("Error occured during Initiate");
		strcpy(sqlstate,"U0002");
		conn->GetErrorInfo(&errorMsg,&errorType);
		if ( errorMsg != NULL ){
			//cout << "Type: " << errorType << endl;
			if (strlen(errorMsg) > 256) {
				strncpy((char *)error_message,errorMsg,256);
				sqlstate[256] = '\0';
			}
			else
				strcpy((char *)error_message,errorMsg);

		}else{
			strcpy((char *)error_message,"No Error Info Available");
		}
	}


	returnValue = conn->Terminate();

	//cout << "Driver Terminated with status " << returnValue << endl;

	if ( returnValue >= TD_Error )
	{  
		//Get Error Information
		debug_trace("Error occured during Terminate");
	        strcpy(sqlstate,"U0006");
		conn->GetErrorInfo(&errorMsg,&errorType);
		if ( errorMsg != NULL ){
		  //cout << "Type: " << errorType << endl;
		  if (strlen(errorMsg) > 256) {
		    strncpy((char *)error_message,errorMsg,256);
		    sqlstate[256] = '\0';
		  }
		  else
		    strcpy((char *)error_message,errorMsg);
		  
		}else{
		  strcpy((char *)error_message,"No Error Info Available");
		}
	}

	delete dmlGr;
	delete schema;
	delete conn;
}


/**************************************************
 * testExport2Load
 *
 * Tests transferring data from Export's GetBuffer
 * to Load's PutBuffer using an XSP. 
 * See the GetBuffer sample for complete step-by-
 * step explanations.
 ***************************************************/
void GenericDriver::testExport2Load(char *sqlstate, SQL_TEXT *error_message, VARCHAR_LATIN *updatetime)
{
	int returnValue = 0;
	char * errorMsg = NULL;
	TD_ErrorType errorType;
	TD_Index dmlGroupIndex = 0;

	BufferLayout layout; /* BufferLayout struct defined in GenericDriver.h */
	char *data;
	int exportStatus = 0;
	int loadStatus = 0;
	int queryStatus = 0;
	char *dataptr;
	TD_Length dataLen = 0;
	int count = 0;

	debug_trace("*** Export to Load Driver Example ***");

	// create api objects to export, no dml group needed 
	Connection *expConn = new Connection();
	Schema *expSchema = new Schema((char*)"output");  //TPT-17718

	// create api objects to load
	Connection * ldConn = new Connection();
	Schema *ldSchema = new Schema((char*)"input");  //TPT-17718
	DMLGroup *ldDmlGr = new DMLGroup();

	// option file contains connection attributes,
	// table schema definition, dml statements
	// option reader reads all that and add to the api objects
	// note input file select statement is an attribute for export
	if (m_om->readOptionsFromFile(expConn, expSchema, NULL, sqlstate, error_message,updatetime) == -100) {
	  delete expConn;
	  delete expSchema;
	  delete ldConn;
	  delete ldSchema;
	  delete ldDmlGr;
	  return;
	}
	expConn->AddAttribute(TD_SYSTEM_OPERATOR,TD_EXPORT);
	expConn->AddAttribute(TD_BUFFER_MODE,(char*)"Yes");  //TPT-17718
	// uncomment to create trace file
	//expConn->AddAttribute(TD_TRACE_OUTPUT,(char*)"/xsptest/e2l_export.txt");  //TPT-17718
        //expConn->AddArrayAttribute(TD_TRACE_LEVEL,2,TD_OPER_ALL,TD_GENERAL);
	expConn->AddSchema(expSchema);


	m_omload->readOptionsFromFile(ldConn, ldSchema, ldDmlGr, sqlstate, error_message);
	
	ldConn->AddAttribute(TD_SYSTEM_OPERATOR,TD_LOAD);
	ldConn->AddAttribute(TD_BUFFER_MODE,(char*)"Yes");  //TPT-17718
	// uncomment to create trace file
	//ldConn->AddAttribute(TD_TRACE_OUTPUT,(char*)"/xsptest/e2l_load.txt");
	//ldConn->AddArrayAttribute(TD_TRACE_LEVEL,2,TD_OPER_ALL,TD_GENERAL,NULL);
	ldConn->AddSchema(ldSchema);
	ldConn->AddDMLGroup(ldDmlGr,&dmlGroupIndex);
	

	returnValue = ldConn->Initiate();

	//cout << "Driver initiated with status " << returnValue << endl;

	if ( returnValue < TD_Error )
	{
	  /**********************************************
	   * Query Buffer Layout
	   **********************************************/
	  queryStatus = ldConn->GetEvent(TD_Evt_BufferLayout, &dataptr, &dataLen);
		
	  if ( queryStatus < TD_Error && queryStatus != TD_Unavailable ){
	    layout = *((BufferLayout*)dataptr);
	    //cout << "TD_Evt_BufferLayout: Max Buffer Size: ";
	    //cout << layout.MaxMsgSize << endl;
	    //cout << "TD_Evt_BufferLayout: Row Header Size: ";
	    //cout << layout.RowHeaderSize << endl;
	    //cout << "TD_Evt_BufferLayout: Row Length Size: ";
	    //cout << layout.RowLengthSize << endl;
	    //cout << "TD_Evt_BufferLayout: Buffer Trailer Size: ";
	    //cout << layout.BufTrailerSize << endl;
	  }else {
	    //printStatus("TD_Evt_BufferLayout event failed with status ",queryStatus);
	    debug_trace("TD_Evt_BufferLayout event failed");
	  }
	  
	  /* Give Buffer Layout to Export Driver */
	  expConn->AddAttribute(TD_BUFFER_MAX_SIZE,layout.MaxMsgSize);
	  expConn->AddAttribute(TD_BUFFER_LENGTH_SIZE,layout.RowLengthSize);
	  expConn->AddAttribute(TD_BUFFER_HEADER_SIZE,layout.RowHeaderSize);
	  expConn->AddAttribute(TD_BUFFER_TRAILER_SIZE,layout.BufTrailerSize);


	  returnValue = expConn->Initiate();
	  
	  if ( returnValue < TD_Error )
	    {
		debug_trace("Driver initiated");

		// Acquistion
		while (exportStatus != -1)
		{
		        debug_trace("GetRow");
			//returnValue = expConn->GetRow(&data, &dataLen);
			returnValue = expConn->GetBuffer(&data, &dataLen);

			if ( returnValue >= TD_Error )
			{
				//cout << "GetBuffer failed on row " << count+1 << " with status " << returnValue << endl;
				debug_trace("GetBuffer failed");
				exportStatus = -1; //Abort

			}else if ( returnValue == TD_END_Method ){
				debug_trace("End of Data Reached");
				exportStatus = -1; //end
				count++;

				// End Acquisition
				returnValue = ldConn->EndAcquisition();
				debug_trace("End Acquistion");

				if (returnValue < TD_Error) {
				  
				// Application
				  returnValue = ldConn->ApplyRows();
				  debug_trace("Apply Rows");
				  
				  if (returnValue < TD_Error) {
				    debug_trace("Load completed successfully");
				  }
				  else {
				    debug_trace("Error occured during Application");
				    ldConn->GetErrorInfo(&errorMsg,&errorType);
				    strcpy(sqlstate,"U0005");
				    if ( errorMsg != NULL ){
				      //cout << "Type: " << errorType << endl;
				      if (strlen(errorMsg) > 256) {
					strncpy((char *)error_message,errorMsg,256);
					sqlstate[256] = '\0';
				      }
				      else
					strcpy((char *)error_message,errorMsg);
				      
				    }else{
				      strcpy((char *)error_message,"No Error Info Available");
				    }
				  }
				}
				else {
				  debug_trace("Error occured during EndAcquisition");
				  strcpy(sqlstate,"U0004");
				  ldConn->GetErrorInfo(&errorMsg,&errorType);
				  if ( errorMsg != NULL ){
				    //cout << "Type: " << errorType << endl;
				    if (strlen(errorMsg) > 256) {
				      strncpy((char *)error_message,errorMsg,256);
				      sqlstate[256] = '\0';
				    }
				    else
				      strcpy((char *)error_message,errorMsg);
				    
				  }else{
				    strcpy((char *)error_message,"No Error Info Available");
				  }
				}

			}else{
			        debug_trace("PutRow");
			        returnValue = ldConn->PutBuffer( data, dataLen , 1);
				if (returnValue >= TD_Error)
				  {
				    debug_trace("PutRow failed");
				    exportStatus = -1; // abort
				  }
				else
				  {
				    exportStatus = 0; // continue
				    count++;
				  }
			}

		}

		if (returnValue < TD_Error) {
			debug_trace("Export completed successfully");
		}
		else {
			debug_trace("Error occured during Export");
			strcpy(sqlstate,"U0007");
			expConn->GetErrorInfo(&errorMsg,&errorType);
			if ( errorMsg != NULL ){
			  //cout << "Type: " << errorType << endl;
			  if (strlen(errorMsg) > 256) {
			    strncpy((char *)error_message,errorMsg,256);
			    sqlstate[256] = '\0';
			  }
			  else
			    strcpy((char *)error_message,errorMsg);
			  
			}else{
			  strcpy((char *)error_message,"No Error Info Available");
			}
		}
	    }
	  else
	    {
	      debug_trace("Error occured during Export Initiate");
	      strcpy(sqlstate,"U0012");
	      expConn->GetErrorInfo(&errorMsg,&errorType);
	      if ( errorMsg != NULL ){
		//cout << "Type: " << errorType << endl;
		if (strlen(errorMsg) > 256) {
		  strncpy((char *)error_message,errorMsg,256);
		  sqlstate[256] = '\0';
		}
		else
		  strcpy((char *)error_message,errorMsg);
		
	      }else{
		strcpy((char *)error_message,"No Error Info Available");
	      }
	    }
	}
	else
	{
		debug_trace("Error occured during Load Initiate");
		strcpy(sqlstate,"U0002");
		ldConn->GetErrorInfo(&errorMsg,&errorType);
		if ( errorMsg != NULL ){
		  //cout << "Type: " << errorType << endl;
		  if (strlen(errorMsg) > 256) {
		    strncpy((char *)error_message,errorMsg,256);
		    sqlstate[256] = '\0';
		  }
		  else
		    strcpy((char *)error_message,errorMsg);
		  
		}else{
		  strcpy((char *)error_message,"No Error Info Available");
		}
	}


	returnValue = expConn->Terminate();

	//cout << "Driver Terminated with status " << returnValue << endl;

	if ( returnValue >= TD_Error )
	{  
		//Get Error Information
	        debug_trace("Error occured during Export Terminate");
	        strcpy(sqlstate,"U0006");
		expConn->GetErrorInfo(&errorMsg,&errorType);
		if ( errorMsg != NULL ){
		  //cout << "Type: " << errorType << endl;
		  if (strlen(errorMsg) > 256) {
		    strncpy((char *)error_message,errorMsg,256);
		    sqlstate[256] = '\0';
		  }
		  else
		    strcpy((char *)error_message,errorMsg);
		  
		}else{
		  strcpy((char *)error_message,"No Error Info Available");
		}
	}

	returnValue = ldConn->Terminate();

	//cout << "Driver Terminated with status " << returnValue << endl;

	if ( returnValue >= TD_Error )
	{  
		//Get Error Information
	        debug_trace("Error occured during Load Terminate");
	        strcpy(sqlstate,"U0006");
		ldConn->GetErrorInfo(&errorMsg,&errorType);
		if ( errorMsg != NULL ){
		  //cout << "Type: " << errorType << endl;
		  if (strlen(errorMsg) > 256) {
		    strncpy((char *)error_message,errorMsg,256);
		    sqlstate[256] = '\0';
		  }
		  else
		    strcpy((char *)error_message,errorMsg);
		  
		}else{
		  strcpy((char *)error_message,"No Error Info Available");
		}
	}

	delete expSchema;
	delete expConn;
	delete ldDmlGr;
	delete ldSchema;
	delete ldConn;

	debug_trace("*** Export to Load Complete ***");
}


/**************************************************
 * testExport2Update
 *
 * Tests transferring data from Export's GetBuffer
 * to Update's PutBuffer using an XSP. 
 * See the GetBuffer sample for complete step-by-
 * step explanations.
 ***************************************************/
void GenericDriver::testExport2Update(char *sqlstate, 
				      SQL_TEXT *error_message, 
				      VARCHAR_LATIN *updatetime)
{
	int returnValue = 0;
	char * errorMsg = NULL;
	TD_ErrorType errorType;
	TD_Index dmlGroupIndex = 0;

	BufferLayout layout; /* BufferLayout struct defined in GenericDriver.h */
	char *data;
	int exportStatus = 0;
	int updateStatus = 0;
	int queryStatus = 0;
	char *dataptr;
	TD_Length dataLen = 0;
	int count = 0;

	debug_trace("*** Export to Update Driver Example ***");

	// create api objects to export, no dml group needed 
	Connection *expConn = new Connection();
	Schema *expSchema = new Schema((char*)"output");  //TPT-17718

	// create api objects to load
	Connection * updConn = new Connection();
	Schema *updSchema = new Schema((char*)"input");  //TPT-17718
	DMLGroup *updDmlGr = new DMLGroup();

	// option file contains connection attributes,
	// table schema definition, dml statements
	// option reader reads all that and add to the api objects
	// note input file select statement is an attribute for export
	
	if (m_om->readOptionsFromFile(expConn, expSchema, NULL, sqlstate, error_message,updatetime) == -100) {
	  delete expConn;
	  delete expSchema;
	  delete updConn;
	  delete updSchema;
	  delete updDmlGr;
	  return;
	}

	expConn->AddAttribute(TD_SYSTEM_OPERATOR,TD_EXPORT);
	expConn->AddAttribute(TD_BUFFER_MODE,(char*)"Yes");  //TPT-17718
	// uncomment to create trace file
	//expConn->AddAttribute(TD_TRACE_OUTPUT,(char*)"/xsptest/e2u_export.txt");  //TPT-17718
        //expConn->AddArrayAttribute(TD_TRACE_LEVEL,2,TD_OPER_ALL,TD_GENERAL,NULL);
	expConn->AddSchema(expSchema);


	m_omload->readOptionsFromFile(updConn, updSchema, updDmlGr, sqlstate, error_message);
	
	updConn->AddAttribute(TD_SYSTEM_OPERATOR,TD_UPDATE);
	updConn->AddAttribute(TD_BUFFER_MODE,(char*)"Yes");  //TPT-17718
	// uncomment to create trace file
	//updConn->AddAttribute(TD_TRACE_OUTPUT,(char*)"/xsptest/e2u_update.txt");  //TPT-17718
	//updConn->AddArrayAttribute(TD_TRACE_LEVEL,2,TD_OPER_ALL,TD_GENERAL);
	updConn->AddSchema(updSchema);
	updConn->AddDMLGroup(updDmlGr,&dmlGroupIndex);
	

	returnValue = updConn->Initiate();

	//cout << "Driver initiated with status " << returnValue << endl;

	if ( returnValue < TD_Error )
	{
	  /**********************************************
	   * Query Buffer Layout
	   **********************************************/
	  queryStatus = updConn->GetEvent(TD_Evt_BufferLayout, &dataptr, &dataLen);
		
	  if ( queryStatus < TD_Error && queryStatus != TD_Unavailable ){
	    layout = *((BufferLayout*)dataptr);
	    //cout << "TD_Evt_BufferLayout: Max Buffer Size: ";
	    //cout << layout.MaxMsgSize << endl;
	    //cout << "TD_Evt_BufferLayout: Row Header Size: ";
	    //cout << layout.RowHeaderSize << endl;
	    //cout << "TD_Evt_BufferLayout: Row Length Size: ";
	    //cout << layout.RowLengthSize << endl;
	    //cout << "TD_Evt_BufferLayout: Buffer Trailer Size: ";
	    //cout << layout.BufTrailerSize << endl;
	  }else {
	    //printStatus("TD_Evt_BufferLayout event failed with status ",queryStatus);
	    debug_trace("TD_Evt_BufferLayout event failed");
	  }
	  
	  /* Give Buffer Layout to Export Driver */
	  expConn->AddAttribute(TD_BUFFER_MAX_SIZE,layout.MaxMsgSize);
	  expConn->AddAttribute(TD_BUFFER_LENGTH_SIZE,layout.RowLengthSize);
	  expConn->AddAttribute(TD_BUFFER_HEADER_SIZE,layout.RowHeaderSize);
	  expConn->AddAttribute(TD_BUFFER_TRAILER_SIZE,layout.BufTrailerSize);


	  returnValue = expConn->Initiate();
	  
	  if ( returnValue < TD_Error )
	    {
		debug_trace("Driver initiated");

		// Acquistion
		while (exportStatus != -1)
		{
		        debug_trace("GetBuffer");
			//returnValue = expConn->GetRow(&data, &dataLen);
			returnValue = expConn->GetBuffer(&data, &dataLen);

			if ( returnValue >= TD_Error )
			{
				//cout << "GetBuffer failed on row " << count+1 << " with status " << returnValue << endl;
				debug_trace("GetBuffer failed");
				exportStatus = -1; //Abort

			}else if ( returnValue == TD_END_Method ){
				debug_trace("End of Data Reached");
				exportStatus = -1; //end 
				count++;

				// End Acquisition
				returnValue = updConn->EndAcquisition();
				debug_trace("End Acquistion");

				if (returnValue < TD_Error) {
				  
				// Application
				  returnValue = updConn->ApplyRows();
				  debug_trace("Apply Rows");
				  
				  if (returnValue < TD_Error) {
				    debug_trace("Update completed successfully");
				  }
				  else {
				    debug_trace("Error occured during Application");
				    updConn->GetErrorInfo(&errorMsg,&errorType);
				    strcpy(sqlstate,"U0005");
				    if ( errorMsg != NULL ){
				      //cout << "Type: " << errorType << endl;
				      if (strlen(errorMsg) > 256) {
					strncpy((char *)error_message,errorMsg,256);
					sqlstate[256] = '\0';
				      }
				      else
					strcpy((char *)error_message,errorMsg);
				      
				    }else{
				      strcpy((char *)error_message,"No Error Info Available");
				    }
				  }
				}
				else {
				  debug_trace("Error occured during EndAcquisition");
				  strcpy(sqlstate,"U0004");
				  updConn->GetErrorInfo(&errorMsg,&errorType);
				  if ( errorMsg != NULL ){
				    //cout << "Type: " << errorType << endl;
				    if (strlen(errorMsg) > 256) {
				      strncpy((char *)error_message,errorMsg,256);
				      sqlstate[256] = '\0';
				    }
				    else
				      strcpy((char *)error_message,errorMsg);
				    
				  }else{
				    strcpy((char *)error_message,"No Error Info Available");
				  }
				}

			}else{
			        debug_trace("PutBuffer");
			        returnValue = updConn->PutBuffer( data, dataLen , 1);
				if (returnValue >= TD_Error)
				  {
				    debug_trace("PutRow failed");
				    exportStatus = -1; // abort
				  }
				else
				  {
				    exportStatus = 0; // continue
				    count++;
				  }
			}

		}

		if (returnValue < TD_Error) {
			debug_trace("Export to Update completed successfully");
		}
		else {
			debug_trace("Error occured during Export");
			strcpy(sqlstate,"U0007");
			expConn->GetErrorInfo(&errorMsg,&errorType);
			if ( errorMsg != NULL ){
			  //cout << "Type: " << errorType << endl;
			  if (strlen(errorMsg) > 256) {
			    strncpy((char *)error_message,errorMsg,256);
			    sqlstate[256] = '\0';
			  }
			  else
			    strcpy((char *)error_message,errorMsg);
			  
			}else{
			  strcpy((char *)error_message,"No Error Info Available");
			}
		}
	    }
	  else
	    {
	      debug_trace("Error occured during Export Initiate");
	      strcpy(sqlstate,"U0012");
	      expConn->GetErrorInfo(&errorMsg,&errorType);
	      if ( errorMsg != NULL ){
		//cout << "Type: " << errorType << endl;
		if (strlen(errorMsg) > 256) {
		  strncpy((char *)error_message,errorMsg,256);
		  sqlstate[256] = '\0';
		}
		else
		  strcpy((char *)error_message,errorMsg);
		
	      }else{
		strcpy((char *)error_message,"No Error Info Available");
	      }
	    }
	}
	else
	{
		debug_trace("Error occured during Update Initiate");
		strcpy(sqlstate,"U0002");
		updConn->GetErrorInfo(&errorMsg,&errorType);
		if ( errorMsg != NULL ){
		  //cout << "Type: " << errorType << endl;
		  if (strlen(errorMsg) > 256) {
		    strncpy((char *)error_message,errorMsg,256);
		    sqlstate[256] = '\0';
		  }
		  else
		    strcpy((char *)error_message,errorMsg);
		  
		}else{
		  strcpy((char *)error_message,"No Error Info Available");
		}
	}


	returnValue = expConn->Terminate();

	//cout << "Driver Terminated with status " << returnValue << endl;

	if ( returnValue >= TD_Error )
	{  
		//Get Error Information
	        debug_trace("Error occured during Export Terminate");
	        strcpy(sqlstate,"U0006");
		expConn->GetErrorInfo(&errorMsg,&errorType);
		if ( errorMsg != NULL ){
		  //cout << "Type: " << errorType << endl;
		  if (strlen(errorMsg) > 256) {
		    strncpy((char *)error_message,errorMsg,256);
		    sqlstate[256] = '\0';
		  }
		  else
		    strcpy((char *)error_message,errorMsg);
		  
		}else{
		  strcpy((char *)error_message,"No Error Info Available");
		}
	}

	returnValue = updConn->Terminate();

	//cout << "Driver Terminated with status " << returnValue << endl;

	if ( returnValue >= TD_Error )
	{  
		//Get Error Information
	        debug_trace("Error occured during Update Terminate");
	        strcpy(sqlstate,"U0006");
		updConn->GetErrorInfo(&errorMsg,&errorType);
		if ( errorMsg != NULL ){
		  //cout << "Type: " << errorType << endl;
		  if (strlen(errorMsg) > 256) {
		    strncpy((char *)error_message,errorMsg,256);
		    sqlstate[256] = '\0';
		  }
		  else
		    strcpy((char *)error_message,errorMsg);
		  
		}else{
		  strcpy((char *)error_message,"No Error Info Available");
		}
	}

	delete expSchema;
	delete expConn;
	delete updDmlGr;
	delete updSchema;
	delete updConn;

	debug_trace("*** Export to Update Complete ***");
}


/**************************************************
 * testExport2Stream
 *
 * Tests transferring data from Export's GetBuffer
 * to Stream's PutBuffer using an XSP. 
 * See the GetBuffer sample for complete step-by-
 * step explanations.
 ***************************************************/
void GenericDriver::testExport2Stream(char *sqlstate, 
				      SQL_TEXT *error_message,
				      VARCHAR_LATIN *updatetime)
{
	int returnValue = 0;
	char * errorMsg = NULL;
	TD_ErrorType errorType;
	TD_Index dmlGroupIndex = 0;

	BufferLayout layout; /* BufferLayout struct defined in GenericDriver.h */
	char *data;
	int exportStatus = 0;
	int streamStatus = 0;
	int queryStatus = 0;
	char *dataptr;
	TD_Length dataLen = 0;
	int count = 0;

	debug_trace("*** Export to Stream Driver Example ***");

	// create api objects to export, no dml group needed 
	Connection *expConn = new Connection();
	Schema *expSchema = new Schema((char*)"output");  //TPT-17718

	// create api objects to load
	Connection * strConn = new Connection();
	Schema *strSchema = new Schema((char*)"input");  //TPT-17718
	DMLGroup *strDmlGr = new DMLGroup();

	// option file contains connection attributes,
	// table schema definition, dml statements
	// option reader reads all that and add to the api objects
	// note input file select statement is an attribute for export
	if (m_om->readOptionsFromFile(expConn, expSchema, NULL, sqlstate, error_message,updatetime) == -100) {
	  delete expConn;
	  delete expSchema;
	  delete strConn;
	  delete strSchema;
	  delete strDmlGr;
	  return;
	}

	expConn->AddAttribute(TD_SYSTEM_OPERATOR,TD_EXPORT);
	expConn->AddAttribute(TD_BUFFER_MODE,(char*)"Yes");  //TPT-17718
	// uncomment to create trace file
	//expConn->AddAttribute(TD_TRACE_OUTPUT,(char*)"/xsptest/e2s_export.txt");  //TPT-17718
        //expConn->AddArrayAttribute(TD_TRACE_LEVEL,2,TD_OPER_ALL,TD_GENERAL,NULL);
	expConn->AddSchema(expSchema);


	m_omload->readOptionsFromFile(strConn, strSchema, strDmlGr, sqlstate, error_message);
	
	strConn->AddAttribute(TD_SYSTEM_OPERATOR,TD_STREAM);
	strConn->AddAttribute(TD_BUFFER_MODE,(char*)"Yes");  //TPT-17718
	// uncomment to create trace file
	//strConn->AddAttribute(TD_TRACE_OUTPUT,(char*)"/xsptest/e2s_stream.txt");  //TPT-17718
	//strConn->AddArrayAttribute(TD_TRACE_LEVEL,2,TD_OPER_ALL,TD_GENERAL);
	strConn->AddSchema(strSchema);
	strConn->AddDMLGroup(strDmlGr,&dmlGroupIndex);
	

	returnValue = strConn->Initiate();

	//cout << "Driver initiated with status " << returnValue << endl;

	if ( returnValue < TD_Error )
	{
	  /**********************************************
	   * Query Buffer Layout
	   **********************************************/
	  queryStatus = strConn->GetEvent(TD_Evt_BufferLayout, &dataptr, &dataLen);
		
	  if ( queryStatus < TD_Error && queryStatus != TD_Unavailable ){
	    layout = *((BufferLayout*)dataptr);
	    //cout << "TD_Evt_BufferLayout: Max Buffer Size: ";
	    //cout << layout.MaxMsgSize << endl;
	    //cout << "TD_Evt_BufferLayout: Row Header Size: ";
	    //cout << layout.RowHeaderSize << endl;
	    //cout << "TD_Evt_BufferLayout: Row Length Size: ";
	    //cout << layout.RowLengthSize << endl;
	    //cout << "TD_Evt_BufferLayout: Buffer Trailer Size: ";
	    //cout << layout.BufTrailerSize << endl;
	  }else {
	    //printStatus("TD_Evt_BufferLayout event failed with status ",queryStatus);
	    debug_trace("TD_Evt_BufferLayout event failed");
	  }
	  
	  /* Give Buffer Layout to Export Driver */
	  expConn->AddAttribute(TD_BUFFER_MAX_SIZE,layout.MaxMsgSize);
	  expConn->AddAttribute(TD_BUFFER_LENGTH_SIZE,layout.RowLengthSize);
	  expConn->AddAttribute(TD_BUFFER_HEADER_SIZE,layout.RowHeaderSize);
	  expConn->AddAttribute(TD_BUFFER_TRAILER_SIZE,layout.BufTrailerSize);


	  returnValue = expConn->Initiate();
	  
	  if ( returnValue < TD_Error )
	    {
		debug_trace("Driver initiated");

		// Acquistion
		while (exportStatus != -1)
		{
		        debug_trace("GetRow");
			//returnValue = expConn->GetRow(&data, &dataLen);
			returnValue = expConn->GetBuffer(&data, &dataLen);

			if ( returnValue >= TD_Error )
			{
				//cout << "GetBuffer failed on row " << count+1 << " with status " << returnValue << endl;
				debug_trace("GetBuffer failed");
				exportStatus = -1; //Abort

			}else if ( returnValue == TD_END_Method ){
				debug_trace("End of Data Reached");
				exportStatus = -1; //end
				count++;
				// End Acquisition
				returnValue = strConn->EndAcquisition();
				debug_trace("End Acquistion");

				if (returnValue < TD_Error) {
				  debug_trace("Stream completed successfully");
				}
				else {
				  debug_trace("Error occured during EndAcquisition");
				  strcpy(sqlstate,"U0004");
				  strConn->GetErrorInfo(&errorMsg,&errorType);
				  if ( errorMsg != NULL ){
				    //cout << "Type: " << errorType << endl;
				    if (strlen(errorMsg) > 256) {
				      strncpy((char *)error_message,errorMsg,256);
				      sqlstate[256] = '\0';
				    }
				    else
				      strcpy((char *)error_message,errorMsg);
				    
				  }else{
				    strcpy((char *)error_message,"No Error Info Available");
				  }
				}

			}else{
			        debug_trace("PutRow");
			        returnValue = strConn->PutBuffer( data, dataLen , 1);
				if (returnValue >= TD_Error)
				  {
				    debug_trace("PutRow failed");
				    exportStatus = -1;// abort
				  }
				else
				  {
				    exportStatus = 0;
				    count++;
				  }
			}

		}

		if (returnValue < TD_Error) {
			debug_trace("Export completed successfully");
		}
		else {
			debug_trace("Error occured during Export");
			strcpy(sqlstate,"U0007");
			expConn->GetErrorInfo(&errorMsg,&errorType);
			if ( errorMsg != NULL ){
			  //cout << "Type: " << errorType << endl;
			  if (strlen(errorMsg) > 256) {
			    strncpy((char *)error_message,errorMsg,256);
			    sqlstate[256] = '\0';
			  }
			  else
			    strcpy((char *)error_message,errorMsg);
			  
			}else{
			  strcpy((char *)error_message,"No Error Info Available");
			}
		}
	    }
	  else
	    {
	      debug_trace("Error occured during Export Initiate");
	      strcpy(sqlstate,"U0012");
	      expConn->GetErrorInfo(&errorMsg,&errorType);
	      if ( errorMsg != NULL ){
		//cout << "Type: " << errorType << endl;
		if (strlen(errorMsg) > 256) {
		  strncpy((char *)error_message,errorMsg,256);
		  sqlstate[256] = '\0';
		}
		else
		  strcpy((char *)error_message,errorMsg);
		
	      }else{
		strcpy((char *)error_message,"No Error Info Available");
	      }
	    }
	}
	else
	{
		debug_trace("Error occured during Stream Initiate");
		strcpy(sqlstate,"U0002");
		strConn->GetErrorInfo(&errorMsg,&errorType);
		if ( errorMsg != NULL ){
		  //cout << "Type: " << errorType << endl;
		  if (strlen(errorMsg) > 256) {
		    strncpy((char *)error_message,errorMsg,256);
		    sqlstate[256] = '\0';
		  }
		  else
		    strcpy((char *)error_message,errorMsg);
		  
		}else{
		  strcpy((char *)error_message,"No Error Info Available");
		}
	}


	returnValue = expConn->Terminate();

	//cout << "Driver Terminated with status " << returnValue << endl;

	if ( returnValue >= TD_Error )
	{  
		//Get Error Information
	        debug_trace("Error occured during Export Terminate");
	        strcpy(sqlstate,"U0006");
		expConn->GetErrorInfo(&errorMsg,&errorType);
		if ( errorMsg != NULL ){
		  //cout << "Type: " << errorType << endl;
		  if (strlen(errorMsg) > 256) {
		    strncpy((char *)error_message,errorMsg,256);
		    sqlstate[256] = '\0';
		  }
		  else
		    strcpy((char *)error_message,errorMsg);
		  
		}else{
		  strcpy((char *)error_message,"No Error Info Available");
		}
	}

	returnValue = strConn->Terminate();

	//cout << "Driver Terminated with status " << returnValue << endl;

	if ( returnValue >= TD_Error )
	{  
		//Get Error Information
	        debug_trace("Error occured during Stream Terminate");
	        strcpy(sqlstate,"U0026");
		strConn->GetErrorInfo(&errorMsg,&errorType);
		if ( errorMsg != NULL ){
		  //cout << "Type: " << errorType << endl;
		  if (strlen(errorMsg) > 256) {
		    strncpy((char *)error_message,errorMsg,256);
		    sqlstate[256] = '\0';
		  }
		  else
		    strcpy((char *)error_message,errorMsg);
		  
		}else{
		  strcpy((char *)error_message,"No Error Info Available");
		}
	}

	delete expSchema;
	delete expConn;
	delete strDmlGr;
	delete strSchema;
	delete strConn;

	debug_trace("*** Export to Stream Complete ***");
}


void GenericDriver::setInput(char *fname)
{
	if (inputFileName != NULL)
	{
		free(inputFileName);
		inputFileName = NULL;
	}

	inputFileName = strdup(fname);
}


void GenericDriver::setOutput(char* fname){
	if( outputFileName != NULL )
	{
		free(outputFileName);
		outputFileName = NULL;
	}

	outputFileName = strdup(fname);
}


/**
* rowBuffer buffer to read the data into, must be big enough
* rowLength set to <= 0 if need to close file, 
*           otherwise doesn't do anything
*/
int GenericDriver::GetRowData(char *rowBuffer, size_t rowLength)
{
	unsigned short rlength;

	//Close File if End Signal Received
	if ( rowLength <= 0 )
	{
		debug_trace("File closing");
		if ( inputStream != NULL ) {
			fclose(inputStream);
			inputStream = NULL;
			debug_trace("File closed");
		}
		return -1;
	}

	//Open File if not already open
	if( inputStream == NULL )
	{
		debug_trace("Opening file");
		inputStream = fopen(inputFileName, "rb");

		if( inputStream == NULL )
		{
			debug_trace("Error opening file ");
			return -1;
		}
	}

	// Read the length of each record..read first two bytes
	size_t recLen = fread(rowBuffer, 1, (size_t)2, inputStream);

	if (recLen < 1) {
		if ( feof(inputStream) ) {
			debug_trace("End of file reached");
			if (inputStream != NULL) {
				fclose(inputStream);
				inputStream = NULL;
			}
			return -1;
		}
		else {
			// return some read error here and exit
			debug_trace("read error");
			if (inputStream != NULL) {
				fclose(inputStream);
				inputStream = NULL;
			}
			return -1;
		}
	}

	// read row of data with size rlength
	rlength = *((unsigned short *)rowBuffer);
	size_t recoLen = fread(rowBuffer+2,1,(size_t)rlength+1,inputStream);

	return 0;
}


// Function puts a row of data in indicator mode in a LOB to be returned
int GenericDriver::WriteRowData(char *data, TD_Length dataLen) {

	if( data != NULL )
	{
		//Open File if not already open
		if( outputStream == NULL )
		{
			outputStream = fopen(outputFileName, "wb");

			if( outputStream == NULL )
			{
				cout << "Error opening file " << outputFileName << endl;
				return -1;
			}
		}

		unsigned long i = 0;
		unsigned short shortdataLen = (unsigned short)dataLen;
		char* rowBuffer = (char*)malloc(dataLen+20);

		memcpy(rowBuffer, &shortdataLen, 2);

		for(i = 0; i < dataLen; i++)
		{
			rowBuffer[i+2] = data[i];
		}
		rowBuffer[i+2] = 0x0a;

		fwrite(rowBuffer, dataLen+3, 1, outputStream);

		free(rowBuffer);

	}else{
		if( outputStream != NULL )
		{
			fclose(outputStream);
		}
		return -1;
	}

	return 0;
}











