/*******************************************
*
* Copyright 2006-2009, 2013-2014 by Teradata Corporation.
* All Rights Reserved.
* TERADATA CONFIDENTIAL AND TRADE SECRET
*
* This copyrighted material is the Confidential, Unpublished
* Property of the Teradata Corporation.  This copyright notice and
* any other copyright notices included in machine readable
* copies must be reproduced on all authorized copies.
*
* Description:
*
* OptionsReader is very different from OptionsManager
* for application sample GenericDriver.
* This class allows reading table schema from file without
* recompilation. For readibility of the code, only
* tables with 8 columns are supported. Look for switch case
* statments for "case 8", and write your own if needed.
* Look for the string 'ssss' where the timestamp token
* is processed.
*
*/

#ifndef __BUILD_APP
#define SQL_TEXT Latin_Text
#include <sqltypes_td.h>

#include <stdlib.h>
#include <string.h>
#endif
#include <errno.h>


#include <stdio.h>

#include "OptionsReader.h"

extern "C" void debug_trace(const char * instring);  //TPT-17718

using namespace teradata::client::API;

OptionsReader::OptionsReader(const char *optionsFileName)
{
	m_optionsFileName = strdup(optionsFileName);
}

OptionsReader::~OptionsReader()
{
	if (m_optionsFileName != NULL) {
		free(m_optionsFileName);
		m_optionsFileName = NULL;
	}
}

int OptionsReader::getRowLength()
{
	return row_length;
}

int OptionsReader::readOptionsFromFile(Connection *conn,
				       Schema *schema,
				       DMLGroup *dmlGr, 
				       char *sqlstate, 
				       unsigned char *error_message, 
				       unsigned char *updatetime)
{

	FILE *optionsFile = NULL;
	char instring[512];
	char *readResult = NULL;
	char *readToken = NULL;
	char lefthandside[512];
	char *righthandside = NULL;
	char *values[128]; 
	unsigned int column_values[3];
	TD_DataType datatype;
	int i;


	optionsFile = fopen(m_optionsFileName, "rb");
	if (optionsFile == NULL) {
		debug_trace("Error opening options file");
		strcpy(sqlstate,"U0010");
		strcat((char *)error_message,strerror(errno));
		return -100;
	}
	else {
		do {
			readResult = NULL;

			readResult = fgets(instring, 512, optionsFile);
			if (readResult == NULL) { // eof or error reached
				if (feof(optionsFile)) {
					//debug_trace("End of options file reached");
					if (optionsFile != NULL) {
						fclose(optionsFile);
						optionsFile = NULL;
					}
					return -1;
				}
				else {
					// return some read error here and exit
					debug_trace("option file read error");
					if (optionsFile != NULL) {
						fclose(optionsFile);
						optionsFile = NULL;
					}
					strcpy(sqlstate,"U0011");
					strcat((char *)error_message,strerror(errno));
					return -100;	
				}
			}

			else { // read in something
				// line cannot have spaces between items. 

				readToken = NULL;
				righthandside = NULL;

				if ((readToken = strtok(instring,",")) != NULL) {
					// special line for dml statement
					if (strcmp(readToken, SAM_DML_STATEMENT) == 0) {
						if (dmlGr != NULL) {
							righthandside = strtok(NULL,"\r\n"); // read everything
							dmlGr->AddStatement(righthandside);
						}

					}
					// row length
					else if (strcmp(readToken, SAM_ROW_LENGTH) == 0 ) {
						if ((righthandside = strtok(NULL,", \r\n")) != NULL) {
							row_length = atoi(righthandside);

						}

					}
					// add each column in same order as appeared in input

					else if (strcmp(readToken, SAM_COLUMN) == 0) {
						// parse columns and add them in order
						i = 0;
						column_values[0] = 0;  // default values
						column_values[1] = 0;
						column_values[2] = 0;
						while ((righthandside = strtok(NULL,", \r\n")) != NULL) {
							switch (i) {
		  case 0:
			  values[i] = strdup(righthandside);
			  break;
		  case 1:
			  datatype = parseDataType(righthandside);
			  break;
		  case 2:
		  case 3:
		  case 4:
			  column_values[i-2] = atoi(righthandside);
			  break;
		  default:
			  break;
							}
							i++;
						}

						// add column to schema

						schema->AddColumn(values[0], datatype, column_values[0],
							column_values[1], column_values[2]);
						if (values[0] != NULL) {
							free(values[0]);
							values[0] = NULL;
						}
						//debug_trace("column added");
					}

					// attributes
					else if (conn != NULL) {
					  
						strcpy(lefthandside, readToken);
						// special case for export select statement
						if (strcmp(lefthandside, SAM_SELECT_STMT) == 0) {
							righthandside = strtok(NULL,"\r\n"); // read everything
							char *updatetokenloc;
							updatetokenloc = strstr(righthandside,"sssssssssssssssssss");
							// this token string is 19 characters in length, to be replaced by an input date string. see input.exp for an example. The string corresponds to a timestamp(0) format. it's certainly possible to extend this to use higher precision by using a longer string
							if (updatetokenloc != NULL && updatetime != NULL && strlen((char *)updatetime) == 19)							
							       strncpy(updatetokenloc,(char *)updatetime,19);
							conn->AddAttribute(TD_SELECT_STMT,righthandside);
						}

						// regular attribute
						else {
							i = 0;
							while ((righthandside = strtok(NULL,", \r\n")) != NULL) {
								values[i++] = strdup(righthandside);
							}

							// put attribute in connection
							if (strcmp(lefthandside, SAM_USER_NAME) == 0) {
								conn->AddAttribute(TD_USER_NAME,values[0]);
							}
							else if (strcmp(lefthandside, SAM_USER_PASSWORD) == 0) {
								conn->AddAttribute(TD_USER_PASSWORD,values[0]);
							}
							else if (strcmp(lefthandside, SAM_TDP_ID) == 0) {
								conn->AddAttribute(TD_TDP_ID,values[0]);
							}
							else if (strcmp(lefthandside, SAM_TARGET_TABLE) == 0) {
								conn->AddAttribute(TD_TARGET_TABLE,values[0]);
							}
							else if (strcmp(lefthandside, SAM_TARGET_TABLE_ARRAY) == 0) {
								switch(atoi(values[0])) {
								  // modify to handle more tables
			case 2:
				conn->AddArrayAttribute(TD_TARGET_TABLE,atoi(values[0]),
					values[1],values[2],NULL);
				break;
								}
							}
							else if (strcmp(lefthandside, SAM_LOG_TABLE) == 0) {
								conn->AddAttribute(TD_LOG_TABLE,values[0]);
							}
							else if (strcmp(lefthandside, SAM_MAX_SESSIONS) == 0) {
								conn->AddAttribute(TD_MAX_SESSIONS,values[0]);
							}
							else if (strcmp(lefthandside, SAM_MIN_SESSIONS) == 0) {
								conn->AddAttribute(TD_MIN_SESSIONS,values[0]);
							}
							else if (strcmp(lefthandside, SAM_ERROR_TABLE) == 0) {
								conn->AddAttribute(TD_ERROR_TABLE,values[0]);
							}
							else if (strcmp(lefthandside, SAM_ERROR_TABLE_1_ARRAY) == 0) {
								switch(atoi(values[0])) {
								  // modify to handle more tables
			case 2:
				conn->AddArrayAttribute(TD_ERROR_TABLE_1,atoi(values[0]),
					values[1],values[2],NULL);
				break;
								}
							}
							else if (strcmp(lefthandside, SAM_ERROR_TABLE_2_ARRAY) == 0) {
								switch(atoi(values[0])) {
								  // modify to handle more tables
			case 2:
				conn->AddArrayAttribute(TD_ERROR_TABLE_2,atoi(values[0]),
					values[1],values[2],NULL);
				break;
								}
							}
							else if (strcmp(lefthandside, SAM_WORK_TABLE_ARRAY) == 0) {
								switch(atoi(values[0])) {
								  // modify to handle more tables
			case 2:
				conn->AddArrayAttribute(TD_WORK_TABLE,atoi(values[0]),
					values[1],values[2],NULL);
				break;
								}
							}
							else if (strcmp(lefthandside, SAM_TENACITY_HOURS) == 0) {
								conn->AddAttribute(TD_TENACITY_HOURS,atoi(values[0]));
							}
							else if (strcmp(lefthandside, SAM_TENACITY_SLEEP) == 0) {
								conn->AddAttribute(TD_TENACITY_SLEEP,atoi(values[0]));
							}
							else if (strcmp(lefthandside, SAM_ROBUST) == 0) {
								conn->AddAttribute(TD_ROBUST,values[0]);
							}
							else if (strcmp(lefthandside, SAM_SERIALIZE_ON) == 0) {
								switch(atoi(values[0])) {
								  // modify to handle different number of columns
			case 8:  
				dmlGr->AddSerializeOn(atoi(values[0]),
					values[1],values[2],values[3],
					values[4],values[5],values[6],
					values[7],values[8],NULL);
				break;
			case 9:  
				dmlGr->AddSerializeOn(atoi(values[0]),
					values[1],values[2],values[3],
					values[4],values[5],values[6],
					values[7],values[8],values[9],NULL);
				break;

								}
							}		  
							else if (strcmp(lefthandside, SAM_USE_LIST) == 0) {
								switch(atoi(values[0])) {
								  // modify to handle different number of columns
			case 8:
				dmlGr->AddUseList(atoi(values[0]),
					values[1],values[2],values[3],
					values[4],values[5],values[6],
					values[7],values[8],NULL);
				break;
                        case 9:
				dmlGr->AddUseList(atoi(values[0]),
					values[1],values[2],values[3],
					values[4],values[5],values[6],
					values[7],values[8],values[9],NULL);
				break;
								}
							}

							while (i > 0) {
								--i;
								if (values[i] != NULL) {
									free(values[i]);
									values[i] = NULL;
								}
							}
						}

					}	    
				}

			}

		} while (readResult != NULL);
	}

	return 0;
}

TD_DataType OptionsReader::parseDataType(const char *datastr) {
	TD_DataType datatype = TD_NONE;
	if (strcmp(datastr, SAM_NONE) == 0)
		datatype = TD_NONE;
	else if (strcmp(datastr, SAM_INTEGER) == 0)
		datatype = TD_INTEGER;
	else if (strcmp(datastr, SAM_SMALLINT) == 0)
		datatype = TD_SMALLINT;
	else if (strcmp(datastr, SAM_FLOAT) == 0)
		datatype = TD_FLOAT;
	else if (strcmp(datastr, SAM_DECIMAL) == 0)
		datatype = TD_DECIMAL;
	else if (strcmp(datastr, SAM_CHAR) == 0)
		datatype = TD_CHAR;
	else if (strcmp(datastr, SAM_BYTEINT) == 0)
		datatype = TD_BYTEINT;
	else if (strcmp(datastr, SAM_VARCHAR) == 0)
		datatype = TD_VARCHAR;
	else if (strcmp(datastr, SAM_LONGVARCHAR) == 0)
		datatype = TD_LONGVARCHAR;
	else if (strcmp(datastr, SAM_BYTE) == 0)
		datatype = TD_BYTE;
	else if (strcmp(datastr, SAM_VARBYTE) == 0)
		datatype = TD_VARBYTE;
	else if (strcmp(datastr, SAM_DATE) == 0)
		datatype = TD_DATE;
	else if (strcmp(datastr, SAM_GRAPHIC) == 0)
		datatype = TD_GRAPHIC;
	else if (strcmp(datastr, SAM_VARGRAPHIC) == 0)
		datatype = TD_VARGRAPHIC;
	else if (strcmp(datastr, SAM_LONGVARGRAPHIC) == 0)
		datatype = TD_LONGVARGRAPHIC;
	else if (strcmp(datastr, SAM_DATE_ANSI) == 0)
		datatype = TD_DATE_ANSI;
	else if (strcmp(datastr, SAM_BIGINT) == 0)
		datatype = TD_BIGINT;
	return datatype;
}







