/*******************************************
*
* Copyright 2006-2009, 2014 by Teradata Corporation.
* All Rights Reserved.
* TERADATA CONFIDENTIAL AND TRADE SECRET
*
* This copyrighted material is the Confidential, Unpublished
* Property of the Teradata Corporation.  This copyright notice and
* any other copyright notices included in machine readable
* copies must be reproduced on all authorized copies.
*
* Description:
*  
* This file is the entry point for the External Stored 
* Procedure. It is capable to launch the four operators
* supported by Teradata PT API - load, update, stream, and
* export. There are three additional operations - export to load,
* export to update and export to stream that demostrates
* moving data from one table to another.
* See GenericDriver.cpp for detailed logic. 
*
* Installation:
*
* 1. Compile this sample using the included makefile/project
* 2. Send this command to the DBS
*    CREATE PROCEDURE xsptest
*    (IN operatortype VARCHAR(64),
*     IN configfilename VARCHAR(64),
*     IN extraconfigfilename VARCHAR(64),
*     IN datafilename VARCHAR(64),
*     IN updatetime VARCHAR(64))
*    LANGUAGE CPP
*    MODIFIES SQL DATA
*    EXTERNAL NAME 'SP!CLI!SP!/xsptest/xsp.so!F!xsptest'
*    PARAMETER STYLE SQL;
* 3. Use the following 7 commands for different operators
*    Load from file to DBS:
*      CALL xsptest('TD_LOAD', '/xsptest/input.lod', null, 
*        '/xsptest/infile.littleEndian', null);
*    Update from file to DBS:
*      CALL xsptest('TD_UPDATE', '/xsptest/input.upd', null, 
*        '/xsptest/infile.littleEndian', null);
*    Stream from file to DBS:
*      CALL xsptest('TD_STREAM', '/xsptest/input.stm', null, 
*        '/xsptest/infile.littleEndian', null);
*    Export from DBS to file:
*      CALL xsptest('TD_EXPORT', '/xsptest/input.exp', null, 
*        '/xsptest/outfile.littleEndian', null);
*    Load all data from one DBS to another:
*      CALL xsptest('TD_EXPORT2LOAD', '/xsptest/input.exp', 
*        '/xsptest/input.lod', null, null);   
*    Update from one DBS to another:
*      CALL xsptest('TD_EXPORT2UPDATE', '/xsptest/input.exp2', 
*        '/xsptest/input.lod', null, '2007-03-03 10:30:00');
*      (note this sample uses the same config file as load)    
*    Stream from one DBS to another:
*      CALL xsptest('TD_EXPORT2STREAM', '/xsptest/input.exp2', 
*        '/xsptest/input.stm', null, '2007-03-03 10:30:00');
*
*/

#ifndef __BUILD_APP
#define SQL_TEXT Latin_Text
#include <sqltypes_td.h>

#include <stdlib.h>
#include <string.h>
#endif

#include <stdio.h>

#include "OptionsReader.h"
#include "connection.h"
#include "GenericDriver.h"

#include <errno.h>

using namespace teradata::client::API;


extern "C"
void debug_trace(char * instring);

/**
 * Entry point of XSP
 * operatortype - one of TD_LOAD, TD_UPDATE, TD_STREAM, TD_EXPORT,
 *                TD_EXPORT2LOAD, TD_EXPORT2UPDATE, TD_EXPORT2STREAM
 * configfilename - location of the config file for this XSP. For
 *                  EXPORT2X operations, this is for the Export part
 * extraconfigname - config file for the Load/Update/Stream part 
 *                   for EXPORT2X operations
 * datafilename - location of input file for load/update/stream or
 *                output file for export
 * updatetime - timestamp string in the form of "YYYY-MM-DD hh:mi:ss"
 *              to be used with an export config file where the
 *              SELECT statement contains a WHERE clause with a 19-
 *              character long string of 'sssssssssssssssssss'. The
 *              'ssss' string is replaced with updatetime. See 
 *              OptionsReader for details.
 */
#ifndef __BUILD_APP
extern "C"
#ifdef WIN32
__declspec(dllexport)
#endif
void xsptest( VARCHAR_LATIN *operatortype,
	      VARCHAR_LATIN *configfilename,
	      VARCHAR_LATIN *extraconfigfilename,
	      VARCHAR_LATIN *datafilename, 
	      VARCHAR_LATIN    *updatetime,   
 	      int           *operatortypeIsNull,
              int           *confignameIsNull,
	      int           *extraconfignameIsNull,
              int           *datafilenameIsNull,
	      int           *updatetimeIsNull,
	      char           sqlstate[6],  // to output any exception state
              SQL_TEXT       extname[129],
              SQL_TEXT       specific_name[129],
              SQL_TEXT       error_message[257])
#else  
void xsptest( char *operatortype,    // to compile as a command line program
	      char *configfilename,  // for troubleshooting xsp 
	      char *extraconfigfilename,
	      char *datafilename, 
		  char *updatetime,
	      char sqlstate[6],
	      unsigned char error_message[257])
#endif
{  
	char tmp_string[65];
	char configname[65];
	char confignameextra[65];
	size_t string_len;	

	string_len = strlen((const char *)operatortype);
	if (string_len > 4)
	{
		strcpy(tmp_string, (char *)operatortype);
	}

	if (*confignameIsNull != -1)
	        strcpy(configname, (char *)configfilename);
	if (*extraconfignameIsNull != -1)
	        strcpy(confignameextra, (char *)extraconfigfilename);
	strcpy((char *)error_message,configname);


	GenericDriver *gd;
	OptionsReader *aor = new OptionsReader(configname);
	OptionsReader *aorextra = NULL;
	if (strcmp(tmp_string, "TD_EXPORT2LOAD") != 0 &&
	    strcmp(tmp_string, "TD_EXPORT2UPDATE") != 0 &&
	    strcmp(tmp_string, "TD_EXPORT2STREAM") != 0) {
	        gd = new GenericDriver(aor);
	}
	else {
	        // EXPORT2X operations takes two config files
	        aorextra = new OptionsReader(confignameextra);
	        gd = new GenericDriver(aor,aorextra);
	}

	gd->setInput((char *)datafilename);
	gd->setOutput((char *)datafilename);

	if (strcmp(tmp_string, "TD_LOAD") == 0) {
	        gd->testLoad(sqlstate,error_message);
	}
	else if (strcmp(tmp_string, "TD_UPDATE") == 0) {
		gd->testUpdate(sqlstate,error_message);
	}
	else if (strcmp(tmp_string, "TD_EXPORT") == 0) {
		gd->testExport(sqlstate,error_message);
	}
	else if (strcmp(tmp_string, "TD_STREAM") == 0) {
		gd->testStream(sqlstate,error_message);
	}
	else if (strcmp(tmp_string, "TD_EXPORT2LOAD") == 0) {
	  if (*updatetimeIsNull == -1)
		gd->testExport2Load(sqlstate,error_message,NULL);
	  else
	        gd->testExport2Load(sqlstate,error_message,updatetime);
	}
	else if (strcmp(tmp_string, "TD_EXPORT2UPDATE") == 0) {
	  if (*updatetimeIsNull == -1)
	        gd->testExport2Update(sqlstate,error_message,NULL);
	  else
		gd->testExport2Update(sqlstate,error_message,updatetime);
	}
	else if (strcmp(tmp_string, "TD_EXPORT2STREAM") == 0) {
	  if (*updatetimeIsNull == -1)
		gd->testExport2Stream(sqlstate,error_message,NULL);
	  else
         	gd->testExport2Stream(sqlstate,error_message,updatetime);
	}


	delete aor;
	delete gd;
	if (aorextra != NULL)
	  delete aorextra;
}


extern "C"
void debug_trace(char * instring)
{
#ifndef __BUILD_APP
    
        char * argv[1];
	int length[1];

	argv[0] = instring;
	length[0] = (int)strlen(instring);
	FNC_Trace_Write_DL(1, (void **)argv, length);
	
#else
	cout << instring << endl;  // print to screen for command line program
#endif
}


#ifdef __BUILD_APP
int main(int argc, char *argv[])
{
  char sqlstate[6];
  unsigned char error_message[257];

  if (argc != 6) {
    debug_trace("Required: <operator> <config file> <load config file> <data file> <update time>");
    return -1;
  }
  xsptest(argv[1], argv[2], argv[3], argv[4], argv[5], sqlstate, error_message);
  cout << sqlstate << endl;
  cout << error_message << endl;
  return 0;
}
#endif








